
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2013-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Peter Chung <peterc@linuxmagic.com>
 *
 * $Id: form-submit.js 11290 2020-03-27 16:26:44Z deiony $
 */

// initialize the input timer to filter exemptions
if (!MS) {
    var MS = {};
}
MS.inputTimer;

//! Filter table entries according to the users input
/*!
 * @param   string formId       the id of the table to operate on
 * @param   string trigger      the element type which triggered the filter
 *
 * @author  Lucas Nincao <lucasn@linuxmagic.com>
 *          Deion Yu <deiony@linuxmagic.com>
 */
function filterTable(formId, trigger){
    // ensure that the table have rows
    var table = document.querySelector('#' + formId + ' .entrylist_table .rowTable');
    if (!table) {
        return;
    }

    // retrieve the filter information
    var filter = document.querySelector('#' + formId + ' .filterInput');
    var filterType = document.querySelector('#' + formId + ' .searchType');
    var filterStatus = document.querySelector('#' + formId + ' .searchStatus');
    var filterValue = filter.value.toUpperCase();

    // no need to filter when changing filter type without any filter value
    if (trigger == 'select' && filterValue == '') {
        return;
    }

    // reset search input timer whenever an input event is triggered
    clearTimeout(MS.inputTimer);

    // filter the table only after the user has stopped entering input for 250 ms (1/4 second)
    MS.inputTimer = setTimeout(function() {
        // set visual indicator for the progress of the filter
        filterStatus.className = "searchStatusSearching";

        // disable input until the filter is complete
        filter.disabled = true;

        // retrieve the rows and filter
        var rows = table.children;
        var rowcolor = 'oddRow';

        // uncheck the select all checkbox
        document.querySelector('#'+ formId +' .fancyHeaderBox input[type=checkbox]').checked = false;

        // go through each row in the table
        for (var i = 0; i < rows.length; i++) {
            // uncheck the row checkbox
            rows[i].querySelector('input[type=checkbox]').checked = false;

            // retrieve the entry for the row based on the filter type
            var entryClass;
            switch (filterType.value) {
            case 'address':
                entryClass = 'entryLeft';
                break;
            case 'comment':
                entryClass = 'entryRight';
                break;
            case 'both':
            default:
                entryClass = 'entryPadding';
            }
            var entry = rows[i].querySelector('div[class=' + entryClass + ']');

            // filter the row based on the entry
            if (entry.innerText.toUpperCase().indexOf(filterValue) > -1) {
                // match, show the row
                rows[i].style.display = "block";
                // alternate the row color
                rows[i].className = rowcolor;
                rowcolor = (rowcolor == 'oddRow') ? 'evenRow' : 'oddRow';
            } else {
                // no match, hide the row
                rows[i].style.display = "none";
            }
        }

        // set visual indicator for the progress of the filter
        filterStatus.className = 'searchStatus';

        // re-enable the input now that the filter is complete
        filter.disabled = false;

        // ensure that focus remains on the filter input
        // NOTE: IE drops focus on the input after filtering which leads to
        //       nasty outcomes within the MailEnable Admin Console
        filter.focus();
    }, 250);
}

//! Sort table entries alphabetically (ascending order)
/*!
 * @param   string formId       the id of the table to operate on
 * @param   string type         the type of the table
 *
 * @author  Lucas Nincao <lucasn@linuxmagic.com>
 *          Deion Yu <deiony@linuxmagic.com>
 */
function sortList(formId, type) {
    // ensure that the table have rows
    var table = document.querySelector('#' + formId + ' .entrylist_table .rowTable');
    if (!table) {
        return;
    }

    var sortBy = document.querySelector('#'+ formId +' .sortDrop').value;
    var rows = table.children;
    var rowcolor = 'oddRow';

    // convert rows into an array
    rows = Array.prototype.slice.call(rows, 0);

    // sort array (either by name or id)
    if (sortBy == "name") {
        if (type == "ip") {
            rows.sort(function(a, b) {
                // retrieve the ip string
                var ipStr1 = a.innerText.trim();
                var ipStr2 = b.innerText.trim();

                // check whether dealing with ipv6
                var is_ipv6_1 = (ipStr1.indexOf(':') > -1);
                var is_ipv6_2 = (ipStr2.indexOf(':') > -1);

                // special handling when ipv6
                if (is_ipv6_1 && is_ipv6_2) {
                    return ipStr1.localeCompare(ipStr2);
                } else if (is_ipv6_1) {
                    return 1;
                } else if (is_ipv6_2) {
                    return -1;
                }

                // split ip string into ip array
                // NOTE: ip array will have 2 elements when dealing with ip ranges
                //       otherwise there will always be at least 1 element
                var ipArr1 = ipStr1.split('-');
                var ipArr2 = ipStr2.split('-');

                // retrieve the left ip
                var ipLeft1 = ipArr1[0].trim().split('.');
                var ipLeft2 = ipArr2[0].trim().split('.');

                // convert left ip into a long
                var ipLeftLong1 = ((((((+ipLeft1[0])*256)+(+ipLeft1[1]))*256)+(+ipLeft1[2]))*256)+(+ipLeft1[3]);
                var ipLeftLong2 = ((((((+ipLeft2[0])*256)+(+ipLeft2[1]))*256)+(+ipLeft2[2]))*256)+(+ipLeft2[3]);

                // compare the left ip long
                if (ipLeftLong1 != ipLeftLong2) {
                    return ipLeftLong1 - ipLeftLong2;
                }

                // no differences with left ip long
                if (ipArr1.length == 2 && ipArr2.length == 2) {
                    // retrieve the right ip
                    var ipRight1 = ipArr1[1].trim().split('.');
                    var ipRight2 = ipArr2[1].trim().split('.');

                    // convert right ip into a long
                    var ipRightLong1 = ((((((+ipRight1[0])*256)+(+ipRight1[1]))*256)+(+ipRight1[2]))*256)+(+ipRight1[3]);
                    var ipRightLong2 = ((((((+ipRight2[0])*256)+(+ipRight2[1]))*256)+(+ipRight2[2]))*256)+(+ipRight2[3]);

                    // compare the right ip long
                    return ipRightLong1 - ipRightLong2;
                } else if (ipArr1.length == 2) {
                    return 1;
                } else if (ipArr2.length == 2) {
                    return -1;
                }

                return 0;
            });
        } else {
            rows.sort(function(a, b) {
                var str1 = a.innerText.trim();
                var str2 = b.innerText.trim();
                return str1.localeCompare(str2);
            });
        }
    } else {
        // sort by data-orderid (order added)
        rows.sort(function(a, b) {
            var order1 = a.getAttribute("data-orderid");
            var order2 = b.getAttribute("data-orderid");
            return order1 - order2;
        });
    }

    // re-add sorted elements to the table
    for (var i = 0; i < rows.length; i++) {
        // append row to table
        row = table.appendChild(rows[i]);
        if (row.style.display != "none") {
            // alternate the row color
            rows[i].className = rowcolor;
            rowcolor = (rowcolor == 'oddRow') ? 'evenRow' : 'oddRow';
        }
    }
}



//! stores the values in params into the form object.
/*!
 * @param string various entries in the form
 *
 * @return none
 *
 * @author Anonymous
 *
 * TODO the error handling has been commented out.
 */
function doFormSubmit(params) {
    var form = document.createElement("form");
    if (params["form"] != undefined) {
        form.setAttribute("name", params["form"]);
        form.setAttribute("id", params["form"]);
    }

    if (params["method"] != undefined) {
        form.setAttribute("method", params["method"]);
    } else {
        form.setAttribute("method", "POST");
    }
    if (params["action"] != undefined) {
        form.setAttribute("action", params["action"]);
    } else {
        form.setAttribute("action", "");
    }

    if (params["hiddenFields"] != undefined) {
        for (var key in params["hiddenFields"]) {
            if (key != 'action') {
                var hiddenField = document.createElement("input");
                hiddenField.setAttribute("type", "hidden");
                hiddenField.setAttribute("name", key);
                hiddenField.setAttribute("value", params["hiddenFields"][key]);
                form.appendChild(hiddenField);
            }
        }
    } else {
        alert("no hidden fields");
    }

    if (document.body.className) {
        document.body.className += ' cursorWait';
    } else {
        document.body.className = 'cursorWait';
    }

    document.body.appendChild(form);
    form.submit();
    document.body.removeChild(form);
}

//! Function to direct the user to the system-settings page
/*!
 * @return nothing
 *
 * @author Kyle Macdonald
 *
 * XXX could very easily be modified to have 'module' and 'page' be
 * parameters to be applicable to all 2nd tier pages
 */
function gotoSystemSettings() {

    var module = 'settings';
    var page = 'system_settings';

    var params = {
        'hiddenFields': {
            'module': module, 'page': page
        }
    };

    doFormSubmit(params)
}

//! registers all the events required to handle exemption comments
function registerExemptionCommentEvents() {
    // the exemption comment modal, don't bother registering if exemption comments isn't there
    modal = document.getElementById("exemptionCommentModal");
    if (!modal) {
        return;
    }

    // the exemption comment HTML elements contained in the modal
    title = document.getElementById("exemptionCommentTitle");
    entry = document.getElementById("exemptionCommentEntry");
    input = document.getElementById("exemptionCommentInput");
    remainder = document.getElementById("exemptionCommentRemainder");
    saveButton = document.getElementById("exemptionCommentSaveButton");
    deleteButton = document.getElementById("exemptionCommentDeleteButton");
    closeButton = document.getElementById("exemptionCommentCloseButton");

    // keep track of the selected entry
    target = null;

    // helper HTML functions
    // NOTE: not perfect but better than nothing
    function encodeHTML(string) {
        string = string.replace(/&/g, "\\&");
        string = string.replace(/"/g, "&quot;");
        string = string.replace(/'/g, "&#039;");
        string = string.replace(/</g, "&lt;");
        string = string.replace(/>/g, "&gt;");
        string = string.replace(/\\&/g, "&amp;");
        return string;
    }
    function decodeHTML(string) {
        string = string.replace(/&quot;/g, "\"");
        string = string.replace(/&#039;/g, "'");
        string = string.replace(/&lt;/g, "<");
        string = string.replace(/&gt;/g, ">");
        string = string.replace(/&amp;/g, "&");
        return string;
    }
    // helper AJAX functions
    // Callback function when AJAX to set exemption comment request succeeds
    function exemptionCommentCallback(res) {
        // hide the loading element
        hideLoading();

        // sanity check
        if (!res || !res.response || res.response == '') {
            return;
        }

        // parse response into JSON format
        try {
            response = JSON.parse(res.response);
        } catch (e) {
            MS.buildNotice(MS.intl("entry_comment_json_fail"), "errors");
            return;
        }

        // display error message if bad status
        if (response.status === false) {
            MS.buildNotice(response.message, "errors");
            return;
        }

        // update the comment
        target.innerHTML = encodeHTML(response.data.comment);

        // hide the modal
        modal.style.display = "none";

        // display the sucess message
        MS.buildNotice(response.message, "notices");
    }
    //! Callback function when AJAX to set exemption comment request fails
    function exemptionCommentErrorCallback(error) {
        // hide the loading element
        hideLoading();

        // display the error message
        MS.buildNotice(error, "errors");
    }
    //! Callback function when AJAX to set exemption comment request fails
    function exemptionCommentAJAX(action) {
        // clear any previous notice or error message
        MS.clearNoticesAndErrors();

        // display the loading element
        showLoading();

        // retrieve the form for the selected entry
        form = target.closest('form');

        // initialize comment based on action
        comment = input.value;
        if (action == "delete") {
            comment = "";
        } else {
            // only single spaces are allowed in comment
            comment = comment.replace(/\s+/g, " ");
        }

        // set up POST data
        data = {
            "output_only": 1,
            "module": form.module.value,
            "submodule": form.submodule.value,
            "page": form.page.value,
            "ms_action": "comment",
            "entry": target.previousSibling.innerHTML,
            "comment": comment.trim(),
        };

        // build the POST data string
        datastring = Object.keys(data).reduce(function(accumulator, key, index) {
            if (index) {
                accumulator = accumulator + "&";
            }
            return accumulator + key + "=" + encodeURIComponent(data[key]);
        }, "");

        // retrieve URL for submitting the AJAX request
        // XXX: little hack for the DirectAdmin URL
        url = window.location.pathname;
        if (url.indexOf('CMD_PLUGINS') === 1 || url.indexOf('CMD_PLUGINS_ADMIN') === 1) {
            url += '/index.raw';
        }

        // send AJAX request
        ajaxRequest(url, exemptionCommentCallback, datastring, exemptionCommentErrorCallback);
    }

    // display the exemption comment
    comments = document.querySelectorAll(".comment");
    for (i = 0; i < comments.length; ++i) {
        comments[i].onclick = function(e) {
            // retrieve the target element of the event
            node = e.target.previousSibling.querySelector(".entryRight");

            // retrieve the list name for the entry
            exemption = node.closest(".collapsibleModule").previousElementSibling;
            list = exemption.querySelector('.collapseTitle').innerHTML.replace(/ \(\d+\)$/gi, '');

            // set the exemption comment
            title.innerHTML = MS.intl("comment_for") + " " + list;
            entry.innerHTML = node.previousSibling.innerHTML;
            input.value = decodeHTML(node.innerHTML);
            remainder.innerHTML = MS.intl("remaining_characters") + ": " + (255 - node.innerHTML.length);

            // set the seleceted entry
            target = node;

            // display the exemption comment
            modal.style.display = "block";

            // set focus to the input textarea
            input.focus();
        }
    }

    // hide the exemption comment
    closeButton.onclick = function(e) {
        modal.style.display = "none";
    }

    // delete the exemption comment
    deleteButton.onclick = function(e) {
        // don't make delete AJAX request when comment was initially empty
        // XXX: might want to use a different system to notify customer
        if (target.innerHTML === "") {
            MS.buildNotice(MS.intl("entry_comment_nothing_to_delete"), "errors");
            return;
        }
        exemptionCommentAJAX("delete");
    }

    // save the exemption comment
    saveButton.onclick = function(e) {
        // don't make save AJAX request when comment is empty
        // XXX: might want to use a different system to notify customer
        if (input.value.trim() === "") {
            MS.buildNotice(MS.intl("entry_comment_nothing_to_save"), "errors");
            return;
        }
        exemptionCommentAJAX("save");
    }

    // ignore enter key when inputting comment
    input.onkeypress = function(e) {
        key = e ? e.which : window.event.keyCode;
        if (key == 13) {
            return false;
        }
    };

    // update remaining character when inputting comment
    input.oninput = function(e) {
        remainder.innerHTML = remainder.innerHTML.replace(/\d+$/gi, 255 - input.value.length);
    }
}

//! registers events required to search exemptions
function registerExemptionSearchEvents() {
    // retrieve all the filter input for all exemptions
    var filterInputs = document.querySelectorAll('.filterInput');

    // disable enter key to prevent submitting the form when filtering
    for (var i = 0; i < filterInputs.length; i++) {
        filterInputs[i].onkeypress = function(e) {
            key = e ? e.which : window.event.keyCode;
            if (key == 13) {
                e.preventDefault();
                return false;
            }
        };
    }
}

MS.addDocumentOnLoad(registerExemptionCommentEvents);
MS.addDocumentOnLoad(registerExemptionSearchEvents);
