/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): William Storey <william@linuxmagic.com>
 *
 * $Id: help-popup.js 5517 2016-05-06 21:54:29Z shaun $
 *
 * functionality to work with help/informational popup widgets.
 *
 * this is based upon that found in month-stats.js originally.
 */

"use strict";

// a global object to contain our functions and data - to encapsulate
// everything so we don't pollute the global scope.
if (!MS) {
    var MS = {};
}

//! generate a unique id for a pop-up element based on its content i18n key.
/*!
 * @param string key The i18n key to use for the pop-up contents.
 *
 * @return mixed string A unique id. This is to act as a hash type value. One
 *   key will always return the same unique id.
 *   If you provide an invalid parameter then we will return undefined.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * we want to give each pop-up a unique "id" attribute. why? because we want
 * to know if the pop-up is already present on the page for a specific help.
 * we don't want to show multiple of the same pop-up at once!
 */
MS.getHelpPopupId = function(key) {
    var prefix = "msPopupId";
    if (typeof key !== 'string' || key.length === 0) {
        return undefined;
    }
    return prefix + key;
};

//! try to find the help link icon element.
/*!
 * @param Element root The root element to begin searching for.
 *   We descend the children looking for an element with 'helplink'
 *   class.
 *
 * @return mixed Element, one with 'helplink' class, or undefined if
 *   we can't find it.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * we descend all children looking for one with the 'helplink' class.
 * this is currently the class given to the icon. we want to use this
 * for positioning purposes.
 */
MS.findHelpIcon = function(root) {
    if (!root) {
        return undefined;
    }

    for (var i = 0; i < root.children.length; i++) {
        var ele = root.children[i];
        // check this element.
        var classes = ele.className.split(' ');
        MS.log("findHelpIcon: " + i + " class " + ele.className);
        for (var j = 0; j < classes.length; j++) {
            if (classes[j] === 'helplink') {
                return ele;
            }
        }
        // check its children for the help icon.
        var help = this.findHelpIcon(ele);
        if (help) {
            return help;
        }
    }
    return undefined;
};

//! hide a popup if it's displayed.
/*!
 * @param string bodyI18n The i18n key to use for the pop-up contents.
 *   NOTE: This will be used to create a unique "id" for popup so.
 *
 * @returns bool True if the popup was removed. False if not (it may
 *   not be present, for one).
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * look for the popup by its unique id.
 * if it's present, remove it from the document.
 */
MS.hideHelpPopup = function(bodyI18n) {
    if (bodyI18n === undefined || typeof bodyI18n !== 'string'
        || bodyI18n.length === 0) {
        MS.log("hideHelpPopup: invalid parameter");
        return false;
    }
    // find its unique id.
    var popupId = MS.getHelpPopupId(bodyI18n);
    if (popupId === undefined) {
        MS.log("hideHelpPopup: failed to find popup id");
        return false;
    }
    var popup = document.getElementById(popupId);
    if (!popup) {
        MS.log("hideHelpPopup: popup not found");
        return false;
    }
    if (!popup.parentNode) {
        MS.log("hideHelpPopup: popup parent not found");
        return false;
    }
    popup.parentNode.removeChild(popup);
    return true;
};

//! generate position for the popup.
/*!
 * @param Element popup The popup
 * @param Element caller The popup triggering caller
 * @param mixed position Desired positioning. See showHelpPopup() for
 *   more information on this parameter.
 *
 * @return mixed Object with properties x and y which are values in pixels
 *   for the placement of the popup, or undefined if failure.
 *
 * @author William Storey <william@linuxmagic.com>
 *
 * @see showHelpPopup().
 *
 * we try to position the popup intelligently.
 */
MS._positionHelpPopup = function(popup, caller, position) {
    // position is optional.
    if (!popup || !caller) {
        MS.log("_positionHelpPopup: Invalid parameter");
        return undefined;
    }
    // position is optional. default to top-right.
    if (position === undefined || typeof position !== 'string'
        || position.length === 0) {
        position = 'top-right';
    }
    if (position !== 'top-right' && position !== 'top-left'
        && position !== 'left') {
        MS.log("_positionHelpPopup: invalid positioning");
        return undefined;
    }

    // retrieve information we can use for positioning.
    var popupRect = popup.getBoundingClientRect();

    // the caller reference we want to position relative to is not the
    // <a> element as that top/right/bottom/left is apparently invisibly
    // not what we might expect. instead, use the help icon.
    // TODO: this is probably not very maintainable ... depends on us
    //   being attached to <a> element and having an element with class
    //   helpLink that is our child!
    var foundCaller = MS.findHelpIcon(caller);
    if (!foundCaller) {
        // even if we don't find it, try to position based on the
        // element we were given initially.
        MS.log("_positionHelpPopup: no caller found");
    } else {
        caller = foundCaller;
    }
    var callerRect = caller.getBoundingClientRect();

    // we need heights and width to help position it.
    var popupHeight = popupRect.bottom - popupRect.top;
    var popupWidth = popupRect.right - popupRect.left;
    var callerHeight = callerRect.bottom - callerRect.top;

    var realTop = 0;
    var realLeft = 0;

    // check for older browsers window scroll position and set variables to be cross-compatible
    // extracted from mozilla.org
    // https://developer.mozilla.org/en-US/docs/Web/API/window.scrollY
    var xWindowScroll = (window.pageXOffset !== undefined) ? window.pageXOffset
                        : (document.documentElement || document.body.parentNode || document.body).scrollLeft;
    var yWindowScroll = (window.pageYOffset !== undefined) ? window.pageYOffset
                        : (document.documentElement || document.body.parentNode || document.body).scrollTop;

    if (position === 'top-right') {
        // top position: the bottom of the icon, raised up the height of the
        //   icon, and raised up again the height of the popup.
        realTop = yWindowScroll + callerRect.bottom - callerHeight - popupHeight;
        // left position: the right side of the icon, shifted over by the width
        //   of the popup.
        realLeft = xWindowScroll + callerRect.right;
        if ( (realLeft + popupWidth) > 1000) {
            realLeft -= popupWidth;
        }
    }
    if (position === 'top-left') {
        // top position is the same as for top-right position.
        realTop = yWindowScroll + callerRect.bottom - callerHeight
            - popupHeight;
        // left position: left side of the icon.
        realLeft = xWindowScroll + callerRect.left - popupWidth;
    }
    if (position === 'left') {
        realTop = yWindowScroll + callerRect.top;
        // shift up a little bit - not exactly matching top corners, but close.
        realTop -= 10;
        // left position: left side of the icon.
        realLeft = xWindowScroll + callerRect.left - popupWidth;
        // give a little bit of buffer space.
        realLeft -= 3;
    }

    // we try to account for scrolling issues (if the popup would display
    // in an area we've scrolled past).
    // if the top of the popup is scrolled out, then we flip and position
    // below our triggering element.
    if (realTop < yWindowScroll) {
        realTop = yWindowScroll + callerRect.bottom;
    }
    // if left edge of the popup is scrolled out, then we flip and position
    // to the right of triggering element
    if (realLeft < xWindowScroll) {
        // position at the left of what we clicked on if it's visible. If
        // it's not, position at our leftmost. Note callerRect.left on its
        // own does not account for scrolling. hence adding window.scrollX
        // to it
        if (callerRect.left + xWindowScroll < xWindowScroll) {
            realLeft = xWindowScroll;
        } else {
            realLeft = callerRect.left + xWindowScroll;
        }
    }
    // account for Panel top padding / overlap
    if (realTop < 50) {
        realTop = 50;
    }
    return {'x': realLeft, 'y': realTop};
};

//! function to show a help pop-up/overlay.
/*!
 * @param Element caller The element clicked to show the pop-up/overlay.
 *   Typically this will be through an onclick event.
 *   This will not be modified.
 * @param string closeI18n The i18n key to show for the close label.
 * @param string bodyI18n The i18n key to use for the pop-up contents.
 *   NOTE: This will be used to create a unique "id" for popup so.
 * @param int x The optional horizontal 'X' coordinate to define a location.
 * @param int y The optional vertical 'Y' coordinate to define a location.
 * @param mixed position The position to place the popup.
 *   This may be left unspecified and we will default to placing the popup
 *   at the top right corner of the caller.
 *   Values:
 *     top-right: Position the popup so that its bottom right edge is touching
 *       the top right of the caller.
 *     top-left: Position the popup so that its bottom right edge is touching
 *       the top left of the caller.
 *     left: Position the popup so that its top right is touching the top
 *       left of the caller.
 *   Note all positioning attempts to take into account the space for the
 *   popup. If there is no space (such as if the element will not fit on
 *   the screen) then the popup may be positioned in a different way (such
 *   as to flip it under the element).
 *   As well this only takes effect if you have not specified x or y.
 *
 * @returns void
 *
 * @author Shaun A. Johnson <shaun@linuxmagic.com>
 * @author William Storey <william@linuxmagic.com>
 *
 * @pre Translations for the two i18n keys we use are available as properties
 *   on the MS.i18n object.
 *
 * this is intended to be a re-usable widget helper for multiple page use.
 *
 * we create a pop-up element, position it, and display it.
 *
 * we use the popup's unique id (based on the content's i18n key) to ensure
 * that only one popup is unique on the page.
 */
MS.showHelpPopup = function(caller, closeI18n, bodyI18n, x, y, position) {
    if (!caller || typeof caller !== 'object'
        || typeof closeI18n !== 'string' || closeI18n.length === 0
        || typeof bodyI18n !== 'string' || bodyI18n.length === 0) {
        return;
    }

    // a popup has a unique id. we use this so we can tell if the popup is
    // already present in the document so that we don't create duplicate
    // popups.
    var popupId = this.getHelpPopupId(bodyI18n);
    if (!popupId) {
        return;
    }
    // check if the popup is in the document already - if it is, we may
    // remove it so the user can remove it by clicking same icon
    var popup = document.getElementById(popupId);
    if (popup) {
        popup.parentNode.removeChild(popup);
        return;
    }

    // create the popup.
    popup = document.createElement('div');
    popup.className = 'popupContainer';
    popup.setAttribute('id', popupId);

    // <span><a></a></span> for the close button.
    var closeSpan = document.createElement('span');
    var closeA = document.createElement('a');
    // NOTE: we have an href so the anchor shows up as clickable... I suppose
    //   we could do that with CSS too though...
    closeA.setAttribute('href', '#');
    closeA.innerHTML = MS.intl(closeI18n);
    // when we click close, it removes the popup element.
    MS.addElementOnClick(closeA, function(e) {
        // don't have the href be followed.
        if (e) {
            // some versions of IE don't have preventDefault...
            if (e.preventDefault) {
                e.preventDefault();
            } else {
                e.returnValue = false;
            }
        }
        MS.hideHelpPopup(bodyI18n);
        return false;
    });
    closeSpan.appendChild(closeA);
    popup.appendChild(closeSpan);

    // we need to become visible - default css style is currently to have
    // us be hidden. TODO: we could change that and remove this alteration...
    popup.style.display = 'block';

    // add popup into the document. why do we add as a child of the wrapper? so
    // we benefit from css rules.
    // NOTE: we do this before positioning it as we need to add it to the
    //   document to determine its size information.
    var wrapper = document.getElementById('magicspam-wrapper');
    if (!wrapper) {
        MS.log("showHelpPopup: wrapper not found");
        return;
    }
    wrapper.appendChild(popup);
    // the body of the popup. NOTE: don't use innerHTML or we'll lose
    // our events!
    // NOTE: for IE11 we *must* set our HTML to the element once the
    //   element is in the document. why? because if we happen to have HTML
    //   with an onclick attribute in it, it will not work if we set it
    //   before adding it into the document. it appears to be an IE11
    //   gotcha. see https://connect.microsoft.com/IE/feedback/details/709187/events-not-triggered-if-setting-innerhtml-when-active-scripting-in-internet-security-zone-to-disabled
    popup.insertAdjacentHTML('beforeend', MS.intl(bodyI18n));

    // we use our dynamic positioning unless we've been provided explicit x/y.
    if (x !== undefined && y !== undefined) {
        popup.style.left = x + 'px';
        popup.style.top = y + 'px';
    } else {
        var coords = MS._positionHelpPopup(popup, caller, position);
        if (!coords) {
            MS.log("showHelpPopup: failed to find positioning coords");
            return;
        }
        popup.style.left = coords.x + 'px';
        popup.style.top = coords.y + 'px';
    }
};

//! removes all elements with className of a specific HTML tag
/*!
* @param tagName the HTML tag of the element
* @param className the class name of the element
*
* @author Daniel Peitl <danielp@linuxmagic.com>
*/
function removeElems(tagName, className) {
    // elems to be captured, array to filter and its counter
    var elems = document.getElementsByTagName(tagName);
    var toDel = [], j = 0;

    // fill the array with className elements
    for (var i=0; i<elems.length; i++) {
        // filter the element of desired className
        if (elems[i].className === className) {
            toDel[j] = elems[i];
            j++;
        }
    }
    // remove every className element from the page
    for (var i=0; i<j; i++) {
        toDel[i].parentNode.removeChild(toDel[i]);
    }
    return true;
}
