<?php
/*
 * vim: ft=php
 *
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): William Storey <william@linuxmagic.com>
 *
 * $Id: ms_notifications_convert_read.php 3825 2014-09-29 21:32:38Z shaun $
 *
 * This program is intended to be run as a postinst step for the upgrade
 * to MagicSpam 1.0.7.
 *
 * In this upgrade we changed the formats of one of the notification system
 * related files: <feed ID>.read.
 * Previously this would contain a single value: A unixtime indicating that
 * all posts on or before that time would be read.
 *
 * We've now converted this to contain instead a serialised PHP array of
 * LMNotificationsEntry objects (like <feed ID>.cache file which contains
 * the unread entries) so that we can mark individual feed items as read
 * or not.
 * Essentially we've now split out feed items to be in two files: .cache
 * and .read files which are in the same format, but any feed item in .cache
 * is treated as unread, and any in .read is treated as read.
 *
 * NOTE: You must be sure to run this as the system user that the interface
 *   uses or else there may be issues if we create feed files because the
 *   interface will then not be able to work with them!
 */

// ensure we have the 'include' paths pointing back to our tree.
// NOTE: MS_PHP_BASE_PATH gets updated on package build.
define('BASE_DIR', 'MS_PHP_BASE_PATH');
set_include_path(BASE_DIR . "includes" . PATH_SEPARATOR . get_include_path());
set_include_path(BASE_DIR . "lib" . PATH_SEPARATOR . get_include_path());

// our base requirements.
// config.php defines and suite specific utility path information
// as well we need it so we can get at what our feed id is that we will work
// with as well as contains the information for how many days back we purge
// each feed.
// NOTE: switched to using full path to the file as config.php may be
//   elsewhere in our path I imagine.
require_once(BASE_DIR . '/includes/config.php');
// TODO: should we just go with a full path for include here too?
//   then we could avoid changing include path.
require_once('lib.msnotifications.php');

//! convert 'read' format of a feed to the new form.
/*!
 * @param int $feedID The feed ID.
 * @param bool $verbose Enable/disable verbose mode.
 *
 * @return bool Whether successful
 *
 * if we find that the .read file for the feed is not in the old format, then
 * we have nothing to do.
 *
 * otherwise we need to load up the .cache entries, and set those read that
 * are on or before the unixtime from the old .read file.
 */
function msncrConvertFeed($feedID) {
    if (!is_int($feedID)) {
        echo "Invalid parameter to msncrConvertFeed().\n";
        return FALSE;
    }
    if (!defined('CONTROLDIR')) {
        header('HTTP/1.0 500 Control directory not set!');
        return FALSE;
    }
    // TODO: should we have this as a method/function somewhere? risky if
    //   we ever change the path - multiple locations to update.
    $readFile = CONTROLDIR . DIRECTORY_SEPARATOR . $feedID . '.read';
    // if we don't have a .read file for this feed, then there's nothing
    // to do.
    if (!file_exists($readFile)) {
        return TRUE;
    }
    // read it in so we can check if it has the old form.
    $contents = file_get_contents($readFile);
    if ($contents === FALSE) {
        header("HTTP/1.0 500 Failed to read contents of feed read file: $readFile");
        return FALSE;
    }
    $contents = trim($contents);
    // if it's a single numeric value, then it's the old format.
    if (!is_numeric($contents)) {
        return TRUE;
    }
    $readUnixtime = intval($contents);
    
    // we need to convert to the new form. this entails loading up all
    // 'unread' local entries and setting any that have a timestamp <= this
    // 'read unixtime' to be read. this will cause them to be in the new
    // format.
    $notification = new MSNotifications($feedID);
    $unreadRepo = $notification->getLocal();
    // get *all* entries - so timestamp 0.
    $retrieveTimestamp = 0;
    $entries = $unreadRepo->getEntries($retrieveTimestamp);
    foreach ($entries as $entry) {
        if ($entry->timestamp > $readUnixtime) {
            continue;
        }
        if (!$notification->setItemRead(intval($entry->timestamp))) {
            header('HTTP/1.0 500 Failed ot set entry with timestamp ['
                . $entry->timestamp . "] as read: "
                . $notification->_error);
            return FALSE;
        }
    }
    return TRUE;
}

//! program entry.
/*!
 * @return bool True if success, false if failure
 *
 * @author William Storey <william@linuxmagic.com>
 *
 */
function main() {
    // we currently hardcode the feeds we work with.
    if (!defined('ALERT_FEED_ID')
        || !defined('ALERT_FEED_ENTRY_LIFETIME_DAYS')) {
        echo "Missing alert feed settings!\n";
        return FALSE;
    }
    if (!msncrConvertFeed(ALERT_FEED_ID)) {
        // we should have logged.
        return FALSE;
    }
    if (!defined('NEWS_FEED_ID')
        || !defined('NEWS_FEED_ENTRY_LIFETIME_DAYS')) {
        echo "Missing news feed settings!\n";
        return FALSE;
    }
    if (!msncrConvertFeed(NEWS_FEED_ID)) {
        // we should have logged.
        return FALSE;
    }
    return TRUE;
}

exit(main() ? 0 : 1);
