<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Logan Borean <loganb@linuxmagic.com>
 *
 * $Id: baseStatsModule.php 4617 2014-12-17 18:51:59Z logan $
 */
require_once('lib.module.php');
require_once('lib.rcptcounter.php');
require_once('lib.datatable.php');
require_once('lib.logsearch.php');


class BaseStatsModule extends Module {

    protected $allowedNumSelections = array(5, 10, 15, 20, 25, 30);
    protected $numSelections;
    protected $rcptStats;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     *
     * @author         Logan Borean <loganb@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->numSelections = null;
        $this->rcptStats = null;
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     *
     * @author         Logan Borean <loganb@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
    }

    //! Returns whether the form submission for the number of selections was valid
    /*!
     * @returns boolean   whether the form submission was valid
     *
     * @author  Logan Borean <loganb@linuxmagic.com>
     */
    protected function isValidFormSubmission() {
        return isset($_POST['num-selections']) &&
               is_numeric($_POST['num-selections']) &&
               in_array(intval($_POST['num-selections']), $this->allowedNumSelections);
    }

    //! Returns the default table HTML
    /*!
     * @returns string   the default table HTML
     *
     * @author  Logan Borean <loganb@linuxmagic.com>
     */
    protected function getDefaultTableHTML() {
        return '<div class="rowTable">'
             .     '<div class="oddRow">'
             .         '<div class="leftCol">'
             .             '<span title="No Stats" class="defaultLabel">'
             .                 intl('todayStatsNoStats')
             .             '</span>'
             .         '</div>'
             .         '<div class="rightCol">'
             .             '<div class="chartBar">&nbsp;</div>'
             .         '</div>'
             .         '<div>&nbsp;</div>'
             .      '</div>'
             . '</div>';
    }

    //! Sets the max_execution_time
    /*!
     *
     * @author  Logan Borean <loganb@linuxmagic.com>
     */
    protected function setMaxExecutionTime() {
        ini_set('max_execution_time', 300);
    }

    //! Returns the number of selections that have been POST'd
    /*!
     * @returns int   the number of selections
     *
     * @author  Logan Borean <loganb@linuxmagic.com>
     */
    protected function getSelectionsFromFormData() {
        if ($this->isValidFormSubmission()) {
            return $_POST['num-selections'];
        } else {
            //set default to first element of the allowed values
            return $this->allowedNumSelections[0];
        }
    }

    //! Returns the number of selections
    /*!
     * @returns int   the number of selections
     *
     * @author  Logan Borean <loganb@linuxmagic.com>
     */
    protected function getNumSelections() {
        if ($this->numSelections === NULL) {
            return $this->allowedNumSelections[0];
        }
        return $this->numSelections;
    }

    //! Generates an html table used for displaying stats
    /*!
     * @param   string   The type of statistic to retrieve from the RCPT stats
     * @param   array    Any options needed to specify for the getTopStats() method
     *                   for the LogSearch.
     *
     * @returns string   HTML table
     *
     * @author  Logan Borean <loganb@linuxmagic.com>
     */
    protected function getStatsTable($statType, $options = array(), $localize = FALSE) {
        $this->setMaxExecutionTime();
        if ($this->numSelections === NULL) {
            $this->numSelections = $this->getSelectionsFromFormData();
        }

        if ($this->rcptStats === NULL) {
            // Determines the top 5 senders / receivers & sorts them from highest
            // to lowest create a LogSearch object.
            $newLogsSearch = new LogSearch();
            $this->rcptStats = $newLogsSearch->getTopStats($this->numSelections, $options);
        }


        $htmlTable = $this->getDefaultTableHTML();
        if (!isset($this->rcptStats[$statType])) {
            return;
        }

        if (count($this->rcptStats[$statType]) > 0) {
            $table = $this->generateTable($this->rcptStats[$statType], $localize);
            $htmlTable = $table->getRenderString();

            // If an error occurs, use our default HTML.
            if (!$htmlTable) {
                $htmlTable = $this->getDefaultTableHTML();
            }
        }
        return $htmlTable;
    }


    public function renderModule() {
    }

    //! method to generate a table used for displaying stats
    /*!
     * @param   array    contains a list of addresses / domains
     * @param   boolean  determines whether values need localization
     *
     *
     * @author  Robert Bisewski <robertb@linuxmagic.com>
     */
    protected function generateTable($aList, $localize = FALSE) {

        // Input validation, if we fail we just return an empty table.
        if (!$aList || !is_array($aList) || !is_bool($localize)) {
            $table = new magicspamDataTable;
            return $table;
        }

        // The MagicSpam color as a RGB value
        $red   = 148;
        $green = 44;
        $blue  = 49;

        // Basic variables for the table
        $maxWidth = 140;
        $largest = reset($aList);
        $table = new magicspamDataTable;

        // Cycle through our values and assemble each row
        foreach ($aList as $label => $value) {

            // If our largest value is zero, don't bother generating the bar
            // graphs. Note that this should probably never happen,
            // nonetheless might as well be on the safe side.
            if ($largest <= 0) {
                break;
            }

            // Attempt to calculate our bar width length
            $barWidth = intval($maxWidth * ($value / $largest));

            // If there's only one element, the bar is full width
            if (count($aList) == 1) {
                $barWidth = $maxWidth;
            }

            // By default we want the minimum bar width to be 1px
            if ($barWidth < 1) {
                $barWidth = 1;
            }

            // Determines the colour of the bar
            $multiplier = ($barWidth < 40) ? $maxWidth / 40 : $maxWidth / $barWidth;
            $barRed     = intval($red   * $multiplier);
            $barGreen   = intval($green * $multiplier);
            $barBlue    = intval($blue  * $multiplier);

            // Creates the bar that is displayed on the graph
            $bar  = "<div class=\"chartBar\" style=\"";
            $bar .= "background-color: rgb(" . htmlspecialchars($barRed);
            $bar .= "," . htmlspecialchars($barGreen) . ",";
            $bar .= htmlspecialchars($barBlue) . "); width: ";
            $bar .= htmlspecialchars($barWidth) . "px;\">&nbsp;</div>\n";

            // If we need to localize the label, go ahead and do so.
            if ($localize) {
                $label = intl("{$label}");

            // Otherwise we probably already have the label, but as a safety
            // measure we take into account possible special characters.
            } else {
                $label = htmlspecialchars($label);
            }

            // If we're not given a proper label, then we likely have a
            // bounced message
            if ($label === '' || $label === "(null)" ) {
                $label = intl('bounce_message');
            }

            // create the label that is displayed on the graph
            $labelHTML = $this->generateLabelForStatsTable($label);

            // Add the label+bar+value row to our graph.
            $table->addRow(array($labelHTML, $bar, $value));
        }

        // Returns the completed table
        return $table;
    }


    //! default function for generating labels for stats table
    /*!
     * @param   string          the label content string
     *
     * @returns html            the html for the labels
     *
     * @author  Lucas Nincao <lucasn@linuxmagic.com>
     *
     * NOTE: overload function when you need to control the behavior of how the
     *       labels for the statistic tables are generated
     *
     * NOTE: currently the Inbound and Outbound Stats module uses the default
     */
    protected function generateLabelForStatsTable($label) {
        // Attempt to assemble the HTML content for our label; at this
        // point the value of $label should already take into account
        // special HTML chars either through the encoding or from the
        // result of our intl() function. Ergo, this is safe.
        return "<span title='$label'>$label</span>\n";
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Logan Borean<loganb@linuxmagic.com>
     */
    public function getCSS() {
        return array('today-stats.css');
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author         Logan Borean<loganb@linuxmagic.com>
     */
    public function getJS() {
        return array();
    }


}
