<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *            Peter Chung <peterc@linuxmagic.com>
 *
 * $Id: lib.datatable.php 10940 2020-01-14 00:44:10Z deiony $
 *
 * This is a brief utility class for generating 2 column data
 * tables for the MagicSpam interface modules.
 *
 * Using the class:
 * // example to output a 2 column table
 * // this will output a table with table class = rowTable
 * // each row will have have alternate rows with classes starting with oddRow then evenRow
 * // the left cell will have class leftCol and the right cell rightCol
 * $table = new magicspamDataTable;
 * $table->addRow( array("old date", "2011-01-01");
 * $table->addRow( array("new date", "2011-01-02");
 * echo $table->getRenderString();
 *
 * // changing the default row cell classes
 * // this code will set the left cell to leftColumn and the right cell to rightColumn
 * $table->setClass("row", array("leftColumn", "rightColumn"));
 *
 * // using the class with more than 2 columns
 * // without defining the cell overwrites
 * // the cells will have classes, leftCol, rightCol, rightCol
 * $table = new magicspamDataTable;
 * $table->addRow( array("1", "2", "3");
 * $table->addRow( array("4", "5", "6");
 * echo $table->getRenderString();
 *
 * // using the class with more than 2 columns and matching cell classes
 * $table = new magicspamDataTable;
 * $table->setClass( "row", array("leftColumn", "dataColumn", "rightColumn");
 * $table->addRow( array("1", "2", "3");
 * $table->addRow( array("4", "5", "6");
 * echo $table->getRenderString();
 *
 * // using the class with more than 2 columns and matching cell classes and header classes
 * $table = new magicspamDataTable;
 * $table->setClass( "row", array("leftColumn", "dataColumn", "rightColumn");
 * $table->setClass( "headerRow", array("leftHeader", "dataHeader", "rightHeader");
 * $table->setHeader(array("first", "second", "third"));
 * $table->addRow( array("1", "2", "3");
 * $table->addRow( array("4", "5", "6");
 * echo $table->getRenderString();
 */

include_once('wizard/lib.wizard_validator.php');

class magicspamDataTable {
    // internal buffer of the 'label' to 'value' rows.
    private $rows;

    // internal buffer for an optional 'header'.
    private $header;

    // internal buffer of the classes
    private $classes;

    // flag to alternate the row colours, default is TRUE
    private $wantAlternateColors;

    //! an associative array where we track a row's index in the table
    // with an optional id attribute. not all rows need to specify this.
    // this is to let us specify id attributes on particular rows which is
    // useful for styling purposes.
    private $_row_index_to_id;

    //! an associative array where we track a row's classes in the table
    private $_row_index_to_classes;

    // an associative array in the format array('name'=>$name, 'value'=>$value)
    // used to insert custom data attributes to html elements
    private $_row_index_to_custom_data;

    //! class constructor
    public function __construct() {
        // initialize the rows as an array.
        $this->classes = array(
            "headerRow" => array("leftCol", "rightCol"),
            "row"       => array("leftCol", "rightCol")
        );
        $this->rows = array();
        $this->header = NULL;
        $this->wantAlternateColors = TRUE;
        $this->_row_index_to_id = array();
        $this->_row_index_to_classes = array();
        $this->_row_index_to_custom_data = array();
    }

    //! sets the flag that determines whether to alternate the row colors
    //
    /*
     * @param boolean $flag the flag that will detemine whether the rows will alternate in color
     */
    public function setAltColors($flag) {
        if (isset($flag) && is_bool($flag)) {
            $this->wantAlternateColors = $flag;
        }
    }

    //! sets the table classes to modify the behaviour of the table
    //  $classes["row"] = array ("leftCol", "dataCol", "rightCol"); will set the row to have classes of these names
    //  $classes["headerRow"] = array ("leftCol", "dataCol", "rightCol");
    //                                 will set the header row to have these classes
    //  if there are not enough columns to fill up the classes, the last column will be used to fill up the remainder
    //
    /*
     * @param string $key the class to set
     *
     * @param array $value the values to set for the class
     */
    public function setClass($key, $value) {
        if (isset($key) && is_string($key) && is_array($value)) {
            $this->classes[$key] = $value;
        }
    }

    //! function to add a row to table
    /*!
     * @param array $row       The row's values.
     * @param mixed $id        An optional string to set as the row element's id
     *                         attribute, or NULL to not specify.
     * @param array $data      an optional array for setting custom data attributes
     * @param array $classes   an optional array for setting custom classes for row
     *
     * @returns boolean     success or fail based on validity of arguments.
     *
     * NOTE: these calls must be made prior to 'render' as this is buffered data.
     */
    public function addRow($row, $id = NULL, $data = array(), $classes = array()) {
        // sanity check
        if (!WV::isArray($row) || !WV::isArray($this->rows)) {
            return FALSE;
        }

        // ensure that the number of header and row columns match
        if (isset($this->header) && count($this->header) != count($row)){
            return FALSE;
        }

        // append the row to the table
        $this->rows[] = $row;

        // calculate the row index
        $rowIndex = count($this->rows) - 1;

        // set id on the row if available
        if (WV::isStringNotEmpty($id)) {
            $this->_row_index_to_id[$rowIndex] = $id;
        }

        // set custom class on the row if available
        if (WV::isArrayNotEmpty($classes)) {
            $this->_row_index_to_classes[$rowIndex] = $classes;
        }

        // set custom data attribute on the row if available
        if (isset($data['name']) && isset($data['value'])) {
            $this->_row_index_to_custom_data[$rowIndex] = $data;
        }

        return TRUE;
    }

    //! function to add a row to table that has center aligned text
    /*!
     * @param   string  the 'value'
     *
     * @returns boolean success or fail based on validity of arguments.
     *
     * NOTE: these calls must be made prior to 'render' as this is buffered data.
     */
    public function addBlurbRow($string) {
        if (!isset($string) || strlen($string) <= 0) {
            return FALSE;
        }

        if (!is_array($this->rows)) {
            return FALSE;
        }

        if (isset($this->header) && count($this->header) != count($row)){
            return FALSE; // header and row count do not match
        }

        $this->rows[] = $string;

        return TRUE;
    }

    //! function to set the 'header' of the datatable.
    /*
     * @param array $row the array of header strings
     *
     * @return FALSE if row is not an array, TRUE if okay
     */
    public function setHeader($row) {
        if (!is_array($row)) {
            return FALSE; }
        // reset the header.
        $this->header = $row;
        return TRUE;
    }


    //! function to get the table for rendering.
    /*!
     * return the string, it is the calling programme's responsibility to do
     *                    output it
     */
    public function getRenderString() {
        if (!is_array($this->rows)) {
            return "";
        } else if (!is_array($this->classes)) {
            return "";
        } else if (!isset($this->classes["row"]) || count($this->classes["row"])==0) {
            return "";
        }
        $html = "";
        $html .= "<div class=\"rowTable\">\n";
        $isodd = TRUE;
        $max_class_size = count($this->classes["row"]);
        if (is_array($this->header)) {
            if (!isset($this->row) || !is_array($this->row) || count($this->row) == 0) { // no rows, still want to display the header
                $max_class_size = count($this->classes["headerRow"]); // use the header count
            }
            $html .= "    <div class=\"rowHeader\">\n";
            foreach ($this->header as $l => $val) {
                $i = $l;
                $class = "";
                if (isset($this->classes["headerRow"])) {
                    if ($i>$max_class_size) {
                        $i = $max_class_size;
                    }
                    $class = $this->classes["headerRow"][$i];
                }
                $html .= "        <div class=\"".$class."\">{$val}</div>\n";
            }
            $html .= "    </div>\n";
        }
        $rowIndex = 0;
        foreach ($this->rows as $row) {
            $html .= "    <div class=\"";
            if ( $this->wantAlternateColors ) {
                if ($isodd) {
                    $html .= "oddRow";
                    $isodd = FALSE;
                } else {
                    $html .= "evenRow";
                    $isodd = TRUE;
                }
            } else {
                $html .= "normalRow";
            }
            $html .= '"';

            // some rows receive an id attribute.
            if (isset($this->_row_index_to_id[$rowIndex])
                && WV::isStringNotEmpty($this->_row_index_to_id[$rowIndex])) {
                $html .= ' id="';
                $html .=  htmlspecialchars($this->_row_index_to_id[$rowIndex]);
                $html .= '"';
            }

            // some rows receive a custom data attribute
            if (isset($this->_row_index_to_custom_data[$rowIndex])
                && isset($this->_row_index_to_custom_data[$rowIndex]['name'])
                && isset($this->_row_index_to_custom_data[$rowIndex]['value'])) {
                $customDataName = htmlspecialchars($this->_row_index_to_custom_data[$rowIndex]['name']);
                $customDataValue = htmlspecialchars($this->_row_index_to_custom_data[$rowIndex]['value']);
                $html .= " data-$customDataName='$customDataValue'";
            }

            $html .= ">\n";

            if (!is_array($row) && isset($row) && strlen($row) > 0) {
                $html .= "<div class='rowBlurb'>" . $row . "</div>\n";
            }

            $row_classes = isset($this->_row_index_to_classes[$rowIndex]) ? $this->_row_index_to_classes[$rowIndex] : $this->classes["row"];
            $max_class_size = count($row_classes);
            if (is_array($row)) {
                foreach ($row as $l => $val) {
                    $i = $l;
                    $class = "";
                    if (isset($row_classes)) {
                        if ($i>$max_class_size) {
                            $i = $max_class_size;
                        }
                        // TODO: can we end up spitting out empty class attributes?
                        if (isset($row_classes[$i])) {
                            $class = $row_classes[$i];
                        }
                    }
                    $html .= "        <div class=\"".$class."\"";
                    $html .= ">{$val}</div>\n";
                }
            }

            $html .= "    </div>\n";
            $rowIndex++;
        }
        $html .= "</div>\n";
        return $html;
    }
}
