<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.dstatus.php 4221 2014-11-04 23:28:57Z shaun $
 *
 * This is a container to access the 'dstatus' utility
 * which interacts with the ability to protect a domain.
 */
include_once('lib.cmdwrapper.php');

class DStatus extends cmdWrapper {
    // internal private singleton.
    static private $object = NULL;

    //! class constructor
    /*!
     * @author Shaun Johnson <shaun@linuxmagic.com>
     * NOTE: we want this class type to be a singleton,
     *       so we deny access to the constructor
     *
     */
    protected function __construct() {
        // instantiate back to the parent.
        parent::__construct(DSTATUS);
    }

    //! static function to get the current singleton instance
    /*!
     * @returns LicenseManager  the active singleton instance.
     *
     * @author Shaun Johnson <shaun@linuxmagic.com>
     *
     * SIDE EFFECTS: If the instance is not yet created,
     *               then this will create it.
     */
    static public function getInstance() {
        if (is_null(self::$object)) {
            self::$object = new DStatus;
        }
        return self::$object;
    }

    //! access function to determine if a given domain is protected or not
    /*!
     * @param   string  the domain to check
     *
     * @returns boolean true on protected, false on not protected.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function domainProtected($domain) {
        $buffer = "";
        // sanity check
        if (!is_string($domain) || !strlen($domain)) {
            $this->lastError = "Invalid parameters";
            return FALSE;
        }
        $args = array('-q');
        $args[] = $domain;
        $rv = $this->execRead($args, $buffer);
        if (strlen($buffer)) {
            return TRUE;
        }
        return FALSE;
    }

    //! method to protect a domain
    /*!
     * @param   string  the domain to protected
     * @param   string  the associated 'home' directory for the domain.
     *
     * @returns boolean success or fail of the operation.
     *                  error messages can be retrieved using the 'error()'
     *                  method.
     *
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function protectDomain($domain, $home) {
        $buffer = "";
        // sanity check.
        if (!is_string($domain) ||
            !is_string($home) ||
            !strlen($domain) ||
            !strlen($home)) {
            $this->lastError = "Invalid parameters";
            return FALSE;
        }
        $args = array('-a');
        $args[] = $domain;
        $args[] = '-h';
        $args[] = $home;
        $rv = $this->execRead($args, $buffer);
        if (strlen($buffer)) {
            $this->lastError = $buffer;
            return FALSE;
        }
        return TRUE;
    }

    //! method to remove protection for a domain.
    /*!
     * @param   string  the domain to remove protection from.
     *
     * @returns boolean success or fail. Error messages can
     * be retrieved using the 'error()' method.
     *
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function unprotectDomain($domain) {
        $buffer = "";
        // sanity check
        if (!is_string($domain) || !strlen($domain)) {
            $this->lastError = "Invalid parameters";
            return FALSE;
        }
        $args = array('-d');
        $args[] = $domain;
        $rv = $this->execRead($args, $buffer);
        if (strlen($buffer)) {
            $this->lastError = $buffer;
            return FALSE;
        }
        return TRUE;
    }

    //! method to remove protection for an entire home directory.
    /*!
     * @param   string  the 'home directory' to remove protection for.
     *
     * @returns boolean success or fail. Error messages can be
     *                  retrieved using the 'error()' method.
     *
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function unprotectHome($home) {
        $buffer = "";
        // sanity check
        if (!is_string($home) || !strlen($home)) {
            $this->lastError = "Invalid parameters";
            return FALSE;
        }
        // run the command
        $args = array('-h');
        $args[] = $home;
        $args[] = '-r';
        $rv = $this->execRead($args, $buffer);
        if ($rv != 0) {
            $this->lastError = $buffer;
            return FALSE;
        }
        return TRUE;
    }
}
