<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s):  Peter Chung <peterc@linuxmagic.com>
 *             Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: lib.entryform.php 11866 2020-07-08 00:08:11Z deiony $
 */

require_once('lib.listmanager.php');
require_once('lib.module.php');
require_once('lib.datatable.php');
require_once('config.php');
require_once('lib.country.php');

class EntryForm extends Module {

    // the name of the entry form
    protected $name;

    // the name of the submodule
    protected $submodule;

    // the list flag for magicspam-listmgr
    protected $cmd_flag;

    //! the type of entry that is handled by the entry form
    protected $entry_type;

    // the name and id of the entry form (HTML)
    protected $form_id;

    // whether wildcards are allowed in the entry
    protected $allow_wildcards;

    // the loaded entries for the list, each entry has an accompanying comment
    protected $entries;

    // the number of entries that were loaded
    protected $entry_count;

    // specific i18n labels for localization
    protected $localization;

    // the entry types that can be handled by an entry form
    const ENTRY_UNKNOWN = 0;
    const ENTRY_IP      = 1;
    const ENTRY_EMAIL   = 2;
    const ENTRY_COUNTRY = 3;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @author Peter Chung <peterc@linuxmagic.com>
     */
    public function __construct($l) {
        global $ms_words;
        parent::__construct($l, FALSE);

        // default initialization
        $this->entry_type = self::ENTRY_UNKNOWN;
        $this->allow_wildcards = FALSE;
        $this->cmd_flag = "";
        $this->form_id = "";
        $this->entry_count = 0;
        $this->entries = array();
        $this->localization = array(
            "desc" => "desc",
            "note" => NULL,
            "header" =>"header",
            "table_header" => "table_header",
        );

        // set module to be migratable
        $this->migratable = TRUE;

        // explicitly load the locale as the EntryForm module does not follow
        // standard Module class conventions
        $moduleLocale = loadMSLocale('EntryForm');
        if (file_exists($moduleLocale)) {
            require_once($moduleLocale);
        }
    }

    //! gets the write folder
    /*!
     * @return the correct folder to write to
     * @author Peter Chung <peterc@linuxmagic.com>
     */
    protected function getFolder() {
        global $home;
        $write_folder = "";

        // when running out of cPanel (user) then the variable 'home' will be set at global level
        // so if it is, we want to use that as our base to write out the data
        if (isset($home)) {
            $write_folder = $home;
        } else {
            // otherwise, we will use the configuration driven base for 'global' settings
            if (!defined("DATA_PATH")) {
                // and also use a safety check.
                $write_folder = "/etc";

                // NOTE: Peter originally set the echo below - this is dangerous
                //       as this function gets called from NON-RENDERING functions
                //       and as such may write ahead of required headers etc. - SAJ
                // echo "setting to /etc DATA_PATH not defined.<br>";
            } else {
                $write_folder = DATA_PATH;
            }
        }
        return $write_folder;
    }

    //! internal method to validate a passed in value and its criteria
    /*!
     * @param string $value the string to validate against
     * @param string $msg the warning or error message
     *
     * @return TRUE if successful, FALSE if error
     *
     * @author Peter Chung <peterc@linuxmagic.com>
     * @author Robert Bisewski <robertb@linuxmagic.com>
     */
    protected function validateEntry($value, &$msg) {
        // ensure that there is an entry to validate
        if (!isset($value)) {
            $msg = "blank_entry";
            return FALSE;
        }

        // ensure that the class has been initialized with a valid entry type
        // so that we know which validation routine to run
        if ($this->entry_type != self::ENTRY_IP &&
            $this->entry_type != self::ENTRY_EMAIL &&
            $this->entry_type != self::ENTRY_COUNTRY) {
            // TODO: change i18n label for this error message
            $msg = "both_email_and_ip";
            return FALSE;
        }

        // run the appropriate validation based on the entry type
        // XXX: consider moving the code in each case into a separate function
        //      i.e. validate_ip_entry, validate_email_entry, etc...
        switch ($this->entry_type) {
        case self::ENTRY_IP:
            // Check for wild cards
            if (strchr($value, "*")) {
                $msg = "wildcards_not_supported";
                return FALSE;
            }
            $value = preg_replace('/\s+/', '', $value);
            // because we support ranges, we want to test that too...
            $range_parts = explode('-', $value);
            if (sizeof($range_parts) > 2) {
                $msg = "invalid_range";
                return FALSE;
            }
            // but we don't want to support ranges for IP rate limiter exemption - not yet.
            if (sizeof($range_parts) == 2 && $this->name === 'ip_rl') {
                // $msg = "invalid_entry";
                $msg = "range_not_supported";
                return FALSE;
            }

            foreach ($range_parts as $r) {
                // this tests if its an IP address - that includes IPv6
                if (!$this->isValidIP($r)) {
                    $msg = 'invalid_entry';
                    return FALSE;
                }
                // test that it isn't a localhost entry
                if ($r == '127.0.0.1' ||
                    $r == '::1') {
                    //$msg = 'invalid_entry';
                    $msg = 'localhost_not_permitted';
                    return FALSE;
                }
                // IF this is a range OR rate limiter exemption list
                if (sizeof($range_parts) == 2 || $this->name === 'ip_rl') {
                    // then ensure that the values are IPv4 only
                    if (!filter_var($r, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) {
                        $msg = 'ip4_only';
                        return FALSE;
                    }
                }
            }
            // ok we are good to go.
            return TRUE;
        case self::ENTRY_EMAIL;
            // first.. we check email address validity
            if (!$this->isRFCCompliantEmail($value)) {
                $msg = "invalid_email";
                return FALSE;
            }

            // then we split the parts up...
            $parts = explode('@', $value);
            // parts should contain [user] [domain]
            if (sizeof($parts) != 2) {
                $msg = "invalid_email";
                return FALSE;
            }
            // we want to know about the domain parts too
            $domain_parts = explode('.', $parts[1]);
            if (sizeof($domain_parts) < 2) {
                $msg = "nonstandard_domain";
                return FALSE;
            }
            if (!isProfessional()) {
                // non pro does not permit full domain wild cards so..
                for ($c = 0; $c < sizeof($domain_parts) - 1; $c++) {
                    if ($domain_parts[$c] == '*') {
                        $msg = "domain_wildcards_not_supported";
                        return FALSE;
                    }
                }
            }
            // and no suite permits the full domain to be '*'
            $all_wild = TRUE;
            for ($c = 0; $c < sizeof($domain_parts); $c++) {
                if ($domain_parts[$c] != '*') {
                    $all_wild = FALSE;
                    break;
                }
            }
            if ($all_wild) {
                $msg = "invalid_email";
                return FALSE;
            }

            // Test the domain part is not just an ip address
            if ($this->isValidIP($parts[1])) {
                $msg = "invalid_email";
                return FALSE;
            }

            // we create a dummy place holder to deal with wildcard validation
            $dummy = $value;

            // Test the entry for attempted wildcard entries....
            if (strchr($value,"*")) {
                if (!$this->allow_wildcards) {
                    $msg = "wildcards_not_supported";
                    return FALSE;
                }
                $msg = "wildcards_not_recommended";
                // and because its a wild card and we know it is prone to error...
                // we create a dummy email address without the glob character..
                // NOTE: we retain all other characters though in case someone is
                //       attempting regex entries.. which we don't support.
                $dummy = preg_replace("/\*/", "a", $value);
            }

            // Finally, uses built-in PHP functions to check if valid
            if (filter_var($dummy, FILTER_VALIDATE_EMAIL) == FALSE) {
                $msg = "invalid_email";
                return FALSE;
            }

            // Otherwise, the email is good, so returns true
            return TRUE;
        case self::ENTRY_COUNTRY:
            return (Country::getCountryNameFromCode($value) !== FALSE);
        default:
            break;
        }

        // this should never be reached...
        $msg = "cannot use this function";
        return FALSE;
    }

    //! add entry from posted value
    /*!
     * @param array $errors the error array
     * @param array $notices the notices array
     *
     * @author Peter Chung <peterc@linuxmagic.com>
     * @author Robert Bisewski <robertb@linuxmagic.com>
     *
     * NOTES
     * any errors will be added to the $errors array
     * any notices will be added to the $notices array
     */
    protected function addEntry(&$errors, &$notices) {
        // ensure the correct POST variables have been sent
        if (!isset($_POST['new_entry']) || !WV::isStringNotEmpty($_POST['new_entry'])) {
            $errors[] = intl('invalid_entry');
            return;
        }

        // remove extra * characters that might be present in an address
        $new_entry = trim($_POST['new_entry']);
        $new_entry = preg_replace('/\*+/', '*', $new_entry);
        $new_entry = preg_replace('/\s+/', '', $new_entry);

        // validate the new entry
        $msg = "";
        if (!$this->validateEntry($new_entry, $msg)) {
            $errors[] = intl($msg);
            return;
        }

        // retrieve the list to operate on
        // NOTE: capitalize the list when operating on IP ranges
        $list = $this->cmd_flag;
        if ($this->entry_type == self::ENTRY_IP && strpos($new_entry, '-') != FALSE) {
            $list = strtoupper($list);
        }

        // initialize the list manager
        $listManager = ListManager::getInstance();

        // build the arguments to pass to the list manager
        $args = array(
            $list,
            "-a", $new_entry,
            "-h", $this->getFolder(),
        );

        // add the new entry to the list
        $ret = $listManager->add($args);
        if ($ret !== TRUE) {
            switch($ret) {
            case ListManager::ENTRY_EXISTS:
                // return standard error message when entry is not an email address
                if (!filter_var($new_entry, FILTER_VALIDATE_EMAIL)) {
                    $errors[] = intl("entry_exists_error");
                    return;
                }

                // build wildcard domain for the email address
                $components = explode("@", $new_entry);
                if ($components === FALSE || count($components) != 2) {
                    $errors[] = intl("entry_exists_error");
                    return;
                }
                $wildcard_domain = "*@" . $components[1];

                // load all the entries from the list
                $entries = array();
                if ($listManager->load($this->cmd_flag, $this->getFolder(), $entries) !== TRUE) {
                    $errors[] = intl("entry_exists_error");
                    return;
                }

                // check whether wildcard domain is already an entry
                if (!in_array($wildcard_domain, $entries)) {
                    $errors[] = intl("entry_exists_error");
                    return;
                }

                // return specialized error message as wildcard domain is an entry on the list
                $errors[] = intl("entry_domain_exists_error");
                return;
            case ListManager::ENTRY_EXISTS_ELSEWHERE:
                $errors[] = intl("entry_exists_elsewhere_error");
                return;
            case ListManager::INVALID_FLAG:
                $errors[] = intl("invalid_flag_error");
                return;
            case ListManager::FLAG_ERROR:
                $errors[] = intl("flag_error");
                return;
            default:
                $errors[] = $listManager->error();
                return;
            }
        }

        // handle optional comment associated with the new entry
        if (isset($_POST["new_comment"]) && WV::isString($_POST["new_comment"])) {
            // trim the whitespace from comment
            $new_comment = trim($_POST["new_comment"]);
            // skip adding the comment when purely whitespace
            if (WV::isStringNotEmpty($new_comment)) {
                // validate that comment is not too long
                if (strlen($new_comment) > 255) {
                    $errors[] = intl("entry_comment_too_long");
                    return FALSE;
                }

                // build the arguments to pass to the list manager
                $args = array(
                    $list,
                    "-a", $new_entry,
                    "-c", $new_comment,
                    "-h", $this->getFolder(),
                );

                // call the appropriate list manager API based on the action
                $rv = $listManager->add($args);
                if ($rv !== TRUE) {
                    switch($rv) {
                    case ListManager::ENTRY_EXISTS;
                        $errors[] = intl("entry_exists_error");
                        return;
                    case ListManager::ENTRY_NOT_FOUND;
                        $errors[] = intl("entry_not_found");
                        return;
                    case ListManager::UNKNOWN_ERROR;
                    default:
                        $errors[] = $listManager->error();
                        return;
                    }
                }
            }
        }

        // store custom notice from the new entry validation if any
        // NOTE: we store notices after all the errors as notices z-index
        //       supercedes errors when rendering on the interface, i.e. errors
        //       will be hidden behind the notices
        if (WV::isStringNotEmpty($msg)) {
            $notices[] = intl($msg);
        }

        // store notice that entry was added successfully
        $notices[] = intl("add_success");
    }


    //! delete entries from posted values
    /*!
     * @param array $errors the error array
     * @param array $notices the notices array
     * @author Peter Chung <peterc@linuxmagic.com>
     */
    protected function deleteEntry(&$errors, &$notices) {
        // ensure the correct POST variables have been sent
        if (!isset($_POST["entry"]) || !WV::isArrayNotEmpty($_POST["entry"])) {
            $errors[] = intl("nothing_to_delete");
            return;
        }

        // initialize the list manager
        $listManager = ListManager::getInstance();

        // no checking is done in case user add something that cannot be deleted
        foreach (array_keys($_POST["entry"]) as $key) {
            // decode the entry to delete
            $entry = urldecode($key);

            // perform additional processing on the entry based on the entry type
            if ($this->entry_type == self::ENTRY_COUNTRY) {
                // convert country name to country code when dealing with country
                $entry = Country::getCountryCodeFromName($entry);
            } else {
                // otherwise remove all whitespace from the entry
                $entry = preg_replace('/\s+/', '', $entry);
            }

            // retrieve the list to operate on
            // NOTE: capitalize the list when operating on IP ranges
            $list = $this->cmd_flag;
            if ($this->entry_type == self::ENTRY_IP && strpos($entry, '-') != FALSE) {
                $list = strtoupper($list);
            }

            // build the arguments to pass to the list manager
            $args = array(
                $list,
                "-d", $entry,
                "-h", $this->getFolder(),
            );

            // delete the entry
            $ret = $listManager->delete($args);
            if ($ret !== TRUE) {
                switch ($ret) {
                case ListManager::ENTRY_NOT_FOUND:
                    $errors[] = intl("entry_not_found");
                    break;
                case ListManager::INVALID_FLAG:
                    $errors[] = intl("invalid_flag_error");
                    break;
                case ListManager::FLAG_ERROR:
                    $errors[] = intl("flag_error");
                    break;
                default:
                    $errors[] = $listManager->error();
                    break;
                }
            }
        }

        // abort, there was at least one error when deleting
        if (count($errors) > 0) {
            return;
        }

        // store notice that entry or entries were deleted successfully
        if (count($_POST["entry"]) == 1) {
            $notices[] = intl("delete_success");
        } else {
            $notices[] = intl("delete_multi_success");
        }
    }

    // handles the comment AJAX request
    protected function commentEntry() {
        // ensure the correct POST variables have been sent
        if (!isset($_POST["entry"], $_POST["comment"])
            || !WV::isStringNotEmpty($_POST["entry"])
            || !WV::isString($_POST["comment"])) {
            sendAjaxResponse(FALSE, intl("entry_comment_invalid"));
            return FALSE;
        }

        // process the entry accordinly based on the entry type
        $entry = trim($_POST["entry"]);
        if ($this->entry_type == self::ENTRY_COUNTRY) {
            // convert country name into country code
            $entry = Country::getCountryCodeFromName($entry);
            if ($entry == FALSE) {
                sendAjaxResponse(FALSE, intl("entry_comment_invalid"));
                return FALSE;
            }
        } else {
            // remove all whitespace from entry
            $entry = preg_replace('/\s+/', '', $entry);
            if (!WV::isStringNotEmpty($entry)) {
                sendAjaxResponse(FALSE, intl("entry_comment_invalid"));
                return FALSE;
            }
        }

        // trim the whitespace from comment
        $comment = trim($_POST["comment"]);
        if (strlen($comment) > 255) {
            sendAjaxResponse(FALSE, intl("entry_comment_too_long"));
            return FALSE;
        }

        // determine the action to take based on the comment
        // NOTE: empty comment represents deleting the comment
        $action = WV::isStringNotEmpty($comment) ? "-a" : "-d";

        // retrieve the list to operate on
        // NOTE: capitalize the list when operating on IP ranges
        $list = $this->cmd_flag;
        if ($this->entry_type == self::ENTRY_IP && strpos($entry, '-') != FALSE) {
            $list = strtoupper($list);
        }

        // build the arguments to pass to the list manager
        $args = array(
            $list,
            $action, $entry,
            "-c", $comment,
            "-h", $this->getFolder(),
        );

        // initialize list manager
        $listManager = ListManager::getInstance();

        // call the appropriate list manager API based on the action
        $rv = ($action == "-a") ? $listManager->add($args) : $listManager->delete($args);
        if ($rv !== TRUE) {
            switch($rv) {
            case ListManager::ENTRY_EXISTS;
                $error = intl("entry_exists_error");
                break;
            case ListManager::ENTRY_NOT_FOUND;
                $error = intl("entry_not_found");
                break;
            case ListManager::UNKNOWN_ERROR;
            default:
                $error = $listManager->error();
                break;
            }
            sendAjaxResponse(FALSE, $error);
            return;
        }

        // add the comment to the response data
        $data["comment"] = $comment;

        // build and return the AJAX response
        $msg = ($action == "-a") ? intl("entry_comment_save_success") : intl("entry_comment_delete_success");
        sendAjaxResponse(TRUE, $msg, $data);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author Peter Chung <peterc@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        if (!isset($action) || strlen($action) == 0) {
            $errors[] = "action is not set";
            return 0;
        } else if (!isset($this->cmd_flag) || strlen($this->cmd_flag) < 2) {
            $errors[] = intl("undefined_cmd_flag");
            return 0;
        }

        if ($action == "add") {
            $this->addEntry($errors, $notices);
        } else if ($action == "delete") {
            $this->deleteEntry($errors, $notices);
        } else if ($action == "comment") {
            $this->commentEntry();
        }

        // sets the state of the collaspible modules
        if ($action === 'remember_expand_state') {
            $module = isset($_POST['changed_module']) ? $_POST['changed_module'] : NULL;
            $value  = isset($_POST['expanded']) ? $_POST['expanded'] : NULL;
            $var = $module . "__collapse";
            $_SESSION[$var] = $value;
        }
    }

    //! get a row for the ip table
    /*
     * @param string $name the name of the checkbox
     *
     * @param string $text the text of the checkbox
     *
     * @param string $js the javascript for onclick
     *
     * @return html of checkbox if $name and $text are set, else an empty string
     *
     * @author Robert Bisewski <robertb@linuxmagic.com>
     *         Peter Chung <peterc@linuxmagic.com>
     */
    public function getRow($name, $text, $comment = "", $js = "", $form_id = "") {
        // sanity check
        if (!WV::isStringNotEmpty($name) || !WV::isStringNotEmpty($text)) {
            return "";
        }

        // retrieve localized titles
        $delete_title = intl('delete_entry');
        $comment_title = intl('comment_entry');

        // build the delete js
        $delete_js = "checkAndSubmitForm('$form_id', 'delete', '$name'); return false;";

        // build the optional checkbox js
        $checkbox_js = "";
        if (WV::isStringNotEmpty($js)) {
            $checkbox_js = "onclick=\"$js\"";
        }

        // build and return the row html
        return "<span class=\"fancyBox\">"
             .     "<input type=\"checkbox\" name=\"$name\" $checkbox_js></input>"
             . "</span>"
             . "<div class=\"entryPadding\">"
             .     "<div class=\"entryLeft\">$text</div>"
             .     "<div class=\"entryRight\">$comment</div>"
             . "</div>"
             . "<span title=\"$comment_title\" class=\"comment\"></span>"
             . "<a title=\"$delete_title\" onclick=\"$delete_js\" href=\"#\">"
             .     "<span class=\"trashcan\"></span>"
             . "</a>";
    }

    public function getModuleTitle() {
        return intl($this->localization['header']);
    }

    //! method to render the full page module.
    /*
     * @author Peter Chung <peterc@linuxmagic.com>
     */
    public function renderModule() {
        global $module, $isFrontend;

        // sanity check: make sure we have our minimum requirements.
        if (!isset($this->form_id) || strlen($this->form_id) == 0) {
            echo "Form id not set";
            return;
        } else if (!isset($this->cmd_flag) || strlen($this->cmd_flag) < 2) {
            echo intl("undefined_cmd_flag");
            return;
        }

        // grab the labels translations
        $colSort = intl("sort_by") . ": ";
        $filter = intl("search") . ": ";
        $col1 = intl("enter_new_value") .  ": ";

        // default placeholder
        $placeholder = "user@example.com";

        // sort by dropdown
        $sort_type = ($this->entry_type == self::ENTRY_IP) ? 'ip' : '';
        $colSortHTML = "<select class=\"sortDrop\" onchange=\"sortList('$this->form_id', '$sort_type')\">"
                     .     "<option value=\"order\">" . intl('order_added') . "</option>"
                     .     "<option value=\"name\">" . intl('address') . "</option>"
                     . "</select>";

        // search input field
        $filterHTML = "<select class=\"searchType\" onchange=\"filterTable('$this->form_id', 'select')\">"
                    .     "<option value=\"address\">" . intl('address') . "</option>"
                    .     "<option value=\"comment\">" . intl('comment') . "</option>"
                    .     "<option value=\"both\">" . intl('both') . "</option>"
                    . "</select>"
                    . "<input class=\"filterInput input\" oninput=\"filterTable('$this->form_id', 'input')\" type=\"text\">"
                    . "<span class=\"searchStatus\"></span>";

        // change placeholder for IP submodules
        if ($this->entry_type == self::ENTRY_IP) {
            $placeholder = "192.168.0.210";

            if ($this->name != 'ip_rl') {
                $placeholder .= " or 192.168.0.210 - 192.168.0.220";
            }
        }

        // create country dropdown menu
        if ($this->entry_type == self::ENTRY_COUNTRY) {
            $html = "<select id=\"country_id\" name=\"new_entry\">\n"
                  . "<option style='display:none;' disabled selected value></option>\n";
            foreach (Country::getCountryCodes() as $cc => $country) {
                if (!array_key_exists($country, $this->entries)) {
                    $html .= "<option value=\"$cc\"> $country </option>\n";
                }
            }
            $html .= "</select>\n";
        } else {
            // default text input for new entries
            $html = "<input class=\"input\" type=\"text\" placeholder=\"$placeholder\"" .
                    " title=\"$placeholder\" name=\"new_entry\" id=\"new_entry\" maxlength=\"1024\"/>";
        }

        // append text input for optional new comments
        $html .= "<input class=\"input\" type=\"text\" placeholder=\"" . intl('optional_comment') . "\""
               . " title=\"" . intl('optional_comment') . "\" name=\"new_comment\" id=\"new_comment\" maxlength=\"255\"/>";

        // new entry add button
        $js       = "formSubmit('" . $this->form_id . "','add');";
        $add_btn  = "<button class=\"msbutton\" onclick=\"".$js."\">\n" .
                    "<span>" . intl("add") . "</span>\n".
                    "</button>";

        // add header row (sort, search and add)
        $new_value_table = new magicspamDataTable;
        $new_value_table->setAltColors(FALSE);

        // add header row for adding entries
        $addRowHtml = array($col1, $html, $add_btn);
        $addRowClass = array("giprlw_col1", "giprlw_col2", "giprlw_col3");
        $new_value_table->addRow($addRowHtml, NULL, array(), $addRowClass);

        // add header row for sorting and searching entries
        $sortSearchRowHtml = array($colSort, $colSortHTML, $filter, $filterHTML);
        $sortSearchRowClass = array("sortCol", "sortColHtml", "filterCol", "filterColHtml");
        // no need to render the sort for country blacklist
        if ($this->entry_type == self::ENTRY_COUNTRY) {
            $sortSearchRowHtml = array_slice($sortSearchRowHtml, 2);
            $sortSearchRowClass = array_slice($sortSearchRowClass, 2);
        }
        $new_value_table->addRow($sortSearchRowHtml, NULL, array(), $sortSearchRowClass);

        // Sanity check, we want to make sure that we have data
        if (!WV::isArray($this->entries)) {
            // while this is technically an error .. we still want to render something...
            $this->entries = array();
            $this->entry_count = 0;
        }

        // Take the entries and print them out
        $data_table = new magicspamDataTable;
        $count = 0;
        foreach ($this->entries as $entry => $comment) {
            $entryRow = $this->getRow("entry[" . urlencode($entry) . "]",
                                      $entry,
                                      htmlspecialchars($comment),
                                      "",
                                      $this->form_id
                                      );

            // add custom data order id for the js sorting functionality
            $customData = array("name" => "orderid", "value" => $count++);
            $data_table->addRow(array($entryRow), NULL, $customData);
        }

        // append optional note to the description
        $description = intl($this->localization['desc']);
        if (!$isFrontend && isset($this->localization['note']) && $this->localization['note']) {
            $description .= intl($this->localization['note']);
        }

        // pass i18n to the frontend
        $this->_i18n_js[] = 'entry_comment_json_fail';
        $this->_i18n_js[] = 'entry_comment_nothing_to_delete';
        $this->_i18n_js[] = 'entry_comment_nothing_to_save';

        // Assembles the form params
        $form_params = array (
            "module"          => $module,
            "submodule"       => $this->submodule,
            "new_value_table" => $new_value_table->getRenderString(),
            "data_table"      => $data_table->getRenderString(),
            "desc"            => $description,
            "delete_key"      => intl("delete_checked"),
            "form_id"         => $this->form_id,
            "delete_js"       => "formSubmit('" . $this->form_id . "','delete');return false;",
            "entryCount"      => $this->entry_count,
            "isProfessional"  => isProfessional(),
        );
        renderTemplate("exemptions", $form_params);
    }

    //! trigger when an ajax call is made
    /*!
     * @param array $modules the modules not used here
     * @param string $controller_file not needed
     *
     * @return string the contents of errors / notices
     *
     * @author Robert Bisewski <robertb@linuxmagic.com>
     */
    public function renderOutput($modules) {
        global $notices, $errors;

        // Stores the responding errors / notices from the module in question
        $outputPresent = false;
        $response = "";
        $outputArray = array();

        // Check if there are errors
        if (is_array($errors) && count($errors) > 0) {

            // Displays the error box
            $outputPresent = true;

            // Appends all of the errors to the
            $response = "<ul>\n";
            foreach ($errors as $e) {
                $response .= "<li>" . $e . "</li>\n";
            }
            $response .= "</ul>\n";

        } else {
            $response = "no_errors";
        }

        // Storing the errors into an array
        $outputArray["errors"] = $response;

        // Check if there are notices
        if (is_array($notices) && count($notices) > 0) {

            // Displays the notice box
            $outputPresent = true;

            // Appends all of the errors
            $response = "<ul>\n";
            foreach ($notices as $n) {
                $response .= "<li>" . $n . "</li>\n";
            }
            $response .= "</ul>\n";

        } else {
            $response = "no_notices";
        }

        // Storing the notices into an array
        $outputArray["notices"] = $response;

        // Returns the response, but only if there are messages
        if ($outputPresent) {
            print json_encode($outputArray);
        }
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author Peter Chung <peterc@linuxmagic.com>
     */
    public function getCSS() {
        return array("list-forms.css");
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author Peter Chung <peterc@linuxmagic.com>
     */
    public function getJS() {
        return array("check-and-uncheck.js",
                     "ip-form.js");
    }

    //! A Standard EMail Address Checker that checks for RFC Compliant String for an email_address
    function isRFCCompliantEmail($email) {
                $qtext = '[^\\x0d\\x22\\x5c\\x80-\\xff]';
                $dtext = '[^\\x0d\\x5b-\\x5d\\x80-\\xff]';
                $atom = '[^\\x00-\\x20\\x22\\x28\\x29\\x2c\\x2e\\x3a-\\x3c'.
                        '\\x3e\\x40\\x5b-\\x5d\\x7f-\\xff]+';
                $quoted_pair = '\\x5c[\\x00-\\x7f]';
                $domain_literal = "\\x5b($dtext|$quoted_pair)*\\x5d";
                $quoted_string = "\\x22($qtext|$quoted_pair)*\\x22";
                $domain_ref = $atom;
                $sub_domain = "($domain_ref|$domain_literal)";
                $word = "($atom|$quoted_string)";
                $domain = "$sub_domain(\\x2e$sub_domain)*";
                $local_part = "$word(\\x2e$word)*";
                $addr_spec = "$local_part\\x40$domain";
                return preg_match("!^$addr_spec$!", $email) ? 1 : 0;
    }

    /*!
     * @param $ip ip address to validate
     * @returns boolean runs a sanity check on an IP address to see if its valid
     * @author Peter Chung <peterc@linuxmagic.com>
     */
    function isValidIP($ip) {
        $val = trim($ip);
        // we test if its a valid ip .. we do NOT support ranges for ip additions...
        if (!filter_var($val, FILTER_VALIDATE_IP)) {
            return FALSE;
        }
        return TRUE;
    }

    //! Gathers the entries and counts them
    /*
     * @return sets the entries and entry_count
     *
     * @author Robert Bisewski <robertb@linuxmagic.com>
     */
    function setEntryData() {

        // Creates a list manager
        $list_mgr = ListManager::getInstance();
        $this->entries = array();
        $this->entry_count = 0;
        $data = array();

        // IF this is an IP based set
        if ($this->entry_type == self::ENTRY_IP && $this->name != 'ip_rl') {
            // we first want to load the ranges
            $ret = $list_mgr->load(strtoupper($this->cmd_flag), $this->getFolder(), $data, FALSE);
            if ($ret) {
                $this->entry_count = count($data);
                $this->entries = $data;
            }
            // and reset back for regular
            $data = array();
        }

        $ret = $list_mgr->load($this->cmd_flag, $this->getFolder(), $data, FALSE);
        if ($ret) {
            $this->entry_count += count($data);

            // convert the country code to country name
            if ($this->entry_type == self::ENTRY_COUNTRY) {
                $country = array();
                foreach ($data as $d => $c) {
                    $country[Country::getCountryNameFromCode($d)] = $c;
                }
                $data = $country;
                ksort($data);
            }

            $this->entries = array_merge($this->entries, $data);
        }
    }

    function getEntryData() {
        // entry data is empty by default
        $data = array();

        // load and return the entry data
        $list_mgr = ListManager::getInstance();
        $list_mgr->load($this->cmd_flag, $this->getFolder(), $data);
        return $data;
    }

    //! Retrieves the number of entries of a given exemption list
    /*
     * @return int # of entries
     *
     * @author Robert Bisewski <robertb@linuxmagic.com>
     */
    function getEntryCount() {
        return $this->entry_count;
    }

    //! retrieve the migratable configurations of the module
    /*!
     * @returns array  the migratable configurations
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function getMigrationData() {
        // the list of migratable configurations
        $configuration = array();

        // retreive all entries for the exemption
        $this->setEntryData();

        // process and store the entries
        foreach ($this->entries as $entry => $comment) {
            // process the entry
            if ($this->entry_type == self::ENTRY_COUNTRY) {
                $entry = Country::getCountryCodeFromName($entry);
            } else {
                $entry = preg_replace('/\s+/', '', $entry);
            }

            // clean up the comment
            $comment = trim($comment);

            // store the entry and comment
            $configuration[] = array(
                "entry" => $entry,
                "comment" => $comment,
            );
        }

        // return the migration configurations
        return $configuration;
    }

    //! loads the migratable configurations of the module
    /*!
     * @param   array  the configurations to load
     *
     * @returns bool   whether the configurations were loaded successfully
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function loadMigrationData($configurations) {
        // sanity check
        if (!isset($configurations) || !WV::isArrayNotEmpty($configurations)) {
            return FALSE;
        }

        // stores entry validation error message
        $msg = "";

        // load each entry in the configurations
        foreach ($configurations as $configuration) {
            // sanity check
            if (!isset($configuration['entry'])) {
                continue;
            }

            // assign friendly name
            $entry = $configuration['entry'];
            $comment = isset($configuration['comment']) ? $configuration['comment'] : "";

            // skip invalid entries
            if (!$this->validateEntry($entry, $msg)) {
                continue;
            }

            // skip user rate limit configuration when not professional
            if (!isProfessional() && $this->name == "user_rl") {
                continue;
            }

            // cast the flag to upper case indicating the range argument counterpart if necessary
            $lst_flag = $this->cmd_flag;
            if ($this->entry_type == self::ENTRY_IP && strpos($entry, '-') != FALSE) {
                $lst_flag = strtoupper($lst_flag);
            }

            // add the entry to the list
            $list_mgr = ListManager::getInstance();
            $args = array($lst_flag, '-a', $entry, '-h', $this->getFolder());
            $list_mgr->add($args);

            // add the comment for the entry to the list if necessary
            if (WV::isStringNotEmpty($comment)) {
                $args[] = '-c';
                $args[] = $comment;
                $list_mgr->add($args);
            }
        }

        // success, migratable configurations successfully loaded
        return TRUE;
    }
}
