<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Peter Chung <peterc@linuxmagic.com>
 *            Shaun A. Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.listmanager.php 10395 2019-09-16 18:11:15Z deiony $
 *
 * This is a container to access license information.
 * This class allows for the reading of License information in conjunction with
 * information
 * pulled from license manager based on the license code.  Licenses are
 * described using 5 variables:
 * $state, $code, $expiry, $nextupdate, $requester.
 */

include_once('lib.cmdwrapper.php');
require_once('wizard/lib.wizard_validator.php');

class ListManager extends cmdWrapper {
    static private $object;
    const FLAG_ERROR             =  -1;
    const INVALID_FLAG           =  -2;
    const ENTRY_EXISTS           =  -3;
    const ENTRY_NOT_FOUND        =  -4;
    const ENTRY_EXISTS_ELSEWHERE =  -5;
    const UNKNOWN_ERROR          = -10;

    // internal private singleton.
    //! class constructor
    /*!
     * we want this class type to be a singleton, so we deny access to the constructor
     */
    protected function __construct() {
        // instantiate back to the parent.
        parent::__construct(LIST_MANAGER);
        // initialize the data
    }

    //! static function to get the current singleton instance
    /*!
     * @returns LicenseManager  the active singleton instance.
     *
     * If the instance is not yet created, then this will create it.
     */
    static public function getInstance() {
        if (is_null(self::$object)) {
            self::$object = new ListManager;
        }
        return self::$object;
    }


    //! access function to 'load' the list data.
    /*!
     * @param array $args The arguments to load using. Typically this
     *   will include a flag only.
     * @param array &$data Where to store the results
     *
     * @returns FLAG_ERROR for parameter errors or invalid flag errors
     *          INVALID_FLAG for invalid flag errors
     *          TRUE for success.
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     *
     * Any errors that occur will store a message that can be retrieved
     * by a call to 'error()' function.
     */
    public function load($list, $home, &$data, $ignore_comment = TRUE) {
        // sanity check
        if (!WV::isStringNotEmpty($list) || !WV::isStringNotEmpty($home)) {
            return self::FLAG_ERROR;
        }

        // initialize the data buffer
        $data = array();

        // initialize the command line arguments
        $args = array(
            "-l",
            $list,
            "-h", $home,
        );

        // execute the list manager program
        $buffer = "";
        $rv = $this->execRead($args, $buffer);

        // the list has no entries, nothing more to do
        if ($buffer == "") {
            return TRUE;
        }

        // parse the output into entry lines
        $lines = explode("\n", $buffer);

        // ensure that the entries were loaded correctly
        if ($rv !== 0) {
            if (preg_match("/invalid option/", $lines[0])) {
                $this->lastError = $rv;
                return self::INVALID_FLAG;
            }
            return self::UNKNOWN_ERROR;
        }

        // trim and store each entry line in the data buffer
        $entry = NULL;
        $comment = NULL;
        foreach ($lines as $line) {
            if (trim($line) === "") {
                continue;
            }

            if (!is_null($entry) && $line[0] !== "\t") {
                if ($ignore_comment) {
                    $data[] = $entry;
                } else {
                    $data[$entry] = $comment;
                }
                $entry = NULL;
                $comment = NULL;
            }

            if ($line[0] !== "\t") {
                $entry = $line;
            } else {
                $comment = trim($line);
            }
        }
        if ($ignore_comment) {
            $data[] = $entry;
        } else {
            $data[$entry] = $comment;
        }
        return TRUE;
    }

    //! access function to 'add' a list item.
    /*!
     * @param array $args The command parameters to run.
     *
     * @returns FLAG_ERROR for parameter errors or invalid flag errors
     *          ENTRY_EXISTS when adding an entry that is pre existing
     *          ENTRY_EXISTS_ELSEWHERE if currently exists in opposite table
     *          TRUE for success.
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     *
     * NOTES: Adding of any duplicates will result in an error of ENTRY_EXISTS
     */
    public function add($args) {
        if (!isset($args) || !is_array($args) || count($args) === 0 ||
            strlen($args[0]) < 2) {
            return self::FLAG_ERROR;
        }
        $buffer = "";
        $rv = $this->execRead($args, $buffer);
        $linearray = explode("\n", $buffer);
        foreach ($linearray as $i => $line) {
            trim($line);
            if ($i == 0 && strlen($line) > 0) {
                if (preg_match("/Entry already exists/", $line)) {
                    return self::ENTRY_EXISTS;
                } else if (preg_match("/Entry is present in the opposite table/", $line)) {
                    return self::ENTRY_EXISTS_ELSEWHERE;
                } else {
                    $this->lastError = $buffer;
                    return self::UNKNOWN_ERROR;
                }
            }
        }
        return TRUE;
    }

    //! access function to 'delete' a list item.
    /*!
     * @param array $args The command parameters to run.
     *
     * @returns FLAG_ERROR for parameter errors or invalid flag errors
     *          ENTRY_NOT_FOUND if the entry is not present
     *          TRUE for success.
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     *
     * Any errors that occur will store a message that can be retrieved
     * by a call to 'error()' function.
     */
    public function delete($args) {
        if (!isset($args) || !is_array($args) || count($args) === 0 ||
            strlen($args[0]) < 2) {
            return self::FLAG_ERROR;
        }
        $buffer = "";
        $rv = $this->execRead($args, $buffer);
        $linearray = explode("\n", $buffer);
        foreach ($linearray as $i => $line) {
            trim($line);
            if ($i == 0 && strlen($line) > 0) {
                if( preg_match("/Entry not found/", $line)) {
                    
                    // TODO: Add a proper error message here!
                    $this->lastError = "Error: Entry index could not be " . 
                                       "found. Please contact Support.";
                    
                    // Return our error code
                    return self::ENTRY_NOT_FOUND;
                } else {
                    $this->lastError = $rv;
                    return self::UNKNOWN_ERROR;
                }
            }
        }
        return TRUE;
    }
}
