<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *            Sean West <sean@linuxmagic.com>
 *            Shaun A Johnson <shaun@linuxmagic.com>
 *            Peter Chung <peterc@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *            Deion Yu <deiony@linuxmagic.com>
 *
 * $Id: lib.logsearch.php 11866 2020-07-08 00:08:11Z deiony $
 */
require_once('lib.logsearchbase.php');

/*
 * Each of the columns contain content which is stored in specific
 * indices. If you change the order of the columns you will need to
 * adjust the below values to the new order.
 * NOTE: the components of log line by numeric index:
 *       0 -> datetime      e.g. 2014-12-04 14:00:51
 *       1 -> type          e.g. HAM, SPAM, EXEMPT, ...
 *       2 -> rule          e.g. from_blacklist, check_reverse_dns, ""
 *       3 -> mua           e.g. yes/no/0/1
 *       4 -> ip            e.g. 192.168.1.1
 *       5 -> rdns          e.g. build-03.linuxmagic.dev
 *       6 -> country code  e.g. CA, **, (null)
 *       7 -> helo argument e.g. test.linuxmagic.com
 *       8 -> mail from     e.g. will@linuxmagic.dev
 *       9 -> rcpt to       e.g. test@linuxmagic.dev
 */
define('INDEX_DATETIME', 0);
define('INDEX_TYPE',     1);
define('INDEX_RULE',     2);
define('INDEX_MUA',      3);
define('INDEX_AUTHUSER', 4);
define('INDEX_IP',       5);
define('INDEX_HOSTNAME', 6);
define('INDEX_COUNTRY',  7);
define('INDEX_HELO',     8);
define('INDEX_SENDER',   9);
define('INDEX_RECIP',   10);

//! these defines are used to uniquely identify if you are filtering
// the logs to specific types.
define('FILTER_ALL',        0);
define('FILTER_HAM',        1);
define('FILTER_SPAM',       2);
define('FILTER_EXEMPT',     3);
define('FILTER_RATELIMIT',  4);

// the filter to specify email authentication status
define('FILTER_MUA_BOTH',   0);
define('FILTER_MUA_YES',    1);
define('FILTER_MUA_NO',     2);

//! regex pattern for log lines
// the new log line format was added to handle authenticated user data
define('LS_PATTERN_FMT'       , '/^(%s %s) magicspam\-[a-zA-Z0-9]+\[\d+\]: (%s)(?:\[(.+)\])?: mua=\[(%s):(%s)\],ip=\[(%s):([^:]+)(?::([A-Z*]{2}))?\],helo=\<(.+)\>,from=\<(%s)\>,rcpt=<(%s)\>$/i');
// the old log line format was added to handle country code data
define('LS_PATTERN_OLD_FMT'   , '/^(%s %s) magicspam\-[a-zA-Z0-9]+\[\d+\]: (%s)(?:\[(.+)\])?: mua=(%s),ip=\[(%s):([^:]+):([A-Z*]{2})\],helo=\<(.+)\>,from=\<(%s)\>,rcpt=<(%s)\>$/i');
// the legacy log line format from times immemorial
define('LS_PATTERN_LEGACY_FMT', '/^(%s %s) magicspam\-[a-zA-Z0-9]+\[\d+\]: (%s)(?:\[(.+)\])?: mua=(%s),ip=\[(%s):([^:]+)\],helo=\<(.+)\>,from=\<(%s)\>,rcpt=<(%s)\>$/i');

class LogSearch extends LogSearchBase
{
    //! string. sender address search filter string.
    protected $sender;

    //! string. recipient address search filter string.
    protected $recipient;

    //! string. authenticated user search filter string.
    protected $auth_user;

    //! string. IP address search filter string.
    protected $ip_address;

    //! integer. the filter type to apply to the results. this is one
    // of the FILTER_* defines.
    protected $filter;

    //! integer. the mua filter type, one of the FILTER_MUA_XXX defines
    protected $mua;

    //! integer the hour to search for the hourly log search
    protected $hour;

    //! integer whether to use yesterday or today log in hourly log search
    protected $yesterday;

    //! the old log line regex
    protected $log_line_old_regex;

    //! the legacy log line regex
    protected $log_line_legacy_regex;

    //! class constructor
    public function __construct() {
        parent::__construct();
        $this->sender = NULL;
        $this->recipient = NULL;
        $this->auth_user = NULL;
        $this->ip_address = NULL;
        $this->filter = FILTER_ALL;
        $this->mua = FILTER_MUA_BOTH;
        $this->hour = NULL;
        $this->yesterday = NULL;
        $this->log_line_old_regex = NULL;
        $this->log_line_legacy_regex = NULL;
    }

    //! parse log line into components according to log line regex (overloaded)
    /*!
     * @param   string          the log line to parse
     *
     * @returns mixed           the array of components, FALSE on error
     *
     * NOTE: overloaded function to handle legacy log line format
     */
    protected function parseLogLine($line) {
        // ensure that the log line is a string
        if (!WV::isString($line)) {
            return FALSE;
        }

        // trim the log line and ignore when empty
        $line = trim($line);
        if (!strlen($line)) {
            return FALSE;
        }

        // perform regex match and standardize the matches to the standard format
        if (preg_match($this->log_line_regex, $line, $matches) == 1) {
            // set to catch-all country code when country code is empty
            // NOTE: log lines from the Lite version will generate log lines
            //       with the authenticated user but not the country code
            if ($matches[INDEX_COUNTRY + 1] == "") {
                $matches[INDEX_COUNTRY + 1] = "**";
            }
        } elseif (preg_match($this->log_line_old_regex, $line, $matches) == 1) {
            // convert the matches from old to standard format by inserting
            // the unknown auth user to the expected position
            array_splice($matches, INDEX_AUTHUSER + 1, 0, array('(unknown)'));
        } elseif (preg_match($this->log_line_legacy_regex, $line, $matches) == 1) {
            // convert the matches from legacy to standard format by inserting
            // the unknown auth user and catch-all country code to the expected
            // position
            array_splice($matches, INDEX_AUTHUSER + 1, 0, array('(unknown)'));
            array_splice($matches, INDEX_COUNTRY + 1, 0, array('**'));
        } else {
            // none of the patterns matched, the log line is malformed
            return FALSE;
        }

        // remove the first capture group as it contains the entire log line
        array_shift($matches);

        // exclude log lines with old and legacy log line formats when filtering
        // by authenticated user
        if (isset($this->auth_user) && $matches[INDEX_AUTHUSER] == "(unknown)") {
            return FALSE;
        }

        // return the rest of the capture groups
        return $matches;
    }

    //! find all the log files within the range of days (overloaded)
    /*!
     * @param   string          the base name of the log file
     * @param   integer         the start of the range of days
     * @param   integer         the end of the range of days
     *
     * @returns mixed           array of log files within the range, FALSE on error
     *
     * NOTE: overloaded function to handle hourly log search from the dashboard
     */
    protected function findLogFilesRange($logFile, $start, $end) {
        if (!isset($this->yesterday)) {
            return parent::findLogFilesRange($logFile, $start, $end);
        }

        if ($this->yesterday == 0) {
            $files[] = $logFile;
        } elseif ($this->yesterday == 1) {
            $files[] = $logFile . '.1';
        }
        return $files;
    }

    //! create the pattern for searching a log file
    protected function createLogSearchPattern() {
        $date_regex = '\d{4}\-\d+\-\d+';

        $time_regex = '\d+:\d+:\d+';
        if (isset($this->hour)) {
            $time_regex = sprintf('%02d:\d+:\d+', $this->hour);
        }

        $mua_regex = $this->getMuaRegex();

        $type_regex = $this->getFilterRegex();

        $from_regex = '.*';
        if (isset($this->sender)) {
            $from_regex = preg_replace('/\*/', '.*', $this->sender);
        }

        $to_regex = '.*';
        if (isset($this->recipient)) {
            $to_regex = preg_replace('/\*/', '.*', $this->recipient);
        }

        $authuser_regex = '.*';
        if (isset($this->auth_user)) {
            $authuser_regex = preg_replace('/\*/', '.*', $this->auth_user);
        }

        $ip_regex = '.+?';
        if (isset($this->ip_address)) {
            // replace dot characters from user input with literal dots
            $ip_regex = preg_replace('/\./', '\.', $this->ip_address);
            // replace wildcard character from user input with the equivalent regex
            $ip_regex = preg_replace('/\*/', '.*', $ip_regex);
            // replace the quantifier ending the regex with the non-greedy version
            $ip_regex = preg_replace('/([\*\+])$/', '\1?', $ip_regex);
        }

        $this->log_line_regex = sprintf(LS_PATTERN_FMT,
            $date_regex,
            $time_regex,
            $type_regex,
            $mua_regex,
            $authuser_regex,
            $ip_regex,
            $from_regex,
            $to_regex
        );

        $this->log_line_old_regex = sprintf(LS_PATTERN_OLD_FMT,
            $date_regex,
            $time_regex,
            $type_regex,
            $mua_regex,
            $ip_regex,
            $from_regex,
            $to_regex
        );

        $this->log_line_legacy_regex = sprintf(LS_PATTERN_LEGACY_FMT,
            $date_regex,
            $time_regex,
            $type_regex,
            $mua_regex,
            $ip_regex,
            $from_regex,
            $to_regex
        );

        return TRUE;
    }

    //! create the pattern for skipping a log file based on date (overloaded)
    protected function createDatePattern() {
        $this->date_regex = '/^(\d{4}\-\d+\-\d+)/';
        return TRUE;
    }

    //! retrieve email country statistics
    /*!
     * @param   int             number of log days to parse (0 = current day)
     * @param   int             number of countries to be displayed
     *
     * @returns array           array with all the country stats
     *
     * @author  Lucas Nincao <lucasn@linuxmagic.com>
     *
     * NOTE: returns inbound and outbound country statistics in the array
     */
    public function getTopEmailStatsByCountryRange($start = 0, $stop = 0, $numSelections = 5)
    {
        // mslog files dir
        $baseFile = LOGDIR . "mslog";

        // associtative arrays to store all the email stats by country
        // initialize data structure for email stats by country
        $emailStats = array(
            'INBOUND'  => array(
                'ALL' => array(),
                'HAM' => array(),
                'SPAM' => array(),
                'EXEMPT' => array(),
                'RATELIMIT' => array()
            ),
            'OUTBOUND' => array(
                'ALL' => array(),
                'HAM' => array(),
                'SPAM' => array(),
                'EXEMPT' => array(),
                'RATELIMIT' => array()
            )
        );

        // find all the desired mslog files
        $files = $this->findLogFilesRange($baseFile, $start, $stop);
        if ($files === FALSE) {
            return FALSE;
        }

        // create the log search regex pattern
        $this->createLogSearchPattern();

        // loop through each eligible magicspam log file
        foreach ($files as $file) {
            // open the log file with the reverse reader
            $reader = new ReverseReader($file, $this->read_buffer_size);

            // read in each line from the file in the reverse order
            while (($line = $reader->readLine()) !== FALSE) {

                // parse the log line into its individual components
                $row = $this->parselogline($line);
                if ($row === false) {
                    continue;
                }

                // assign friendly names for easier manipluation
                $countryCode = $row[INDEX_COUNTRY];
                $emailType = $row[INDEX_TYPE];
                $mua = $row[INDEX_MUA];

                // convert mua type to country email stats base key
                $authStatus = ($mua === "yes") ? "OUTBOUND" : "INBOUND";

                // keep count of countries based on the email type and mua
                // NOTE: email type will either be ham, spam, exempt, ratelimit
                $this->incrementStat($emailStats[$authStatus]['ALL'], $countryCode, FALSE);
                $this->incrementStat($emailStats[$authStatus][$emailType], $countryCode, FALSE);
            }

            unset($reader);
        }

        // sort and splice both arrays
        $this->prepareMultiArrayResults($emailStats['INBOUND'], $numSelections);
        $this->prepareMultiArrayResults($emailStats['OUTBOUND'], $numSelections);

        return $emailStats;
    }

    public function getTopEmailStatsByCountry($numSelections = 5) {
        return $this->getTopEmailStatsByCountryRange(0, 0, $numSelections);
    }

    public function getTopStatsRange($start = 0, $stop = 0, $numSelections = 5, $options = array())
    {
        // for the time being, this function is only meant for the mslog files
        $baseFile = LOGDIR . "mslog";

        // initialize the associative array of statistics
        $statistic = array(
            "spam_rules"      => array(),
            "ip_reputation"   => array(),
            "sender"          => array(),
            "receiver"        => array(),
            "sender_domain"   => array(),
            "receiver_domain" => array(),
            "ratelimit"       => array(),
            "ratelimit_ptr"   => array(),
        );

        // find all the desired mslog files
        $files = $this->findLogFilesRange($baseFile, $start, $stop);
        if ($files === FALSE) {
            return FALSE;
        }

        // create the log search regex pattern
        $this->createLogSearchPattern();

        // loop through each eligible magicspam log file
        foreach ($files as $file) {
            // open the log file with the reverse reader
            $reader = new ReverseReader($file, $this->read_buffer_size);

            // read in each line from the file in the reverse order
            while (($line = $reader->readLine()) !== FALSE) {
                // parse the log line into its individual components
                $row = $this->parseLogLine($line);
                if ($row === FALSE) {
                    continue;
                }

                // ignore non HAM rows when the onlyHAM option is turned ON
                if (isset($options['onlyHAM'])) {
                    if ($options['onlyHAM'] === TRUE && $row[INDEX_TYPE] !== "HAM") {
                        continue;
                    }
                }

                // ignore all rows that do not have the specified MUA
                if (isset($options['muaOnly'])) {
                    if (($options['muaOnly'] === TRUE && $row[INDEX_MUA] !== 'yes') ||
                        ($options['muaOnly'] === FALSE && $row[INDEX_MUA] !== 'no')) {
                        continue;
                    }
                }

                // specific handling for the spam and quarantine log lines
                if ($row[INDEX_TYPE] === "SPAM" || $row[INDEX_TYPE] === "QUARANTINE") {
                    // skip the row if the spam rule is missing
                    if (empty($row[INDEX_RULE])) {
                        continue;
                    }

                    // start off by assuming that this is not a RBL or BMS spam rule
                    $rule_name = $row[INDEX_RULE];
                    $list_string = "";

                    // retrieve the lists from RBL and BMS spam rules
                    if (preg_match("/^(\S+)\s+\\((\S+)\\)*$/", $rule_name, $matches) === 1) {
                        $rule_name = $matches[1];
                        $list_string = trim($matches[2], "()");
                    }

                    // increment the count of the spam rule
                    $this->incrementStat($statistic['spam_rules'], "ts_$rule_name");

                    // handle the list from RBL and BMS spam rules if any
                    if (strlen($list_string)) {
                        // split into individual lists
                        $lists = explode(",", $list_string);
                        foreach ($lists as $list) {
                            // ignore empty lists
                            $list = trim($list);
                            if (!strlen($list)) {
                                continue;
                            }
                            // BMS lists are numeric, append the BMS list identifier
                            if (is_numeric($list)) {
                                $list = "ts_bms$list";
                            }
                            // increment the count of the list
                            $this->incrementStat($statistic['ip_reputation'], $list);
                        }
                    }

                } elseif ($row[INDEX_TYPE] === "RATELIMIT") {
                    // increment the count of the ratelimited IP
                    $this->incrementStat($statistic['ratelimit'], $row[INDEX_IP]);

                    // keep track of ratelimited IPs with their PTR records when passed the ratelimitPTR option
                    if (isset($options['ratelimitPTR']) && $options['ratelimitPTR'] === TRUE) {
                        if (!isset($statistic['ratelimit_ptr'][$row[INDEX_IP]])) {
                            $this->incrementStat($statistic['ratelimit_ptr'], $row[INDEX_IP] . " (" . $row[INDEX_HOSTNAME] . ")");
                        }
                    }
                }

                // increment the count of the the sender
                $this->incrementStat($statistic['sender'], $row[INDEX_SENDER]);

                // increment the count of the the sender domain
                $sender_domain = explode("@", $row[INDEX_SENDER]);
                if (!empty($sender_domain) && is_array($sender_domain) && count($sender_domain) == 2) {
                    $this->incrementStat($statistic['sender_domain'], $sender_domain[1]);
                }

                // increment the count of the the receiver
                $this->incrementStat($statistic['receiver'], $row[INDEX_RECIP]);

                // increment the count of the the receiver domain
                $receiver_domain = explode("@", $row[INDEX_RECIP]);
                if (!empty($receiver_domain) && is_array($receiver_domain) && count($receiver_domain) == 2) {
                    $this->incrementStat($statistic['receiver_domain'], $receiver_domain[1]);
                }
            }
            unset($reader);
        }

        array_walk($statistic, array($this, 'prepareArrayResults'), $numSelections);
        return $statistic;
    }

    //! retrieves the top senders / recipients
    /*!
     * @returns  array    the highest from high to low, for both senders
     *                    and recips, and for both addresses and domains
     *
     * @author Robert Bisewski <robertb@linuxmagic.com>
     *
     * NOTE: Used by the 'stats' module only - FLAG TO MOVE TO DERIVED CLASS
     */
    public function getTopStats($numSelections = 5, $options = array()) {
        return $this->getTopStatsRange(0, 0, $numSelections, $options);
    }

    protected function incrementStat(&$stats, $stat, $lowercase = TRUE) {
        $stat_processed = $lowercase ? strtolower($stat) : $stat;
        $stats[$stat_processed] = isset($stats[$stat_processed]) ? $stats[$stat_processed] + 1 : 1;
    }

    protected function prepareArrayResults(&$stats, $stat_type, $numSelections) {
        arsort($stats);
        $stats = array_slice($stats, 0, $numSelections);
    }

    //! Sorts array by values and keys and slices to the requested number of entries
    /*!
     * @param   array           the multi array
     * @param   int             the requested number of entries
     *
     * @author  Lucas Nincao <lucasn@linuxmagic.com>
     *
     * NOTE: the array is sorted by values in descending order followed by
     *       keys in the ascending order
     */
    protected function prepareMultiArrayResults(&$arr, $num) {
        foreach ($arr as $key => $data) {
            if (!empty($data)) {
                array_multisort(array_values($arr[$key]), SORT_DESC,
                    array_keys($arr[$key]), SORT_ASC,
                    $arr[$key]);
                $arr[$key] = array_slice($arr[$key], 0, $num);
            }
        }
    }

    //! converts the filter type to the filter regex
    protected function getFilterRegex() {
        switch($this->filter) {
            case FILTER_HAM:
                $regex = "HAM|NDR|EXEMPT";
                break;
            case FILTER_SPAM:
                $regex = "SPAM|QUARANTINE";
                break;
            case FILTER_EXEMPT:
                $regex = "EXEMPT";
                break;
            case FILTER_RATELIMIT:
                $regex = "RATELIMIT";
                break;
            case FILTER_ALL:
            default:
                $regex = "HAM|NDR|EXEMPT|SPAM|QUARANTINE|RATELIMIT";
                break;
        }
        return $regex;
    }

    //! converts the filter type to the filter regex
    protected function getMuaRegex() {
        switch($this->mua) {
            case FILTER_MUA_YES:
                $regex = "yes";
                break;
            case FILTER_MUA_NO:
                $regex = "no";
                break;
            case FILTER_MUA_BOTH:
            default:
                $regex = "yes|no";
                break;
        }
        return $regex;
    }

    //! setter for sender.
    public function setSender($sender) {
        // ensure the sender is a non-empty string
        if (!isset($sender) || !WV::isStringNotEmpty($sender)) {
            return FALSE;
        }
        // trim and check to ensure the sender was not just whitespace
        $sender = trim($sender);
        if ($sender === '') {
            return FALSE;
        }
        // set the sender class variable
        $this->sender = $sender;
        return TRUE;
    }

    //! setter for recipient.
    public function setRecipient($recipient) {
        // ensure the recipient is a non-empty string
        if (!isset($recipient) || !WV::isStringNotEmpty($recipient)) {
            return FALSE;
        }
        // trim and check to ensure the recipient was not just whitespace
        $recipient = trim($recipient);
        if ($recipient === '') {
            return FALSE;
        }
        // set the recipient class variable
        $this->recipient = $recipient;
        return TRUE;
    }

    //! setter for auth user
    public function setAuthUser($auth_user) {
        // ensure the auth user is a non-empty string
        if (!isset($auth_user) || !WV::isStringNotEmpty($auth_user)) {
            return FALSE;
        }
        // trim and check to ensure the auth user was not just whitespace
        $auth_user = trim($auth_user);
        if ($auth_user === '') {
            return FALSE;
        }
        // set the auth user class variable
        $this->auth_user = $auth_user;
        return TRUE;
    }

    //! setter for recipient.
    public function setIpAddress($ip_address) {
        // ensure the ip address is a non-empty string
        if (!isset($ip_address) || !WV::isStringNotEmpty($ip_address)) {
            return FALSE;
        }
        // trim and check to ensure the ip address was not just whitespace
        $ip_address = trim($ip_address);
        if ($ip_address === '') {
            return FALSE;
        }
        // set the ip address class variable
        $this->ip_address = $ip_address;
        return TRUE;
    }

    //! setter for hour
    public function setHour($hour) {
        // ensure the hour is a positive numeric
        if (!isset($hour) || !WV::isIntPositiveOrZero($hour) || $hour > 23) {
            return FALSE;
        }
        // set the hour class variable
        $this->hour = $hour;
        return TRUE;
    }

    //! setter for yesterday
    public function setYesterday($yesterday) {
        // ensure the yesterday is a positive numeric
        if (!isset($yesterday) || !WV::isIntPositiveOrZero($yesterday)) {
            return FALSE;
        }
        // set the yesterday class variable
        $this->yesterday = $yesterday;
        return TRUE;
    }

    //! setter for filter
    public function setFilter($filter) {
        // ensure the filter is a positive numeric
        if (!isset($filter) || !WV::isIntPositiveOrZero($filter)) {
            return FALSE;
        }
        // convert numeric string into integer
        $filter = intval($filter);
        // ensure that the filter type is known
        if ($filter !== FILTER_ALL &&
            $filter !== FILTER_HAM &&
            $filter !== FILTER_SPAM &&
            $filter !== FILTER_EXEMPT &&
            $filter !== FILTER_RATELIMIT) {
            return FALSE;
        }
        // set the filter class variable
        $this->filter = $filter;
        return TRUE;
    }

    //! setter for mua
    public function setMua($mua) {
        // ensure the mua is a positive numeric
        if (!isset($mua) || !WV::isIntPositiveOrZero($mua)) {
            return FALSE;
        }
        // convert numeric string into integer
        $mua = intval($mua);
        // ensure that the mua type is known
        if ($mua !== FILTER_MUA_BOTH &&
            $mua !== FILTER_MUA_YES &&
            $mua !== FILTER_MUA_NO) {
            return FALSE;
        }
        // set the mua class variable
        $this->mua = $mua;
        return TRUE;
    }
}
