<?php
/* vim:et:sw=4:ts=4
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2008-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc. 
 *
 * The Original Code is distributed on an 'AS IS' basis, 
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC 
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET 
 * ENJOYMENT OR NON-INFRINGEMENT. 
 *
 * Do NOT download, distribute, use or alter this software or file in any 
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.modulecontainer.php 10940 2020-01-14 00:44:10Z deiony $
 */

/* vim:et:sw=4:ts=4
 *
 * This is the base definition for all top level modules in MagicSpam.
 * This class must never be used directly but rather be extended to define a
 * working whole.
 *
 * This file is part of the MagicSpam(TM) Project and is
 * Copyright (c) 2008-2011 LinuxMagic Inc. All Rights Reserved.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.modulecontainer.php 10940 2020-01-14 00:44:10Z deiony $
 */
require_once('lib.module.php');

abstract class ModuleContainer extends Module {
    //! variable to hold the active page
    protected $activepage;

    //! variable to hold the active submodule
    protected $activesubmodule;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @param   boolean whether or not the module should render a menu link.
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l, $m, $submodules = array()) {
        parent::__construct($l, $m);
        $this->submodules = $submodules;
        $this->activepage = NULL;
        $this->activesubmodule = NULL;

        // set the active loaded page and submodule if any
        $this->initActivePage();
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $modules;

        // initialize the active page as being the first 'page'
        $modkeys = array_keys($this->submodules);
        $this->activepage = $modkeys[0];

        // do we have a specific page we need active?
        $this->initActivePage();
        
        // now we need pass the action along...
        $page = $this->submodules[$this->activepage];

        // we load the module if it doesn't exist yet...
        if (!isset($modules[$page])) {
            loadModule($page);
        }
        // now we need a handle to the object
        $loadedMod = $modules[$page];
      
        // Since the rate-limiter page has a complex heirarchy of submodules, 
        // this code grabs the submodule that was recently POST
        if (isset($_POST['page']) 
            && strlen($_POST['page']) > 0 
            && $_POST['page'] == 'rate_limiter' 
            && isset($_POST['submodule']) 
            && strlen($_POST['submodule']) > 0) {

            // Loads submodule if not currently loaded
            if (!isset($modules[$_POST['submodule']])) {
               loadModule($_POST['submodule']);
            }

            // Grabs the needed submodule for setting values
            $loadedMod = $modules[$_POST['submodule']];
        }

        // and we pass the action down the line.
        return $loadedMod->actionProcessor($action, $errors, $notices);
    }
    
    //! method to process any actions and react accordingly.
    /*!
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    protected function initActivePage() {
        if (empty($this->submodules)) {
            return;
        }

        $modkeys = array_keys($this->submodules);

        // set the active page as the first submodule in the array
        $this->activepage = $modkeys[0];

        // set activepage to the page (submodule) being posted
        // if valid and is in the submodules array
        if (isset($_POST['page'])
            && strlen($_POST['page']) > 0
            && array_key_exists($_POST['page'], $this->submodules) ) {
            $this->activepage = $_POST['page'];
        }

        // this variable is also the active page, but it uses
        // the submodule value from associative array 
        $this->activesubmodule = isset($this->submodules[$this->activepage]) ?
                                 $this->submodules[$this->activepage] : NULL;
    }

    //! method to render the full page module.
    /*!
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $modules;

        // make sure we know what page we are rendering'
        $this->initActivePage();

        // render the master menu
        renderMenu();
        // render the submenu
        $this->renderSubMenu();
        
        // and render the active page
        $page = $this->submodules[$this->activepage];
       
        // load the module if it isn't yet...
        if (!isset($modules[$page])) {
            loadModule($page);
        }
        
        // grab the handle
        $loadedMod = $modules[$page];
        // and render it off
        return $loadedMod->renderModule();
    }

    //! method to render the sub menu
    /*!
     * @author  Shaun Johnson <shaun@linuxmagic.com>
     */
    private function renderSubMenu() {
        global $module;
        if (sizeof($this->submodules) == 0) {
            return;
        }
        // we render a 'mini' form for quick page nav.
        echo "<form name=\"submenuform\" method=\"post\">\n";
        echo "<input type=\"hidden\" name=\"page\" value=\"\"/>\n";
        echo "<input type=\"hidden\" name=\"module\" value=\"{$module}\"/>\n";
        echo "</form>\n";
        echo "<div class=\"submenu\">\n<div class=\"submenuLeft\"></div><div class=\"submenuCenter\"><ul>\n";

        $sCount = 1;
        foreach ($this->submodules as $lbl => $mod) {
            echo "<li";
            if ($lbl == $this->activepage) {
                echo " class=\"active\"";
            }
            $params = "{'hiddenFields':{'module':'{$module}','page':'{$lbl}'}}";

            if ($sCount == count($this->submodules)) { 
                echo "><a href=\"#\" class=\"nodivider\" onclick=\"doFormSubmit({$params});\">" . intl($lbl) . "</a></li>\n";
            } else {
                echo "><a href=\"#\" onclick=\"doFormSubmit({$params});\">" . intl($lbl) . "</a></li>\n";
            }
            $sCount++;
        }
        echo "</ul></div><div class=\"submenuRight\"></div></div>\n";
    }

    //! internal utility function for merging 'sub' modules CSS (overloaded)
    /*!
     * @returns array   an array of the CSS files to include as dictated by submodules
     *
     * @author  Lucas Nincao <lucasn@linuxmagic.com>
     */
    protected function subModuleCSS($submodule = NULL) {
        return parent::subModuleCSS($this->activesubmodule);
    }

    //! internal utility function for merging 'sub' modules JS (overloaded)
    /*!
     * @returns array   an array of the JS files to include as dictated by submodules
     *
     * @author  Lucas Nincao <lucasn@linuxmagic.com>
     */
    protected function subModuleJS($submodule = NULL) {
        return parent::subModuleJS($this->activesubmodule);
    }

}
