<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: lib.rcptcounter.php 11866 2020-07-08 00:08:11Z deiony $
 *
 * RCPTCounter is a simple method for obtaining stored data
 * counts from the magicspam engine.
 */

//! definitions for numeric index identifiers within the rcptcounter format.
define('RCPT_TYPE_TOTAL',          0);
define('RCPT_TYPE_HAM',            1);
define('RCPT_TYPE_SPAM',           2);
define('RCPT_TYPE_EXEMPT',         3);
define('RCPT_TYPE_RATELIMIT_IP',   4);
define('RCPT_TYPE_RATELIMIT_USER', 5);
define('RCPT_TYPE_ALL',            6);
define('RCPT_TYPE_INBOUND',        7);
define('RCPT_TYPE_OUTBOUND',       8);

class RCPTCounter
{
    //! string variable for storing the full file path of the stats in question.
    protected $_m_strFilePath;

    //! array of the months data.
    protected $_m_arrData;

    //! string buffer for any errors that may occur during processing.
    protected $_m_strError;

    // integer to hold the month this instance is operating on.
    protected $_month;

    protected $_fileSize;

    protected $_validVersions;
    //! constructor
    /*!
     * @param   integer the month to load.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     *
     * Constructs object based on month passed and file contents.
     */
    public function __construct($month) {
        // TODO: could we ensure the month is an integer? we could intval() it.
        if ( is_numeric($month) && $month >= 1 && $month <= 12) {
            $this->_month = $month;
        } else {
            $this->_month = date('n');
        }
        $this->_m_strError = "";
        $this->_m_strFilePath = $this->_findStatFilePath(intval($this->_month));
        $this->_m_arrData = array();


        // We expect the MONTHLY file size
        // add up the expected size in bytes.
        // format: <version (uint8_t)><sent date (uint32_t)><31 record days>
        $datsize = 1;
        // account for the 'sent' timestamp field. this is a header at the
        // beginning of the file (32 bits). NOTE: it's not used for anything.
        $datsize += 4;
        // 31 day records, 8 integers (4 bytes each) per record.
        $datsize += 31 * 8 * 4;

        $this->_fileSize = $datsize;

        $this->_validVersions = array(0x2);
    }

    //! accessor to get the last buffered error message.
    /*!
     * @returns string  the last error message (if any)
     */
    public function error() {
        return $this->_m_strError;
    }

    //! generate path to the new format stat file for the given month.
    /*!
     * @param int $month The month to look for stats for.
     *
     * @returns mixed string The stat file to work with (full path),
     *   or NULL if some kind of error occurred.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * SIDE EFFECT: We'll set the instance variable _m_strError if an
     *   error occurred.
     *
     * we have two stat file formats. this method builds the path to the
     * new format one.
     */
    private function _getNewStatFilePath($month) {
        global $home;
        if (!isset($month) || !is_int($month) || $month < 1 || $month > 12) {
            $this->_m_strError = 'Invalid month.';
            return NULL;
        }
        if (!defined('COUNTER_PATH') && !defined('COUNTER_PATH_FE')) {
            $this->_m_strError = 'Statistics counter path is not available.';
            return NULL;
        }
        if (isset($home) && defined('COUNTER_PATH_FE')) {
            $path = sprintf('%s/%s.%02d', $home, COUNTER_PATH_FE, $month);
        } else {
            $path = sprintf('%s.%02d', COUNTER_PATH, $month);
        }
        return $path;
    }

    //! determine which stat file to work on.
    /*!
     * @param int $month The month to look for stats for.
     *
     * @returns mixed string The stat file to work with (full path),
     *   or NULL if some kind of error occurred.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * SIDE EFFECT: We'll set the instance variable _m_strError if an
     *   error occurred.
     */
    private function _findStatFilePath($month) {
        if (!isset($month) || !is_int($month) || $month < 1 || $month > 12) {
            $this->_m_strError = 'Invalid month.';
            return NULL;
        }
        if (!defined('COUNTER_PATH')) {
            $this->_m_strError = 'Statistics counter path is not available.';
            return NULL;
        }

        // fetch the generated full path based on the month
        $newPath = $this->_getNewStatFilePath($month);
        if ($newPath === NULL) {
            // we expect we already set an error.
            return NULL;
        }

        return $newPath;
    }

    //! load data from a stored stats data file - new (v2) format.
    /*!
     * @param resource $fh The opened for reading file handle.
     *
     * @returns boolean success or fail. Any errors that occur can be
     *   retrieved from 'error()' function.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     *
     * we changed the stats file format at one point. this method is to
     * read the new v2 format.
     */
    private function _loadStatsV2($fh) {
        if (!is_resource($fh)) {
            $this->_m_strError = 'Invalid statistics file handle.';
            return FALSE;
        }

        // add up the expected size in bytes.
        // format: <version (uint8_t)><sent date (uint32_t)><31 record days>
        $datsize = 1;
        // account for the 'sent' timestamp field. this is a header at the
        // beginning of the file (32 bits). NOTE: it's not used for anything.
        $datsize += 4;
        // 31 day records, 8 integers (4 bytes each) per record.
        $datsize += 31 * 8 * 4;

        // is the file the expected size? if not, we *must* conclude it's
        // corrupt.
        if (fseek($fh, 0, SEEK_END) !== 0) {
            $this->_m_strError = "Failed to seek through counter data file.";
            return FALSE;
        }


        if (ftell($fh) !== $this->_fileSize) {
            $this->_m_strError = "File {$this->_m_strFilePath} is not a valid"
               . " counter data file: " . ftell($fh) . " : " . $this->_fileSize;
            return FALSE;
        }

        // rewind back again.
        if (fseek($fh, 0, SEEK_SET) !== 0) {
            $this->_m_strError = "Failed to seek through counter data file.";
            return FALSE;
        }
        $bytesLeft = $this->_fileSize;

        // read in the version flag.
        $data = fread($fh, 1);
        if ($data === FALSE) {
            $this->_m_strError = "Failed to read counter data file version"
               . " flag.";
            return FALSE;
        }
        // at this time we expect only version 2 (which is 0x02)
        $version = unpack('C', $data);

        $version = $version[1];
        if ($version != 0x2) {
            $this->_m_strError = "Invalid counter data file version: $version";
            return FALSE;
        }
        $bytesLeft -= 1;
        // grab the 'sent' flag... this may change from bool to uint32_t
        // (timestamp)
        $sent = FALSE;
        $data = fread($fh, 4);
        if ($data === FALSE) {
            $this->_m_strError = "Failed to read counter data file sent flag.";
            return FALSE;
        }
        if ($data != 0) {
            $sent = TRUE;
        }
        // NOTE: this flag is not actually used for anything.
        //        This is for future use - when we can support an infrastructure
        //        for automated stats feedback to give a global map of MagicSpam
        //        effectiveness.
        $this->_m_arrData['sent'] = $sent;
        $bytesLeft -= 4;

        // read the actual stats. 31 days worth, but we track by file size.
        // TODO: why are we tracking counter? we don't use it, it seems.
        $counter = 0;
        while ($bytesLeft > 0) {
            $data = fread($fh, 4); // grab 'intsize bytes - timestamp
            if ($data === FALSE) {
                $this->_m_strError = "Failed to read statistic: timestamp";
                return FALSE;
            }
            $ts = unpack("L", $data); // unpack it - unsigned long
            $bytesLeft -= 4;

            $data = fread($fh, 4); // grab 'intsize' bytes - ham
            if ($data === FALSE) {
                $this->_m_strError = "Failed to read statistic: ham";
                return FALSE;
            }
            $ham = unpack("L", $data); // unpack it - unsigned long
            $bytesLeft -= 4;

            $data = fread($fh, 4); // grab - spam
            if ($data === FALSE) {
                $this->_m_strError = "Failed to read statistic: spam";
                return FALSE;
            }
            $spam = unpack("L", $data);
            $bytesLeft -= 4;

            $data = fread($fh, 4); // and.. grab exempt
            if ($data === FALSE) {
                $this->_m_strError = "Failed to read statistic: exempt";
                return FALSE;
            }
            $exempt = unpack("L", $data);
            $bytesLeft -= 4;

            // ip rate limit.
            $data = fread($fh, 4);
            if ($data === FALSE) {
                $this->_m_strError = "Failed to read statistic: IP rate limit";
                return FALSE;
            }
            $ipRatelimit = unpack("L", $data);
            $bytesLeft -= 4;

            // user rate limit.
            $data = fread($fh, 4);
            if ($data === FALSE) {
                $this->_m_strError = "Failed to read statistic: User ratelimit";
                return FALSE;
            }
            $userRatelimit = unpack("L", $data);
            $bytesLeft -= 4;

            // inbound counter
            $data = fread($fh, 4);
            if ($data === FALSE) {
                $this->_m_strError = "Failed to read statistic: Inbound count";
                return FALSE;
            }
            $inboundCount = unpack("L", $data);
            $bytesLeft -= 4;

            // outbound counter
            $data = fread($fh, 4);
            if ($data === FALSE) {
                $this->_m_strError = "Failed to read statistic: Outbound count";
                return FALSE;
            }
            $outboundCount = unpack("L", $data);
            $bytesLeft -= 4;
       

            // now store our hard earned data.
            $this->_m_arrData[$counter] = array();
            $this->_m_arrData[$counter]['timestamp'] = $ts[1];
            $this->_m_arrData[$counter]['ham'] = $ham[1];
            $this->_m_arrData[$counter]['spam'] = $spam[1];
            $this->_m_arrData[$counter]['exempt'] = $exempt[1];
            $this->_m_arrData[$counter]['ip_ratelimit'] = $ipRatelimit[1];
            $this->_m_arrData[$counter]['user_ratelimit'] = $userRatelimit[1];
            $this->_m_arrData[$counter]['inbound'] = $inboundCount[1];
            $this->_m_arrData[$counter]['outbound'] = $outboundCount[1];
            $counter++;
        }
        return TRUE;
    }

    //! Checks that the version is valid
    /*!
     * @returns boolean success or fail
     *
     * @author Logan Borean <loganb@linuxmagic.com>
     */
    protected function _isValidVersion($version) {
        return !isset($version) || empty($version) || in_array($version, $this->_validVersions);
    }

    //! method to 'load' the data from the stored data file.
    /*!
     * @returns boolean success or fail. Any errors that occur can be
     *   retrieved from 'error()' function.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function load() {
        // sanity check.
        if (!is_string($this->_m_strFilePath)
            || strlen($this->_m_strFilePath) === 0) {
            $this->_m_strError = "Invalid file path";
            return FALSE;
        }
        // open the file up. both formats are binary.
        // sanity check
        if (!is_string($this->_m_strFilePath)
            || strlen($this->_m_strFilePath) === 0) {
            $this->_m_strError = "Invalid file path";
            return FALSE;
        }
        // make sure the path exists
        if (!file_exists($this->_m_strFilePath)) {
            $this->_m_strError = "File {$this->_m_strFilePath} does not exist!";
            return FALSE;
        }

        // try to open the file in binary mode.
        $fh = fopen($this->_m_strFilePath, "rb");
        if ($fh === FALSE) {
            $this->_m_strError = "Error opening {$this->_m_strFilePath}";
            return FALSE;
        }

        // in the event we are 'reloading'... let's be good neighbours and clean
        // up our trash.
        unset($this->_m_arrData);
        $this->_m_arrData = array();

        $res = $this->_loadStatsV2($fh);
        fclose($fh);

        return $res;
    }

    //! method to get a specific count value, or 'all'.
    /*!
     * @param   integer the day of the month to grab
     * @param   integer the count type (see defines at top of file).
     *
     * @returns mixed   the count value found or 0 if nothing is loaded. IF the
     *                  count type is 'ALL', then returns the full array of counts
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function get_value($day, $type) {
        // sanity check.
        if (!is_numeric($day) || $day < 1 || $day > 31) {
            return 0;
        }
        if ( !is_numeric($type) ||
            // TODO: how about a function to check if it's a valid type?
            ( $type != RCPT_TYPE_TOTAL &&
              $type != RCPT_TYPE_HAM &&
              $type != RCPT_TYPE_SPAM &&
              $type != RCPT_TYPE_EXEMPT &&
              $type != RCPT_TYPE_RATELIMIT_IP &&
              $type != RCPT_TYPE_RATELIMIT_USER &&
              $type != RCPT_TYPE_ALL &&
              $type != RCPT_TYPE_INBOUND &&
              $type != RCPT_TYPE_OUTBOUND ) ) {
            return 0;
        }

        // since the array is based on a zero indexed count...
        $day--;
        // make sure our internal data is set up
        // TODO: just return if it's not... reduce nesting!
        if (is_array($this->_m_arrData)) {
            // and that there is data for the requested day index.
            if (isset($this->_m_arrData[$day])) {
                // grab the timestamp.
                $ts = $this->_m_arrData[$day]['timestamp'];
                // and make sure the timestamp matches our month...
                if ($this->_month == date("n",$ts)) {
                    switch($type) {
                    case RCPT_TYPE_TOTAL:
                        $total = $this->_m_arrData[$day]['ham'];
                        $total += $this->_m_arrData[$day]['spam'];
                        $total += $this->_m_arrData[$day]['exempt'];
                        // TODO: should we add the counts for IP/user ratelimit
                        //   in here?
                        return $total;
                        break;
                    case RCPT_TYPE_HAM:
                        return $this->_m_arrData[$day]['ham'];
                        break;
                    case RCPT_TYPE_SPAM:
                        return $this->_m_arrData[$day]['spam'];
                        break;
                    case RCPT_TYPE_EXEMPT:
                        return $this->_m_arrData[$day]['exempt'];
                        break;
                    case RCPT_TYPE_RATELIMIT_IP:
                        return $this->_m_arrData[$day]['ip_ratelimit'];
                        break;
                    case RCPT_TYPE_RATELIMIT_USER:
                        return $this->_m_arrData[$day]['user_ratelimit'];
                        break;
                    case RCPT_TYPE_INBOUND:
                        return $this->_m_arrData[$day]['inbound'];
                        break;
                    case RCPT_TYPE_OUTBOUND:
                        return $this->_m_arrData[$day]['outbound'];
                        break;
                    case RCPT_TYPE_ALL:
                        return $this->_m_arrData[$day];
                        break;
                    }
                }
            }

        }


        return 0;
    }

    //! Returns the largest of any of the types (i.e. Ham / Spam / Exempt / Rate Limit)
    /*!
     * @author Robert Bisewski <robertb@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function get_greatest() {
        $ret = 0;
        if (is_array($this->_m_arrData)) {
            foreach ($this->_m_arrData as $day => $dat) {
                if (!is_array($dat)) {
                    continue;
                }
                if ($dat['ham'] > $ret) {
                    $ret = $dat['ham'];
                }
                if ($dat['spam'] > $ret) {
                    $ret = $dat['spam'];
                }
                if ($dat['exempt'] > $ret) {
                    $ret = $dat['exempt'];
                }
                if (($dat['ip_ratelimit'] + $dat['user_ratelimit']) > $ret) {
                    $ret = $dat['ip_ratelimit'] + $dat['user_ratelimit'];
                }
            }
        }
        return $ret;
    }

    //! utility function to fetch the full timestamp for a given index.
    /*!
     * @param   integer the index to seek.
     *
     * @returns mixed   the unix timestamp if found, or FALSE if not found.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function get_date($idx) {
        // sanity check.
        if (!is_numeric($idx) || $idx < 1 || $idx > 31) {
            $this->_m_strError = "Invalid argument to get_date function";
            return FALSE;
        }
        $idx--;
        if (is_array($this->_m_arrData)) {
            if (isset($this->_m_arrData[$idx])) {
                return $this->_m_arrData[$idx]['timestamp'];
            }
        }
        $this->_m_strError = "Data for index $idx not found";
        return FALSE;
    }
}

