<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): William Storey <william@linuxmagic.com>
 *
 * $Id: lib.reversereader.php 4538 2014-12-08 17:36:56Z william $
 *
 * abstraction to read a file backwards, line by line.
 */
class ReverseReader {
    //! error: file not set to open.
    const ERROR_FILE_NOT_SET      = 1;
    //! error: file does not exist.
    const ERROR_FILE_NOT_EXIST    = 2;
    //! error: file is not readable.
    const ERROR_FILE_NOT_READABLE = 3;
    //! error: file could not be opened.
    const ERROR_FILE_NOT_OPENED   = 4;
    //! error: file seek failed.
    const ERROR_FILE_NOT_SEEKED   = 5;
    //! error: file read failed.
    const ERROR_FILE_READ         = 6;
    //! error (well, not an error): no more to read.
    const ERROR_NO_MORE_TO_READ   = 7;
    //! error extracting string.
    const ERROR_EXTRACT_STRING    = 8;
    //! error searching strnig.
    const ERROR_SEARCH_STRING     = 9;

    //! string. path to the file we will read.
    private $_file_path;
    //! integer. buffer size to read in at once.
    private $_buffer_size;
    //! resource. file open for reading.
    private $_fh;
    //! integer. file's size.
    private $_file_size;
    //! integer. last integer code.
    private $_last_error_code;
    //! string. current buffer read from the file.
    private $_buf;

    //! constructor.
    /*!
     * @param string $filePath Path to the file to read.
     * @param int $bufSize Buffer size to read in at once.
     *
     * @constructor
     *
     * @author William Storey <william@linuxmagic.com>
     */
    public function __construct($filePath, $bufSize) {
        if (isset($filePath) && is_string($filePath) && strlen($filePath) > 0) {
            $this->_file_path = $filePath;
        }
        if (isset($bufSize) && is_int($bufSize) && $bufSize > 0) {
            $this->_buffer_size = $bufSize;
        }
    }

    //! destructor.
    /*!
     * @destructor
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * if our file is open, close it.
     */
    public function __destruct() {
        if ($this->_fh !== NULL) {
            fclose($this->_fh);
            $this->_fh = NULL;
        }
    }

    //! open our file for reading.
    /*!
     * @return bool Whether successful
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * @access private
     *
     * @pre File has not been opened
     *
     * open our file for reading. this is from our instance variables.
     */
    public function _openFile() {
        // we must have a file path set.
        if ($this->_file_path === NULL || !is_string($this->_file_path) ||
            strlen($this->_file_path) === 0) {
            $this->_last_error_code = self::ERROR_FILE_NOT_SET;
            return FALSE;
        }
        // ensure the file exists.
        if (!file_exists($this->_file_path)) {
            $this->_last_error_code = self::ERROR_FILE_NOT_EXIST;
            return FALSE;
        }
        // ensure we can read it.
        if (!is_readable($this->_file_path)) {
            $this->_last_error_code = self::ERROR_FILE_NOT_READABLE;
            return FALSE;
        }
        // try to open it for reading.
        // binary mode because we're handling lines ourselves.
        $fh = fopen($this->_file_path, 'rb');
        if ($fh === FALSE) {
            $this->_last_error_code = self::FILE_NOT_OPENED;
            return FALSE;
        }
        // get its file size. useful for knowing how much we've read.
        // NOTE: fstat() apparently can't fail.
        $fileStats = fstat($fh);
        $this->_file_size = intval($fileStats[7]);
        // position to the end of the file since we're reading backwards.
        // NOTE: this seek is redundant. we will seek prior to each read
        //   anyway.
        if (fseek($fh, 0, SEEK_END) !== 0) {
            fclose($fh);
            $this->_last_error_code = self::FILE_NOT_SEEKED;
            return FALSE;
        }
        $this->_fh = $fh;
        // track how much we read.
        $this->_bytes_read = 0;
        return TRUE;
    }

    //! read in more of the file into our buffer.
    /*!
     * @return bool Whether succesful
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * @access private
     *
     * @pre File has been opened
     *
     * decide if there is more to read. if not, we're done.
     * first position for our read (because we read backwards through
     * the file), and then read in a chunk.
     *
     * NOTE: we will always try to read more if requested. we don't enforce our
     *   internal buffer being under a certain size.
     *   with a small enough buffer size that would not be possible if we
     *   want to return full lines however. consider a buffer size of 1, we
     *   could never return a line that is 10 characters long if we enforced
     *   a buffer size of 1.
     */
    public function _readMore() {
        // we might have read everything already.
        if (!$this->_isThereMoreToRead()) {
            $this->_last_error_code = self::ERROR_NO_MORE_TO_READ;
            return FALSE;
        }
        // reposition for the next read. remember we start at the very end.
        $leftToRead = $this->_file_size - $this->_bytes_read;
        $positionToSeek = max(0, $leftToRead-$this->_buffer_size);
        if (fseek($this->_fh, $positionToSeek, SEEK_SET) !== 0) {
            $this->_last_error_code = self::ERROR_FILE_NOT_SEEKED;
            return FALSE;
        }
        // read it. don't read the same thing twice by accident!
        $amountToRead = min($this->_buffer_size, $leftToRead);
        $buf = fread($this->_fh, $amountToRead);
        if ($buf === FALSE) {
            $this->_last_error_code = self::ERROR_FILE_READ;
            return FALSE;
        }
        // prepend buf.
        if ($this->_buf === NULL) {
            $this->_buf = $buf;
        } else {
            $this->_buf = $buf . $this->_buf;
        }
        // track how much we've read in total.
        $this->_bytes_read += strlen($buf);
        return TRUE;
    }

    //! read a line.
    /*!
     * @return mixed String, the line, or bool FALSE.
     *   Inspect the object's error code for details about the failure.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * @access public
     *
     * we read the last unread line from the file.
     * this function will open the file and take care of filling our buffer.
     * to read from a file, call this function repeatedly until it returns
     * false. to ensure we read the whole file, check that the error code
     * is NO_MORE_TO_READ.
     *
     * lines will be returned with their preceding newline.
     * for example if you have a file like:
     * "test\none two\nthree four\n" you will get lines back:
     * "\n"
     * "\nthree four"
     * "\none two"
     * "test"
     */
    public function readLine() {
        // ensure no error code is hanging around from last time.
        $this->_last_error_code = NULL;
        // ensure the file is open.
        if ($this->_fh === NULL && !$this->_openFile()) {
            return FALSE;
        }
        // might not have any buffer yet.
        if ($this->_buf === NULL && !$this->_readMore()) {
            return FALSE;
        }
        // loop until we find the next line. read more if necessary.
        // we tell we have a line once we see a newline or reach the end
        // of the file.
        // a note on how this works: if we want to not return \n from the
        // preceding line, we have to be careful in the case of returning
        // "" because then we risk finding the same \n each time this
        // function is called.
        // to get around this it is simpler if we return the last \n we
        // see each time. the \n that tells us where our line starts.
        // we then trim that \n off and have a good start for the next
        // iteration.
        // we expect our buffer to have the piece we return removed each time.
        while (1) {
            // start at end of buffer.
            $nlPosition = self::_strrchr($this->_buf, "\n",
                strlen($this->_buf)-1);
            // if we found a newline then we can return a line.
            if ($nlPosition !== FALSE) {
                $line = substr($this->_buf, $nlPosition);
                // remove this portion of the line.
                // if the newline is in position 0 then we reset the buffer
                // because we're taking the whole thing.
                if ($nlPosition > 0) {
                    $this->_buf = substr($this->_buf, 0, $nlPosition);
                    if ($this->_buf === FALSE) {
                        $this->_last_error_code = self::ERROR_EXTRACT_STRING;
                        return FALSE;
                    }
                } else {
                    $this->_buf = NULL;
                }
                return $line;
            }
            // no newline found. read to find one, or if there's nothing
            // to read, then that's it, we have a line.
            if (!$this->_isThereMoreToRead()) {
                $line = $this->_buf;
                $this->_buf = NULL;
                return $line;
            }
            // read a bit more to find a newline.
            if (!$this->_readMore()) {
                // if we encountered an error other than EOF then abort.
                if ($this->getLastErrorCode() !== self::ERROR_NO_MORE_TO_READ) {
                    return FALSE;
                }
                // loop again and find our next line.
            }
        }
        // we should not reach here!
        $this->_last_error_code = self::ERROR_SEARCH_STRING;
        return FALSE;
    }

    //! is there more to read?
    /*!
     * @return bool Whether more to read or not.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * @access private
     *
     * @pre File has been opened.
     *
     * we can tell this by comparing our file size with how much we've read.
     */
    public function _isThereMoreToRead() {
        return $this->_file_size - $this->_bytes_read > 0;
    }

    //! retrieve the last error code, if any.
    /*!
     * @return mixed Integer one of our ERROR_ constants, or NULL if no
     *   error known.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * @access public
     */
    public function getLastErrorCode() {
        return $this->_last_error_code;
    }

    //! a more c-like strchr
    /*!
     * @param string $s String to search in
     * @param string $c The character to look for
     * @param int $pos The position to start searching from
     *
     * @return mixed integer Index into the string where the character is
     *   found, or bool FALSE if it's not found.
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * @access private
     *
     * starting as position pos in the string, look at each character in
     * the string, moving backwards through the string. if the character
     * at an index matches character c then we return that index.
     *
     * so it seems kind of odd to write this function. however, both strpos()
     * and strchr() seemed to not do what I want, which is to start searching
     * back in a string from a position, and not copy memory!
     */
    public static function _strrchr($s, $c, $pos) {
        if (!is_string($s) || strlen($s) === 0) {
            return FALSE;
        }
        if (!is_string($c) || strlen($c) !== 1) {
            return FALSE;
        }
        if (!is_int($pos) || $pos < 0 || strlen($s)-1 < $pos) {
            return FALSE;
        }
        for ($i = $pos; $i >= 0; $i--) {
            if ($s[$i] === $c) {
                return $i;
            }
        }
        return FALSE;
    }
}
