<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2014 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *
 * $Id: lib.rlmanager.php 4221 2014-11-04 23:28:57Z shaun $
 *
 * This is a container to access the rate limiter 'manager' utility.
 * There are only two operations available in this class:
 *
 *     list
 *     remove
 */
include_once('lib.cmdwrapper.php');

class RateLimiterManager extends cmdWrapper {
    // internal private singleton.
    static private $object = NULL;

    //! class constructor
    /*!
     * @author Shaun Johnson <shaun@linuxmagic.com>
     * we want this class type to be a singleton, so we deny access to the constructor
     *
     */
    protected function __construct() {
        // instantiate back to the parent.
        parent::__construct(RL_MANAGER);
    }

    //! static function to get the current singleton instance
    /*!
     * @returns RateLimiterManager  the active singleton instance.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     *
     */
    static public function getInstance() {
        if (is_null(self::$object)) {
            self::$object = new RateLimiterManager;
        }
        return self::$object;
    }
    //! public function to remove a rate limited ip from the list
    /*!
     * @returns bool whether or not the function succeeded.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     *
     */
    public function removeIP($ip) {
        // NOTE: we skip the sanity checks in favor of passing
        //       them on to the common function
        return $this->removeEntry($ip);
    }

    //! public function to remove a rate limited user from the list
    /*!
     * @param $user person to be removed
     * @returns bool whether or not the function succeeded.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function removeUser($user) {
        // NOTE: we skip the sanity checks in favor of passing
        //       them on to the common function
        return $this->removeEntry($user, "-a");
    }

    // public function to list the current rate limited IP addresses
    /*!
     * @param $user person to be removed
     * @returns bool whether or not the function succeeded.
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function listIPS() {
        return $this->listEntries();
    }
    // public function to list the current rate limited users
    /*!
     * @returns list of rate limited users
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    public function listUsers() {
        return $this->listEntries("-a");
    }

    //! internal function to 'remove' a rate limited entry.
    /*!
     * @param   string      the entry to remove from the blocked list.
     * @param   addflag	    optional flag to pass to the command
     *
     * @returns boolean     success or fail of operation. Truth to tell this will likely be ignored....
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    private function removeEntry($entry, $flag = NULL) {
        $buffer = "";
        // basic sanity check
        if (!is_string($entry) || !strlen($entry)) {
            $this->lastError = "Invalid parameter";
            return FALSE;
        }
        $args = array('-r');
        $args[] = $entry;
        if ($flag && strlen($flag) > 0) {
            $args[] = $flag;
        }
        $rv = $this->execRead($args, $buffer);
        if (strlen($buffer)) {
            $this->lastError = $buffer;
            return FALSE;
        }
        return TRUE;
    }

    //! internal function to get a list of the current rate limited entries.
    /*!
     * @param	string	optional flag to pass to list command
     *
     * @returns array   hashed array of current entries:
     *
     *                  [full string ip address] => [unix time stamp when it was blocked]
     *
     *          or.. FALSE on errors (use 'error()' function to get the buffered message)
     * @author Shaun Johnson <shaun@linuxmagic.com>
     */
    private function listEntries($flag = NULL) {
        $buffer = "";
        $retval = array();
        $args = array('-l');
        if ($flag && strlen($flag) > 0) {
            $args[] = $flag;
        }
        $rv = $this->execRead($args, $buffer);
        if ($rv == 0) {
            // split up into the individual lines
            $lines = explode("\n", $buffer);
            foreach ($lines as $line) {
                $parts = explode(":", $line);
                if (sizeof($parts) == 2) {
                    $retval[$parts[0]] = $parts[1];
                }
            }
        } else {
            $this->lastError = $buffer;
            return FALSE;
        }
        return $retval;
    }
}
