<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2017-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *            Deion Yu <deiony@linuxmagic.dev>
 *
 * $Id$
 */

class SplashScreen {
    // constants for body type
    const VIEW_NONE = 0;
    const VIEW_FILE = 1;
    const VIEW_FUNC = 2;

	// Splash screens can be displayed in two modes:
	//        1. Recurring with cookies
	//        2. Conditionally no cookies
    protected $_cookie_name;
    protected $_cookie_value;
    protected $_frequency_in_days;

    protected $_title;
    protected $_css;

	// The source of the view can be:
	//  1. file
	//  2. function
	// Requirements:
	//  1. Both should output html for display within the splash screen
    protected $_view;
    protected $_view_type;
    protected $_bind_variable;

    public function __construct() {
        $this->_initialize();
    }

    //! prepares and print the HTML
    public function render() {
        print $this->getView();
    }

    //! prepares and returns the HTML as a string
    public function getView() {
        if ($this->_validateSplashPage() !== TRUE) {
            return "";
        }

        extract($this->_bind_variable);
        $html  = $this->_templateHeader();
        $html .= $this->_loadView();
        $html .= $this->_templateFooter();
        $html .= $this->_includeCss();

        return $html;
    }

    protected function _initialize() {
        $this->_title = "";
        $this->_view_type = self::VIEW_NONE;
        $this->_bind_variable = array();
        $this->_css = array();
    }

    //! validates the splash page session
    protected function _validateSplashPage() {
        // a splash page needs a title and view
        if ($this->_isStringEmpty($this->_title) ||
            $this->_view_type === self::VIEW_NONE) {
            return FALSE;
        }

        // validate the session if splash page has a cookie
        if (isset($this->_cookie_name)) {
            if (!empty($_COOKIE[$this->_cookie_name])) {
                // the splash page has already been display
                if ($_COOKIE[$this->_cookie_name] === $this->_cookie_value) {
                    return FALSE;
                }
            }

            // set the cookie with the frequency
            $exp = time() + ($this->_frequency_in_days * 60 * 60 *24);
            setcookie($this->_cookie_name, $this->_cookie_value, $exp, '/');
        }

        return TRUE;
    }

    //! returns the splash page template header
    protected function _templateHeader() {
        $html  = '  <!-- Modal -->' . "\n";
        $html .= '  <div class="modal fade" id="splash-screen" role="dialog">' . "\n";
        $html .= '      <div class="modal-dialog">' . "\n";
        $html .= '          <!-- Modal content-->' . "\n";
        $html .= '          <div class="modal-content">' . "\n";
        $html .= '              <div class="modal-header">' . "\n";
        $html .= '                  <button type="button" class="close" onclick="closeSplash();" data-dismiss="modal">&times;</button>' . "\n";
        $html .= '                  <h3 class="modal-title">' . htmlspecialchars($this->_title) . '</h3>' . "\n";
        $html .= '              </div>' . "\n";
        $html .= '              <div class="modal-body">' . "\n";

        return $html;

    }

    //! returns the splash page template body
    protected function _loadView() {
        switch ($this->_view_type) {
            case self::VIEW_FUNC:
                return $this->_view();
            case self::VIEW_FILE:
                ob_start();
                include($this->_view);
                return ob_get_clean();
            case self::VIEW_NONE:
            default:
                return "";
        }
    }

    //! retrieves the splash page template footer
    protected function _templateFooter() {
        $html  = '';
        $html .= '              </div>' . "\n";
        $html .= '          </div>' . "\n";
        $html .= '      </div>' . "\n";
        $html .= '  </div>' . "\n";

        return $html;
    }

    //! JS function that will include the CSS in header and define the function we use to close the modal
    public function _includeCss() {
        # add js to close the splash page at bare minimum
        $jsSnip  = '<script type="text/javascript">' . "\n";
        $jsSnip .= '    function closeSplash() {' . "\n";
        $jsSnip .= '        var modal = document.getElementById("splash-screen");' . "\n";
        $jsSnip .= '        modal.classList.add("fadeout");' . "\n";
        $jsSnip .= '    }' . "\n";

        // no css to include, skipping...
        if (!empty($this->_css)) {
            // build the CSS array string for JS
            $css = "['" . join($this->_css, "', '") . "']";

            $jsSnip .= '    document.addEventListener("DOMContentLoaded", function() {' . "\n";
            $jsSnip .= '        var head  = document.getElementsByTagName("head")[0];' . "\n";
            $jsSnip .= '        var cssArray   = ' . $css . ';' . "\n";
            $jsSnip .= '        cssArray.forEach(function(css) { ' . "\n";
            $jsSnip .= '            var link  = document.createElement("link");' . "\n";
            $jsSnip .= '            link.rel  = "stylesheet";' . "\n";
            $jsSnip .= '            link.type = "text/css";' . "\n";
            //  XXX: Find path prefix to CSS for all suites
            $jsSnip .= '            link.href = "' . RELPREFIX . '" + css;' . "\n";
            $jsSnip .= '            link.media = "all";' . "\n";
            $jsSnip .= '            head.appendChild(link);' . "\n";
            $jsSnip .= '        });' . "\n";
            $jsSnip .= '    });' . "\n";
        }

        $jsSnip .= '</script>' . "\n";

        return $jsSnip;
    }

    // validator functions from WizardValidator
    protected function _isStringEmpty($string) {
        return !isset($string)     ||
               !is_string($string) ||
               empty($string)      ||
               strlen(trim($string)) === 0;
    }
    protected function _isIntPositive($int) {
        return isset($int)         &&
               $this->_isInt($int) &&
               intval($int) > 0;
    }
    protected function _isArray($array) {
        return isset($array) &&
               is_array($array);
    }
    protected function _isCallback($callback) {
        return isset($callback) &&
               is_callable($callback);
    }
    protected function _isInt($int) {
        return isset($int)          &&
               is_numeric($int)     &&
               intval($int) == $int &&
               $int <= PHP_INT_MAX;
    }
    protected function _isValidCookie($name, $value, $frequency) {
        return !$this->_isStringEmpty($name)  &&
               !$this->_isStringEmpty($value) &&
                $this->_isIntPositive($frequency);
    }

    // setter functions
    public function setCookie($name, $value, $frequency) {
        if (!$this->_isValidCookie($name, $value, $frequency)) {
            return;
        }
        $this->_cookie_name = $name;
        $this->_cookie_value = $value;
        $this->_frequency_in_days = $frequency;
    }
    public function setTitle($title) {
        if ($this->_isStringEmpty($title)) {
            return;
        }
        $this->_title = $title;
    }
    public function setViewFile($file) {
        if ($this->_isStringEmpty($file) || !is_readable($file)) {
            return;
        }
        $this->_view = $file;
        $this->_view_type = self::VIEW_FILE;
    }
    public function setViewFunction($func) {
        if ($this->_isCallback($func)) {
            return;
        }
        $this->_view = $func;
        $this->_view_type = self::VIEW_FUNC;
    }
    public function setCss($css) {
        $this->_css = $this->_isArray($css) ? $css : array();
    }
    public function setBindVariables($bind_var) {
        $this->_bind_variable = $this->_isArray($bind_var) ? $bind_var : array();
    }
}
