<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id: interface.wiz_db_abstract.php 154 2017-03-07 18:24:37Z rares $
 *
 * These are the interfaces that must be implemented by a DB class
 * Please note that these interfaces are following the PEAR syntax
 * 
 * There are two interfaces, one that should be used when we initialize a transaction
 * and a second one for dealing with Statements and fetching results
 */

interface WizDbBase {
    //! @int const for fetch type numeric index 
    const FETCH_NUM =  1;
    
    //! @int const for fetch type associative array 
    const FETCH_ASSOC = 2;
}

interface WizDb extends WizDbBase {

    //! Helper method to create a database connection and return the instance
    /*!
     * @param array $elements array with the username, database, password required to make the connection 
     *
     * @return mixed object with the connection or NULL if it fails
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     static public function getInstance($elements);

    //! Helper method to check if there was an error while running a query
    /*!
     * @param object $object the object that we want to check
     * 
     * @return bool whether there was an error
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public static function isError($object);

    //! Method used to return the error message if there was an error
    /*!
     * @return mixed string with the message or NULL  
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function getMessage();

    //! Method used to return the additional information about an error message
    /*!
     * @return mixed string with the message or NULL  
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function getUserInfo();

    //! Method used to free the prepared statement 
    /*!
     * @param object $sth the prepared statement that we want to free 
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function freePrepared($sth);

    //! Helper method to make a query, this could be used to do prepare statements if the second argument is passed
    /*!
     * @param string $sql the SQL statement that you want to run 
     * @param array $params array with arguments if you want to bind arguments
     * @param bool $bind_type if you want to bind by type, this is required for bool PDO
     * 
     * @return mixed the result set or object with error or NULL
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function query($sql, $params = array(), $bind_type = FALSE);

    //! Helper method to prepare a statement 
    /*
     * @param string $sql the SQL statement that you want to run 
     * 
     * @return mixed the prepare statements object or NULL
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function prepare($sql);

    //! Helper method to execute a prepare statement
    /*
     * @param object $sth the prepare statement object 
     * @param array $params array with argument that you want to bind
     * @param bool $bind_type if we wantt to bind by type, this is required for bool PDO
     * 
     * @return mixed the results set object or NULL
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function execute($sth, $params = array(), $bind_type = FALSE);

    //! Helper method to quote a variable and prepare for a SQL 
    /*
     * @return mixed $var the quoted variable 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function quote($var);

    //! Helper method to get the next Id from a table 
    /*
     * @return mixed the quoted variable 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * XXX This should be deprecated, but there are sc
     */
     public function nextId($sequence, $onDemand = TRUE);

    //! Helper method to turn on or off the autocommit 
    /*
     * @param bool $onOff, method used to start transactions
     * 
     * @return bool whether the autocommit was set 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function autoCommit($onOff = FALSE);

    //! Helper method to begin a set of transactions to DB 
    /*
     * @return bool whether the begin transaction was successful 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function begin();


    //! Helper method to commit a set of transactions to DB 
    /*
     * @return bool whether the commit was successful 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function commit();

    //! Helper method to that will determine if we are in a transaction
    /*
     * @return bool whether we are in the middle of a DB transaction 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function inTransaction();

    //! Method used to get the number of rows from a INSERT, UPDATE, DELETE statement 
    /*
     * @return mixed int with the number or rows from the last query or NULL id there was a problem 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function affectedRows();

    //! Helper method to rollback a set of transactions to DB 
    /*
     * @return bool whether the commit was successful 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function rollback();

    //! Helper method to fetch the first column of the first row from a query result
    /*
     * @param object $sth the prepare statement object 
     * @param array $params array with argument that you want to bind
     * @param bool $bind_type if we want to bind by type, this is required for bool PDO
     * 
     * @return mixed the value from first column or NULL
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function getOne($sql, $params = array(), $bind_type = FALSE);

    //! Helper method to fetch all of the rows from a query result 
    /*
     * @param object $sth the prepare statement object 
     * @param array $params array with argument that you want to bind
     * @param int $fetchMode the fetch method that you prefer, 0 default  
     * @param bool $bind_type if we want to bind by type, this is required for bool PDO
     * 
     * @return mixed array with results or NULL
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function getAll($sql, $params = array(), $fetchmode = 0, $bind_type = FALSE);

    //! Helper method to do a query and return the first result set into an array
    /*
     * @param object $sth the prepare statement object 
     * @param array $params array with argument that you want to bind 
     * @param int $fetchMode the fetch method that you prefer, 0 default  
     * @param bool $bind_type if we want to bind by type, this is required for bool PDO
     * 
     * @return mixed array with a result or NULL 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function getRow($sql, $params = array(), $fetchmode = 0, $bind_type = FALSE);
}


interface WizDbSth extends WizDbBase {
    //! Method used to fetch records from a results set
    /*
     * @param int $mode, the fetch mode type numeric or associative array
     *
     * @return mixed array with a result or NULL 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function fetchRow($mode = 0);

    //! Method used to get the number of rows from a SELECT statement 
    /*
     * @return mixed int with the number or rows from the last query or NULL id there was a problem 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function numRows();

    //! Method used to free the memory with the last statement 
    /*
     * @return void 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function free();

    //! Method used to return the error message if there was an error
    /*
     * @return mixed string with the message or NULL  
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function getMessage();
 
}
