<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): HsuanWei Fu <hsuanwei@linuxmagic.com>
 *            Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id$
 */

// this class requires another class specific to geoip lookups
require_once(__DIR__ . '/lib.mm_db_lookup.php');


class HttpVar {

    //! default expire time for a cookie (3 months)
    /*!
     * @const int default expire time
     */
    const DEFAULT_EXPIRE = 7776000;

    //! Checks if session is started
    /*!
     * @returns bool TRUE if started, FALSE if unsuccess or not started
     */
    public static function isSessionStarted() {
        // does this run in CLI mode
        if (php_sapi_name() === 'cli' ) {
            return FALSE;
        }
        // DO we have the session_status function in this PHP version 
        if (version_compare(phpversion(), '5.4.0', '>=')) {
            return session_status() === PHP_SESSION_ACTIVE ? TRUE : FALSE;
        } else {
            return session_id() === '' ? FALSE : TRUE;
        }
    }

    //! Get session value by key 
    /*!
     * @param string $key key in the session array
     * @param mixed $default default value to return if key is not found
     *
     * @return mixed Session value by key or NULL if not set
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     *         Rares Harnagea <rares@linuxmagic.com>
     */
    public static function getSession($key, $default = NULL) {
        // sanity check
        if (!isset($key) || !WizardValidator::isStringNotEmpty($key)) {
            WizardDebug::log('Function called incorrectly');
            return $default;
        }
        if (!self::isSessionStarted() || !isset($_SESSION[$key])) {
            return $default;
        }

        return $_SESSION[$key];
    }

    //! Set session value by key 
    /*!
     * @param string $key key in the session array
     * @param mixed $value value to store in the session
     * @param bool $overwrite overwrite the value if the key already exists
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     *         Rares Harnagea <rares@linuxmagic.com>
     */
    public static function setSession($key, $value, $overWrite = TRUE) {
        // sanity check
        if (!isset($key)    || !WizardValidator::isStringNotEmpty($key) ||
            !isset($value)  || !WizardValidator::isBool($overWrite)) {
            WizardDebug::log('Function called incorrectly');
            return;
        }
        if (!self::isSessionStarted()) { 
            // Probably headers were already sent
            if (!session_start()) {
                return; 
            }
        }
        // check if overwrite is enabled
        if (!$overWrite && isset($_SESSION[$key])) {
            return;
        }

        $_SESSION[$key] = $value;
    }

    //! Add a value to a session array 
    /*!
     * @param string $key key in the session array
     * @param mixed $value value to store in the session
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function addSession($key, $value) {
        // sanity check
        if (!isset($key)    || !WizardValidator::isStringNotEmpty($key) ||
            !isset($value) || is_object($value)) {
            WizardDebug::log('Function called incorrectly');
            return FALSE;
        }
        if (!self::isSessionStarted()) { 
            // Probably headers were already sent
            if (!session_start()) {
                return; 
            }
        }

        // check if we there is a session[$key] set and if not let set it now
        if (!isset($_SESSION[$key])) {
            $_SESSION[$key] = array(); 
        } elseif (!is_array($_SESSION[$key])) {
            return FALSE;
        }
        // add the new value to the array session        
        $_SESSION[$key][] = $value;

        return TRUE;
    }


    //! Remove session variable by key 
    /*!
     * @param string $key key in the session array
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public static function removeSession($key) {
        // sanity check
        if (!isset($key) || !WizardValidator::isStringNotEmpty($key)) {
            WizardDebug::log('Function called incorrectly');
            return;
        }
        if (!self::isSessionStarted()) {
            session_start();
        }

        unset($_SESSION[$key]);
    }

    //! Get post value by key 
    /*!
     * @param string $key key in the post array
     * @param mixed $default default value to return if key is not found
     * @param bool $trim flag to trim any string variable, default TRUE
     *
     * @return mixed Post value by key 
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     *         Rares Harnagea <rares@linuxmagic.com>
     */
    public static function getPost($key, $default = NULL, $trim = TRUE) {
        // sanity check
        if (!isset($key) || !WizardValidator::isStringNotEmpty($key)) {
            WizardDebug::log('Function called incorrectly');
            return $default;
        }
        if (!isset($_POST) || !isset($_POST[$key])) {
            return $default;
        }
        // A $_POST variable could be a string or an array 
        if (is_array($_POST[$key])) {
            $ret = array();
            foreach ($_POST[$key] as $i => $v) {
                $ret[$i] = str_replace("\0", '', $v);  
                // TODO make this recursive
                if (is_string($ret[$i]) && $trim) {
                    $ret[$i] = trim($ret[$i]);
                }
            }
            // return array cases
            return $ret;
        }
        // This is a string    
        // trin any remove any NULL bytes injection  
        $value = str_replace("\0", '', $_POST[$key]);
        if ($trim) {
            $value = trim($value);
        }
        return $value;
    }

    //! Get get value by key 
    /*!
     * @param string $key key in the get array
     * @param mixed $default default value to return if key is not found
     *
     * @return mixed Get value by key 
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     *         Rares Harnagea <rares@linuxmagic.com>
     */
    public static function getGet($key, $default = NULL, $trim = TRUE) {
        // sanity check
        if (!isset($key) || !WizardValidator::isStringNotEmpty($key)) {
            WizardDebug::log('Function called incorrectly');
            return $default;
        }
        if (!isset($_GET) || !isset($_GET[$key])) {
            return $default;
        }
        // A $_GET variable could be a string or an array 
        if (is_array($_GET[$key])) {
            $ret = array();
            foreach ($_GET[$key] as $i => $v) {
                // TODO make this recursive
                $ret[$i] = str_replace("\0", '', $v);  
                if (is_string($ret[$i]) && $trim) {
                    $ret[$i] = trim($ret[$i]);
                }
            }
            return $ret;

        }

        // This is a string    
        // trin any remove any NULL bytes injection  
        $value = str_replace("\0", '', $_GET[$key]);
        if ($trim) {
            $value = trim($value);
        }
        return $value;
    }

    //! Get cookie value by key 
    /*!
     * @param string $key key in the cookie array
     * @param mixed $default default value to return if key is not found
     *
     * @return mixed Cookie value by key 
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public static function getCookie($key, $default = NULL) {
        // sanity check
        if (!isset($key) || !WizardValidator::isStringNotEmpty($key)) {
            return $default;
        }
        if (!isset($_COOKIE) || !isset($_COOKIE[$key])) {
            return $default;
        }

        return str_replace("\0", '', $_COOKIE[$key]);
    }

    //! Set cookie value by key 
    /*!
     * @param string $key key in the cookie array
     * @param mixed $value value to store in the cookie
     * @param bool $overwrite overwrite the value if the key already exists
     * @param int $expire expire time
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public static function setCookie($key, $value, $overWrite = TRUE, $expire = HttpVar::DEFAULT_EXPIRE, $httpOnly = TRUE, $path = '/') {
        // sanity check
        if (!isset($key)    || !WizardValidator::isStringNotEmpty($key) ||
            !isset($value)  || !WizardValidator::isBool($overWrite)     ||
            !WizardValidator::isBool($httpOnly) || !WizardValidator::isInt($expire)) {

            WizardDebug::log('Function called incorrectly');
            return FALSE;
        }

        // check if overwrite is enabled
        if (!$overWrite && isset($_COOKIE[$key])) {
            return FALSE;
        }
       
        return setcookie($key, $value, time() + $expire, $path, '', TRUE, $httpOnly);
    }

    //! Remove cookie variable by key 
    /*!
     * @param string $key key in the cookie array
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     */
    public static function removeCookie($key, $httpOnly = TRUE, $path = '/') {
        // sanity check
        if (!isset($key) || !WizardValidator::isStringNotEmpty($key)) {
            return;
        }
        if (!isset($_COOKIE) || !isset($_COOKIE[$key])) {
            return;
        }

        unset($_COOKIE[$key]);
        setcookie($key, '', time() - 3600, $path, '', TRUE, $httpOnly);
    }

    //! Get request value by key
    /*!
     * @param string $key key in the request array
     * @param mixed $default default value to return if key is not found
     * @param string $priority Choose whether post or get should take priority
     *
     * @return mixed Request value by key
     *
     * @author HsuanWei Fu <hsuanwei@linuxmagic.com>
     *
     * Note: This get's only POST and/or GET
     */
    public static function getRequest($key, $default = NULL, $priority='post') {
        // sanity check
        if (!isset($key) || !WizardValidator::isStringNotEmpty($key)) {
            return $default;
        }

        // If priority is set to a string and is 'get' then we are prioritizing
        // GET attributes
        if (WizardValidator::isString($priority) && strtolower($priority) === 'get') {
            $first = 'getGet';
            $second = 'getPost';
        // If priority is not a string or isn't 'get' then we will default to a
        // priority of POST
        } else {
            $first = 'getPost';
            $second = 'getGet';
        }

        $request = NULL;

        if (($request = static::$first($key, $default)) === $default) {
            $request = static::$second($key, $default);
        }

        return $request;
    }

    //! Returns the country code for a given IP
    /*!
     * @param string $ipAddress the IP address that you want the lookup 
     * 
     * @returns string with '**' or the countr code 2 letters
     * 
     * @author Logan Borean <loganb@linuxmagic.com>
     */
    public static function getCountryForIpv4($ipAddress) {
        if (empty($ipAddress) || !WizardValidator::isIpv4($ipAddress)) {
            return '**';
        }
        // Check if there is hook
        if (function_exists('getCountryForIpv4Hook')) {
            // The function needs to return bool TRUE if it can resolve the country for
            // the $ipAdrress
            // the definition should be like
            // function getCountryForIpv4Hook($ipAddress, &$countryCode) { ... }
            // NOTE: this $countryCode is a ref
            $countryCode = '**';
            if (getCountryForIpv4Hook($ipAddress, $countryCode)) {
                return $countryCode;
            }
        }

        $mmDbLookup = new MmDbLookup();
        $lookupResult = $mmDbLookup->getCountryCode($ipAddress);

        if ($lookupResult !== FALSE) {
            return $lookupResult;
        }

        // if lookup failed, try and fall back to legacy method
        $legacyLookup = self::legacyGetCountryForIpv4($ipAddress);

        if ($legacyLookup !== FALSE) {
            return $legacyLookup;
        }

        return '**';
    }

    //! Helper function that will retrieve the country code from an IP v4 
    /*!
     * UPDATE: this is now the 'legacy' method 
     * 
     * @param string $ipAddress the IP address that you want the lookup 
     * 
     * @returns string with '**' or the countr code 2 letters
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function legacyGetCountryForIpv4($ipAddress) {
        if (empty($ipAddress) || !WizardValidator::isIpv4($ipAddress)) {
            return '**';
        }
        // XXX
        // For testing you can uncomment this line and use what ever country code country you want 
        // return 'CA';

        // If we don't have permission or the file is not readable this function will segfault
        // this path is hardcoded '/usr/share/GeoIP/GeoIP.dat'
        if (!function_exists('geoip_country_code_by_name') || !is_readable('/usr/share/GeoIP/GeoIP.dat')) {
            return '**';
        }
        // I am adding the STFU operator, just because this function
        // throws a PHP notice in case it can't find the host name
        // PHP Notice:  geoip_country_code_by_name(): Host 192.168.1.242 not found in /var/www/portal/controllers/auth.php on line 143
        $code = @geoip_country_code_by_name($ipAddress);
        if (empty($code)) {
            return '**';
        }
        return $code;

    }


    //! Brief: Helper method that retrieve the browser information that are required by the CID 
    /*!
     * @param string $forceUserAgent, optional the user agent if you to force a USer Agent 
     * 
     * @returns mixed array with the platfor, browser and device_type or bool FALSE
     * 
     * @author Rares Harnagea <rares@linuxmagic.com> 
     */
    public static function getBrowserInfo($forceUserAgent = NULL) {
        if ((empty($_SERVER) || empty($_SERVER['HTTP_USER_AGENT'])) && empty($forceUserAgent) ) {
            return;
        }

        // If there is no force user agent, let's pass the server user agent 
        // since there is some sort of a bug on get_browser() https://bugs.php.net/bug.php?id=69983
        $userAgent = empty($forceUserAgent) ? $_SERVER['HTTP_USER_AGENT'] : $forceUserAgent; 

        // attempt to get the browser information using http://browscap.org/    
        $browser = get_browser($forceUserAgent, true); 

        if (empty($browser) || !is_array($browser)) {
            return FALSE;
        }

        // we want to retrive the following info:
        // Browser name
        // Platform 
        // Device type  
        $wantedProps = array(
            // Linux
            'platform',
            // Firefox
            'browser',
            // Desktop
            'device_type',
        ); 
        $ret = array();

        // Check if the above properties exists and set them to the ret variable  
        foreach ($wantedProps as $prop) {
            if (isset($browser[$prop])) {
                $ret[$prop] = $browser[$prop];
            }
        }
        return $ret;
    }

    //! Brief: Helper method that return a string with the browser info, this is required by the MM Fingerprint 
    /*!
     * @param string $forceUserAgent, optional the user agent if you to force a User Agent 
     * 
     * @returns string with the browser info
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function getJsBrowserInfo($forceUserAgent = NULL) {
        $browserProps = self::getBrowserInfo($forceUserAgent);

        if (empty($browserProps)) {
            return 'Unknown';
        }

        return implode(', ', $browserProps);
    }
}
