<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2017-2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Logan Borean <loganb@linuxmagic.com>
 *
 * $Id$
 */

class MmDbLookup extends WizardDebug {

    //! @string with the ACL binary location
    const MM_DB_LOOKUP_BIN = '/usr/bin/mmdblookup';

    //! @string the path to the country code database file
    const DB_FILE_PATH = '/usr/share/GeoIP/GeoLite2-Country.mmdb';


    //! @array with the allowed arguments
    protected static $_allowedArgs = array(
        'file' => '--file',
        'ip'   => '--ip',
        'country' => 'country',
        'registered_country' => 'registered_country',
        'iso_code' => 'iso_code',
    );


    //! ctor
    /*!
     * @returns void
     * 
     * @author Logan Borean <loganb@linuxmagic.com>
     */
    public function __construct() {
        parent::__construct();
    }

    //! Executes the maxmind db lookup for the IP
    /*!
     * @param array $cmdArgs with the arguments we want to use on the UDC execution
     * 
     * @returns FALSE if unsuccessful, the country code if successful
     * 
     * @author Logan Borean <loganb@linuxmagic.com>
     */
    protected function _execMMDBLookup($cmdArgs) {
        if (!isset($cmdArgs) || !WizardValidator::isArrayNotEmpty($cmdArgs)) {
            $this->debug('The cmdArgs are expected as array');
            return FALSE;
        }
    
        if (!is_executable(self::MM_DB_LOOKUP_BIN)) {
            $this->debug("The acl utility is not installed");
            return FALSE;
        }

        $cmd = '';
        // we take only the allowed arguments from the $cmdArgs array
        foreach (self::$_allowedArgs as $arg => $opt) {
            if (!isset($cmdArgs[$arg])) {
                continue;
            }
            // we need to escape only the data that's comming from the user
            // $opt is something that we define in this class
            $cmd .= ' ' .  $opt;
            if (!empty($cmdArgs[$arg])) {
                $cmd .= ' ' . escapeshellarg($cmdArgs[$arg]);
            }
        }

        if (empty($cmd)) {
            $this->debug('You did not provide any allowed arguments ');
            return FALSE;
        }

        $cmd =  self::MM_DB_LOOKUP_BIN . ' ' . $cmd . ' 2> /dev/null';

        // init the exec arguments 
        $output = "";
        $rv = -1;

        exec($cmd, $output, $rv);

        if ($rv !== 0 || !is_array($output) || empty($output)) {
            $this->debug("MM DB lookup utility failed command [$cmd] with exit code: [$rv]");
            return FALSE;
        }
       
        $output = implode($output);

        $matches = array();
        if (preg_match('/"([^"]+)"/', $output, $matches) === FALSE) {
            $this->debug("MM DB lookup utility returned bad output [$cmd]");
            return FALSE;
        }

        if (strlen($matches[1]) !== 2) {
            $this->debug("MM DB lookup utility returned a country code with an unexpected length [$matches[1]]", self::ERROR);
            return FALSE;
        }

        return $matches[1];
    }


    
    //! Returns the country code an ip is in or FALSE
    /*!
     * @param array $ip the ip to find the country code for
     * 
     * @returns the country code or FALSE
     * 
     * @author Logan Borean <loganb@linuxmagic.com>
     */
    public function getCountryCode($ip) {
        if (!WizardValidator::isIpv4($ip) || !file_exists(self::DB_FILE_PATH)) {
            return FALSE;
        }

        $args = array(
            'ip' => $ip,
            'file' => self::DB_FILE_PATH,
            'country' => '',
            'iso_code' => '',
        );

        return $this->_execMMDBLookup($args);
    }
}

