<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2017 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id$
 */

class WizardBase extends WizardDebug {


    //! @char the prefix for member variable some of the members variable start with _ other 

    // classes implement starting with alpha directly 
    protected $_prefixMembers;

    //! array with allowed setter and the callbacks required for validation
    protected $_setters;
        // method name     // callbacks that you want to use for validation, they must return bool
    //    'setLimit'      => array('WizardValidator::isInt'),
    //    'setPeekLength' => array('WizardValidator::isInt'),

    //! array with allowed getter methods
    // For this to work you need to follow the standards naming convention getImapFlags() => _imap_flags
    protected $_getters;
        // getter name
    //  'getImap',
    //  'getLimit',

    //!
    /*! WizardObject()
     *
     * This constructor should be called with the PEAR::DB or PDO object.
     *
     * @param object $dbinstance  A value containing a valid PEAR::DB object.
     *
     * @return void
     */
    public function __construct() {
        parent::__construct();

        $this->_prefixMembers = '_'; 
        $this->_setters = array();
        $this->_getters = array();

    }


    //! Helper method to determine if $this object has a method, by looking at some dynamic methods 
    /*!
     * @param string $method the method name 
     *
     * @return bool success
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * NOTE: This is to overcome the weakness of method_exists() and __call() implementation 
     */
    public function hasMethod($method) {
        if (!isset($method) || !WizardValidator::isStringNotEmpty($method)) {
            $this->debug("Missing or invalid method argument!");
            return FALSE;
        } 
        // first check if the method is defined
        if (method_exists($this, $method)) {
            return TRUE;
        }
        // Why do we have these checks?
        // This is because we use the magic method __call() and will be impossible to
        // determine what method are implemented
        // so let's look if we setup a _setters or _getters array
        if (WizardValidator::isArrayNotEmpty($this->_setters) && isset($this->_setters[$method])) {
            return TRUE;
        }
        if (WizardValidator::isArrayNotEmpty($this->_getters) && isset($this->_gettters[$method])) {
            return TRUE;
        }
        return FALSE;
    }

    //! Helper method to convert member to method  
    /*!
     * @param string $member the member name that we want to convert  
     * @param string $prefix the prefix that we want to add
     *
     * @return mixed string with the method name or bool on failure 
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * Note: this will convert the member 'is_active', 'set' to setIsActive();
     *       PHP does ignore the case on methods for now but let's do a good job now
     */
    protected function memberToMethod($member, $prefix) {
        if (!isset($member) || !WizardValidator::isStringNotEmpty($member)) {
            $this->debug("Missing or invalid parameter member");
            return FALSE;
        }
        if (!isset($prefix) || !WizardValidator::isStringNotEmpty($prefix)) {
            $this->debug("Missing or invalid parameter prefix");
            return FALSE;
        }
        // if the member starts with a _ let's remove it
        if ($member[0] === '_') {
            $member = substr($member, 1);
        }

        $newMethod = $prefix;
        $upper = FALSE;
        // loop over the chars from this string and remove any '_'
        // and then upper case the next char
        // ex: is_active => IsActive
        for ($i = 0; $i < strlen($member); $i++) {
            // upper case the first char or if the upper flag is TRUE
            if ($i === 0 || $upper) {
                $newMethod .= strtoupper($member[$i]);
                $upper = FALSE;
            } elseif ($member[$i] === '_') {
                $upper = TRUE;
            } else {
                $newMethod .= $member[$i];
            }
        }
        return $newMethod;
    }

    //! helper method that transform a part of the method name into the member variable
    /*!
     * @param string $method the part of the $method for example ImapFlags from setImapFlags()
     *
     * @return string with the member variable in the above example case will return _imap_flags
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    protected function methodToMember($method, $prefix) {
        // sane the param
        if (!isset($method) || !WizardValidator::isStringNotEmpty($method)) {
            return FALSE;
        }
        if (!isset($prefix) || !WizardValidator::isStringNotEmpty($prefix)) {
            $this->debug("Missing or invalid parameter prefix");
            return FALSE;
        }
        $method = substr($method, strlen($prefix));
        if (empty($method)) {
            $this->debug("Empty method name after trimming the prefix");
            return FALSE;
        }
        // we want to convert Username to _username or ImapFlags to _imap_flags
        $var = '';
        for ($i = 0; $i < strlen($method); $i++) {
            $ord = ord($method[$i]);
            // is the char upper case letter
            if ($ord >= 65 && $ord <= 90) {
                // for first char we may or may not have a prefix
                // depending of the convention on that class
                if ($i === 0) {
                    $var .= $this->_prefixMembers . chr($ord + 32);
                } else {
                    // add the underline and convert the letter to lowercase
                    $var .= '_' . chr($ord + 32);
                }
            } else {
                $var .= $method[$i];
            }
        }
        return $var;
    }
    
    //! PHP magic method that we use to have controlled setters and getters for the member variables
    /*!
     * @param string $method the method that you called e.g. setUsername()
     * @param array $args the arguments tat you passed to the method
     *
     * @return mixed bool for setters if the variable is sane and was set, mixed for getters
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * @Throw Exception
     */
    public function __call($method, $args) {
        // if is starting with set and the method is a key in the _setters array
        // then is a legit setter call, now we need to validate the args
        if (substr($method, 0, 3) === 'set' && isset($this->_setters[$method])) {
            // check if they passed the argument for the setter
            if (!isset($args) || !WizardValidator::isArrayNotEmpty($args) || sizeof($args) !== 1) {
                $this->debug("The method [$method] is a setter and is expecting the argument to be set");
                return FALSE;
            }
            $param = $args[0];
            foreach ($this->_setters[$method] as $callback) {
                if ($this->callTarget($callback, $param) !== TRUE) {
                    $this->debug("The param [$param] is not valid for the method [$method], failed to validate against [$callback]!");
                    return FALSE;
                }
            }
            // If we are here the variable is sane
            // now we need to convert setUsername to _username to set the member variable
            $var = $this->methodToMember($method, 'set');
            // final stage, if the params is in the static $columns and is string we need 
            // to trim the variable to the max_length 
            if (isset(static::$columns) &&  isset(static::$columns[$var]) && static::$columns[$var]['type'] === 'string' && static::$columns[$var]['max_length'] !== NULL) {
                $maxLength = static::$columns[$var]['max_length'];
                if (strlen($param) > $maxLength) {
                    $param = substr($param, 0, $maxLength);
                }
            }        
            // because the value passed the validation,the value was set and we return TRUE
            $this->$var = $param;
            return TRUE;
        } elseif (substr($method, 0, '3') === 'get' && in_array($method, $this->_getters)) {
            $var = $this->methodToMember($method, 'get');
            return $this->$var;
        } else {
            // consider throwing an exception here
            $this->debug("Invalid method called [$method] on class " . get_class($this));
            return FALSE;
        }
    }

    //! helper method that can help us to validate a variable against a callable
    /*!
     * @param mixed $callable the callable method could be Closure, class static method, class method or method from this class
     * @param mixed $dataToValidate the data that need to be validated
     *
     * @return mixed bool what ever the data is valid or not
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    protected function callTarget(&$callable, &$dataToValidate) {
        if (!isset($callable) || !isset($dataToValidate)) {
            return;
        }

        // This is an Anonymous function, let's call it
        if ($callable instanceof Closure) {
            return $callable($dataToValidate);
        // We need to call a static method
        } elseif (is_string($callable) && strpos($callable, '::') !== FALSE) {
            list($class, $method) = explode('::', $callable, 2);
            return $class::$method($dataToValidate);

        // We need to call a method that requires an instance
        } elseif (is_string($callable) && strpos($callable, '@') !== FALSE) {
            list($class, $method) = explode('@', $callable, 2);
            $obj = new $class;
            return $obj->$method($dataToValidate);

        // We need to call a method from this class
        } elseif (is_string($callable) && strpos($callable, '$this->') !== FALSE) {
            list($class, $method) = explode('->', $callable, 2);
            // make sure that this method is callable
            if ($this->hasMethod($method)) {
                return $this->$method($dataToValidate);
            } 
        // We need to call a regular function
        } elseif (is_string($callable) && function_exists($callable)) {
            return $callable($dataToValidate);
        // This function is not callable 
        } else {
            $this->debug("The function [$callable] is not callable");
            return FALSE;
            //throw new Exception('Invalid callable, it not callable');
        }
    }
}
