<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id: lib.wizard_db_pdo.php 155 2017-03-07 22:36:08Z rares $
 *
 * This is an abstract DB class that follows PEAR syntax, but is for PDO 
 * implementation
 */


require_once(__DIR__ . '/interface.wiz_db_abstract.php');
require_once(__DIR__ . '/lib.wizard_debug.php');
require_once(__DIR__ . '/lib.wizard_db_pdo_sth.php');

class WizDbPdo extends WizardDebug implements WizDb {

    //! @object the instance of our Db
    protected $_db;

    //! @object the instance of the last STH to get
    // data that is not available to db
    protected $_sth;

    //! The constructor of our class
    /*!
     * @param object $instance an instance of the database connection object to avoid creating new connections
     * This constructor should be called with the PEAR::DB or PDO object.
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function __construct($instance = '') {
        parent::__construct();
        $this->_db = NULL;
        if (is_object($instance)) {
            $this->_db = $instance;
        }
    }

    //! Helper method to create a database connection and return the instance
    /*!
     * @param array $elements array with the username, database, password required to make the connection
     *
     * @return mixed object with the connection or NULL if it fails
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function getInstance($elements) {
        if (!isset($elements) || !is_array($elements)) {
            $this->debug("Missing or invalid arguments");
            return NULL;
        }

        $scheme     = isset($elements["sql_scheme"])     ? $elements["sql_scheme"]     : '';
        $database   = isset($elements["sql_database"])   ? $elements["sql_database"]   : '';
        $host       = isset($elements["sql_host"])       ? $elements["sql_host"]       : '';
        $port       = isset($elements["sql_port"])       ? $elements["sql_port"]       : '';
        $username   = isset($elements["sql_username"])   ? $elements["sql_username"]   : '';
        $password   = isset($elements["sql_password"])   ? $elements["sql_password"]   : '';
        $persistent = isset($elements["sql_persistent"]) ? $elements["sql_persistent"] : FALSE;

        // XXX This is implemented only for Postgres for now
        // TODO add implementation for MySQL 
        if ($scheme === 'pgsql_pdo') {
            $dsn = "pgsql:host=$host;dbname=$database;port=$port";
        } else {
            $dsn = "mysql:host=$host;dbname=$database;";
        }

        $opt = array(
            PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_NUM,
            PDO::ATTR_PERSISTENT         => $persistent,
            // Error reporting :  Just set error codes. no exception           
            PDO::ATTR_ERRMODE            => PDO::ERRMODE_SILENT,
        );

        // This needs to be wrapped up in a try catch block, because otherwise will throw exception
        // the exeception are thrown only on constructor, then the object will use the error reporing
        // value
        try {
            $db = new PDO($dsn, $username, $password, $opt);
        } catch (PDOException $e) {
            WizardDebug::log("($database): DB::connect() failed: -- " . $e->getMessage());
            return NULL;
        }

        if (!$db || !is_object($db)) {
            $this->debug("Unable to create a PDO connection");
            return NULL;
        }
        return $db;
    }

    //! Helper method to translate the fetch method to PEAR
    /*!
     * @param int $mode, the fetch mode type numeric or associative array
     *
     * @return mixed array with a result or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    protected function _translateFetchMethod($mode = '') {
        if (!isset($mode) || empty($mode) || $mode === self::FETCH_NUM) {
            // these are PDO constants
            return PDO::FETCH_NUM;
        } elseif ($mode === self::FETCH_ASSOC) {
            return PDO::FETCH_ASSOC;
        }

        // in the worst case return fetch ordered
        return PDO::FETCH_NUM;
    }

    //! Helper method to check if there was an error while running a query
    /*!
     * @param object $object the object that we want to check
     *
     * @return mixed TRUE if there was an error while running a query, FALSE otherwise.
     *               NULL if this function failed.
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function isError($object) {
        if (!isset($object) || !is_object($object)) {
            return NULL;
        }
        if (is_a($object, 'WizDbPDOSth')) {
            return $object->getSth()->errorCode() !== PDO::ERR_NONE;
        }

        if (is_a($object, 'WizDbPDO')) {
            return $object->getDb()->errorCode() !== PDO::ERR_NONE;
        }

        return NULL;
    }



    //! Method used to return the error message if there was an error
    /*!
     * @return string the message. Otherwise empty string
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getMessage() {
        $mess = $this->_db->errorInfo();
        if (isset($mess) && is_array($mess) && isset($mess[2])) {
            return $mess[2];
        }
        return '';
    }

    //! Method used to return the additional information about an error message
    /*!
     * @return string the message
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * Note: There is no such equivalent function in PDO
     * this is what PDO::errorInfo() returns array with 3 index 
     * 0     SQLSTATE error code (a five characters alphanumeric identifier defined 
     *       in the ANSI SQL standard).
     * 1   Driver-specific error code.
     * 2   Driver-specific error message.
     */
    public function getUserInfo() {
        return $this->getMessage();
    }

    //! Helper method to make a query, this could be used to do prepare statements if the second argument is passed
    /*!
     * @param string $sql the SQL statement that you want to run
     *
     * @param array $params array with arguments you want to bind arguments
     *
     * @param bool $bind_type if we want to bind by type, this is required for bool PDO
     *
     * @return mixed the result set or object with error or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * The third argument $bind_type is not used here
     */
    public function query($sql, $params = array(), $bind_type = FALSE) {
        if (!isset($sql) ||  !WizardValidator::isStringNotEmpty($sql) || !is_array($params)) {
            $this->debug('Invalid arguments');
            return NULL;
        }

        $sth = $this->_db->prepare($sql);

        $obj = new WizDbPdoSth($sth);
        // convert HASH to indexed array 
        $params = array_values($params);

        if ($bind_type === TRUE) {
            // bind the arguments by their type
            // XXX this function will assume that the params values are case correctly 
            // to match the type from DB
            $this->bindByType($obj, $params);
            // note that we don't pass the params to execute
            $obj->getSth()->execute();

        } else {
            $obj->getSth()->execute($params);
        }

        // this is required to retrieve later attributes or methods that are not implemented by
        // the PDO class and are implemented on the PDO statement class
        $this->_sth = $obj;

        return $obj;
    }

    //! Helper method to prepare a statement
    /*!
     * @param string $sql the SQL statement that you want to run
     *
     * @return mixed the prepare statements object or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function prepare($sql) {
        if (!isset($sql) || !WizardValidator::isStringNotEmpty($sql)) {
            $this->debug('Invalid arguments');
            return NULL;
        }
        $sth = $this->_db->prepare($sql);
        $obj = new WizDbPdoSth($sth);

        // this is required to retrieve later attributes or methods that are not implemented by
        // the PDO class and are implemented on the PDO statement class
        $this->_sth = $obj;

        return $obj;
    }

    //! Method used to free the prepared statement
    /*!
     * @param object $sth the prepared statement that we want to free
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function freePrepared($sth) {
        if (isset($sth)) {
            unset($sth);
        }
    }

    //! Helper method to execute a prepare statement
    /*!
     * @param object $sth the prepare statement object
     *
     * @param array $params array with argument that you want to bind
     *
     * @param bool $bind_type if we want to bind by type, this is required for bool PDO
     *
     * @return mixed the object with the results set object or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * Note: This should be in STH, but on PEAR this is here since the STH is passed as argument
     * The third argument $bind_type is not used here
     */
    public function execute($sth, $params = array(), $bind_type = FALSE) {
        if (!isset($sth) || !is_object($sth) || !is_a($sth, 'WizDbPdoSth') || $sth->getSth() === NULL) {
            $this->debug('Invalid arguments');
            return NULL;
        }
        if (!is_array($params)) {
            $this->debug('Execute was called with invalid params arguments, expected array');
            $params = array();
        } else {
            // convert HASH to indexed array 
            $params = array_values($params);
        }
        
        if ($bind_type === TRUE) {
            // bind the arguments by their type
            // XXX this function will assume that the params values are case correctly 
            // to match the type from DB
            $this->bindByType($sth, $params);
            // note that we don't pass the params to execute
            $sth->getSth()->execute();

        } else {
            $sth->getSth()->execute($params);
        }
        // this is not working in PDO, because execute return bool

        return $sth;
    }

    //! Helper method to bind arguments by their type to a prepare statement
    /*!
     * @param object $sth the prepare statement object
     *
     * @param array $params array with argument that you want to bind
     *
     * @return mixed int with the number of affected rows
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * Note: this function assume the arguments are cast to correct type
     */
    protected function bindByType($sth, $params) {
        if (!isset($sth) || !is_object($sth) || !is_a($sth, 'WizDbPdoSth') || $sth->getSth() === NULL) {
            $this->debug('Invalid arguments');
            return NULL;
        }
        
        if (!isset($params) || !is_array($params)) {
            $this->debug('Execute was called with invalid params arguments, expected array');
        }
        $index = 1;
        // bind the arguments by their type
        foreach ($params as $value) {
            
            if (is_string($value)) {
                $sth->getSth()->bindValue($index,  $value, PDO::PARAM_STR);
            } elseif (is_int($value)) {
                $sth->getSth()->bindValue($index,  $value, PDO::PARAM_INT);
            } elseif (is_bool($value)) {
                $sth->getSth()->bindValue($index,  $value, PDO::PARAM_BOOL);
            } else {
                $sth->getSth()->bindValue($index,  $value, PDO::PARAM_NULL);
            }
            $index++;
        }
    }

    //! Method used to get the number of rows from a INSERT, UPDATE, DELETE statement
    /*!
     * @return mixed int with the number of affected rows
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function affectedRows() {
        // This need to be audited to see if is working
        return $this->_sth->rowCount();
    }


    //! Helper method to quote a variable and prepare for a SQL 
    /*!
     * @param string $var the variable to quote
     *
     * @return mixed the quoted variable 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function quote($var) {
        if (!isset($var)) {
            return NULL;
        }
        return $this->_db->quote($var);
        
     }

    //! Helper method to get the next Id from a table 
    /*!
     * @return mixed the quoted variable 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * XXX This should be deprecated, but there are libraries which are using this
     */
     public function nextId($sequence, $onDemand = TRUE) {
        if (!isset($sequence) || !WizardValidator::isStringNotEmpty($sequence)) {
            return NULL;
        }
        $sql = 'SELECT NEXTVAL(?) AS value';
        $params = array($sequence);
        return $this->getOne($sql, $params);
     }



    //! Helper method to turn on or off the autocommit
    /*!
     * @param bool $onOff, method used to start transactions
     *
     * @return bool what ever the autocommit was set
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function autoCommit($onOff = FALSE) {
        // sane the argument
        if (!is_bool($onOff)) {
            return FALSE;
        }
        // so this is used when we want to start transaction
        if ($onOff === FALSE) {
            return $this->begin();
        }
        // for autoCommit(TRUE) I don't think that we need to do anything
        return TRUE;
    }

    //! Helper method to that will determine if we are in a transaction
    /*!
     * @return bool whether we are in the middle of a DB transaction 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function inTransaction() {
        return $this->_db->inTransaction();
    }

    //! Helper method to begin a transactions to DB
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function begin() {
        if ($this->_db->query('BEGIN') === FALSE) {
            $this->debug("failed to begin transaction: " . $res->getMessage());
            return FALSE;
        }
        return TRUE;
    }

    //! Helper method to commit a set of transactions to DB
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function commit() {
        if ($this->_db->query('COMMIT') === FALSE) {
            $this->debug("failed to commit transaction: " . $res->getMessage());
            return FALSE;
        }
        return TRUE;
    }

    //! Helper method to rollback a set of transactions to DB
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function rollback() {
        // call straight the db
        if ($this->_db->query('ROLLBACK') === FALSE) {
            $this->debug("failed to rollback transaction: " . $res->getMessage());
            return FALSE;
        }
        return TRUE;
    }

    //! Getter method to return the acctualy DB
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getDb() {
        return $this->_db;
    }

    //! Helper method to fetch the first column of the first row from a query result
    /*!
     * @param object $sth the prepare statement object
     *
     * @param array $params array with argument that you want to bind
     *
     * @param bool $bind_type if we want to bind by type, this is required for bool PDO
     *
     * @return mixed the value from first column or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getOne($sql, $params = array(), $bind_type = FALSE) {
        if (!isset($sql) ||  !WizardValidator::isStringNotEmpty($sql) || !is_array($params)) {
            return NULL;
        }
        $result = $this->query($sql, $params, $bind_type);

        if (static::isError($result)) {
            return NULL;
        }
        $row = $result->fetchRow();
        // now check if there is a row and that we have a first column
        if (!isset($row) || !is_array($row) || !isset($row[0])) {
            return NULL;
        }
        return $row[0];
    }

    //! Helper method to fetch all of the rows from a query result
    /*!
     * @param object $sth the prepare statement object
     *
     * @param array $params array with argument that you want to bind
     *
     * @param int $fetchMode the fetch method that you prefer, 0 default
     *
     * @param bool $bind_type if we want tot bind by type, this is required for bool PDO
     *
     * @return mixed array with results or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getAll($sql, $params = array(), $fetchmode = 0, $bind_type = FALSE) {
        if (!isset($sql) ||  !WizardValidator::isStringNotEmpty($sql) || !is_array($params)) {
            $this->debug('Invalid arguments');
            return NULL;
        }
        $result = $this->query($sql, $params, $bind_type);

        if (static::isError($result)) {
            return NULL;
        }
        $rows = array();
        while ($row = $result->fetchRow($this->_translateFetchMethod($fetchmode))) {
            $rows[] = $row;
        }
        return $rows;
    }
    
    //! Helper method to quick fetch one rows from a query result
    /*!
     * @param object $sth the prepare statement object
     *
     * @param array $params array with argument that you want to bind
     *
     * @param int $fetchMode the fetch method that you prefer, 0 default
     *
     * @param bool $bind_type if we want to bind by type, this is required for bool PDO
     *
     * @return mixed array with results or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getRow($sql, $params = array(), $fetchmode = 0, $bind_type = FALSE) {
        if (!isset($sql) ||  !WizardValidator::isStringNotEmpty($sql) || !is_array($params)) {
            $this->debug('Invalid arguments');
            return NULL;
        }
        $result = $this->query($sql, $params, $bind_type);

        if (static::isError($result)) {
            return NULL;
        }
        return $result->fetchRow($this->_translateFetchMethod($fetchmode));
    }
}
