<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id: lib.wizard_db_pdo_sth.php 155 2017-03-07 22:36:08Z rares $
 *
 * This is an abstract DB class that follows PEAR syntax, but is for PDO
 * implementation
 */


require_once(__DIR__ . '/interface.wiz_db_abstract.php');
require_once(__DIR__ . '/lib.wizard_debug.php');

class WizDbPdoSth extends WizardDebug implements WizDbSth {

    //! @object with the result set
    protected $_sth;

    //! The constructor of our class
    /*!
     * @param mixed $sth int if this is a prepared statement or object if is a result set
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function __construct($sth = NULL) {
        parent::__construct();
        // $sth could 
        $this->_sth = $sth;
    }

    //! Helper method to translate the fetch method to PEAR
    /*!
     * @param int $mode, the fetch mode type numeric or associative array
     *
     * @return mixed array with a result or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    protected function _translateFetchMethod($mode = '') {
        if (!isset($mode) || empty($mode) || $mode === self::FETCH_NUM) {
            // these are PDO constants
            return PDO::FETCH_NUM;
        } elseif ($mode === self::FETCH_ASSOC) {
            return PDO::FETCH_ASSOC;
        }

        // in the worst case return fetch ordered
        return PDO::FETCH_NUM;
    }


    //! Method used to get the number of rows from a INSERT, UPDATE, DELETE statement
    /*!
     * @return mixed int with the number of affected rows
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * XXX This is PDO specific
     */
    public function rowCount() {
        // This need to be audited to see if is working
        return $this->_sth->rowCount();
    }


    //! Method used to fetchRow records from a results set
    /*!
     * @param int $mode, the fetch mode type numeric or associative array
     *
     * @return mixed array with a result or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function &fetchRow($mode = 0) {
        // we need NULL set to a variable since we cannot return
        // a static NULL in this case - due to this function returning
        // a variable reference.
        $null = NULL;
        if (!isset($this->_sth)) {
            return $null;
        }
        $ret = $this->_sth->fetch($this->_translateFetchMethod($mode));
        if (is_null($ret) || $ret === FALSE) {
            return $null;
        }
        return $ret;
    }

    //! Method used to fetch alias for fetchRow records from a results set
    /*!
     * @param int $mode, the fetch mode type numeric or associative array
     *
     * @return mixed array with a result or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function &fetch($mode = 0) {
        // alias for fetchRow
        return $this->fetchRow($mode);
    }


    //! Method used to return the error message if there was an error
    /*!
     * @return mixed string with the message or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getMessage() {
        $mess = $this->_sth->errorInfo();
        if (isset($mess) && is_array($mess) && isset($mess[2])) {
            return $mess[2];
        }
        return '';
    }


    //! Method used to get the number of rows from a SELECT statement
    /*!
     * @return mixed int with the number or rows from the last query or NULL id there was a problem
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function numRows() {
        return $this->rowCount();
    }

    //! Method used to free the memory with the last statement
    /*!
     * @return void
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function free() {
        return $this->_sth->closeCursor();
    }

    //! Helper method to free the memory with the results
    /*!
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function freeResult($result) {
        if (isset($result)) {
            unset($result);
        }
     }

    //! Getter method to return the acctualy DB
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getSth() {
        return $this->_sth;
    }
}
