<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id: lib.wizard_db_pear.php 154 2017-03-07 18:24:37Z rares $
 *
 * This is an abstract DB class that follows PEAR syntax
 */


require_once(__DIR__ . '/interface.wiz_db_abstract.php');
require_once(__DIR__ . '/lib.wizard_debug.php');
require_once(__DIR__ . '/lib.wizard_db_pear_sth.php');

class WizDbPear extends WizardDebug implements WizDb {

    //! @object the instance of our Db
    protected $_db;

    //! The constructor of our class
    /*!
     * @param object $instance an instance of the database connection object to avoid creating new connections
     *                          This constructor should be called with the PEAR::DB or PDO object.
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function __construct($instance = '') {
        parent::__construct();
        $this->_db = NULL;
        if (is_object($instance)) {
            $this->_db = $instance;
        }
    }

    //! Helper method to create a database connection and return the instance
    /*!
     * @param array $elements array with the username, database, password required to make the connection
     *
     * @return mixed object with the connection or NULL if it fails
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function getInstance($elements) {
        if (!isset($elements) || !is_array($elements)) {
            return NULL;
        }
        // dependency on PEAR DB package
        require_once "PEAR.php";
        require_once "DB.php";

        $scheme     = isset($elements["sql_scheme"])     ? $elements["sql_scheme"]     : '';
        $database   = isset($elements["sql_database"])   ? $elements["sql_database"]   : '';
        $host       = isset($elements["sql_host"])       ? $elements["sql_host"]       : '';
        $port       = isset($elements["sql_port"])       ? $elements["sql_port"]       : '';
        $username   = isset($elements["sql_username"])   ? $elements["sql_username"]   : '';
        $password   = isset($elements["sql_password"])   ? $elements["sql_password"]   : '';
        $persistent = isset($elements["sql_persistent"]) ? $elements["sql_persistent"] : FALSE;

        if ($username) {
            $user_section = $username;
            if ($password) {
                $user_section .= ":". $password;
            }
            $user_section .= "@";
        }

        $options = array(
            'optimize'   => 'portability',
            'persistent' => $persistent,
        );

        switch ($scheme) {
            case "pgsql":
                if (!defined('CURRENT_USER_FUNC')) {
                    define('CURRENT_USER_FUNC', 'current_user');
                }
            case "mysql":
                if ($scheme != 'pgsql') {
                    if (!defined('CURRENT_USER_FUNC')) {
                        define('CURRENT_USER_FUNC', 'user()');
                    }
                }

                if ($host) {
                    $host_section = $host;
                    if ($port) {
                        $host_section .= ":". $port;
                    }
                } else {
                    $this->debug(".getDBInstance($database): " . "No database host specified.");
                    return NULL;
                }

                $dsn = "${scheme}://${user_section}${host_section}/${database}";
                break;
            case "oci8":
                if (!defined('CURRENT_USER_FUNC')) {
                    define('CURRENT_USER_FUNC', 'user');
                }
                // Oracle uses the format "oci8://username:password/database" and does
                // not use the host specification
                $dsn = "${scheme}://${user_section}${database}";
                break;
            default:
                break;

        }

        $dbObj = new DB();
        $db = $dbObj->connect($dsn, $options);

        if (!$db || !is_object($db)) {
            return NULL;
        }
        return $db;
    }

    //! Helper method to translate the fetch method to PEAR
    /*!
     * @param int $mode the fetch mode type numeric or associative array
     *
     * @return mixed array with a result or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    protected function _translateFetchMethod($mode = '') {
        if (!isset($mode) || empty($mode) || $mode === self::FETCH_NUM) {
            // these are PEAR constants
            return DB_FETCHMODE_ORDERED;
        } elseif ($mode === self::FETCH_ASSOC || $mode === DB_FETCHMODE_ASSOC) {
            return DB_FETCHMODE_ASSOC;
        }

        // in the worst case return fetch ordered
        return DB_FETCHMODE_ORDERED;
    }

    //! Helper method to check if there was an error while running a query
    /*!
     * @param object $object the object that we want to check
     *
     * @return mixed TRUE on error, FALSE on no error. NULL if this function failed
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function isError($object) {
        if (!isset($object) || !is_object($object)) {
            return NULL;
        }
        $obj = new PEAR();
        if (is_a($object, 'WizDbPearSth')) {
            return $obj->isError($object->getSth());
        }

        if (is_a($object, 'WizDbPear')) {
            return $obj->isError($object->getDb());
        }

        if (is_a($object, 'DB_Error')) {
            return $obj->isError($object->getDb());
        }

        return NULL;
    }

    //! Method used to return the error message if there was an error
    /*!
     * @return mixed string with the message or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getMessage() {
        return $this->_db->getMessage();
    }

    //! Method used to return the additional information about an error message
    /*!
     * @return mixed string with the message or NULL  
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getUserInfo() {
        return $this->_db->getUserInfo();
    }

    //! Helper method to make a query, this could be used to do prepare statements if the second argument is passed
    /*!
     * @param string $query the SQL statement that you want to run
     *
     * @param array $params array with arguments you want to bind
     *
     * @param bool $bind_type if we want to bind by type, this is required for bool PDO
     *
     * @return mixed the result set or object with error or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * The third argument $bind_type is not used here
     */
    public function query($sql, $params = array(), $bind_type = FALSE) {
        if (!isset($sql) ||  !WizardValidator::isStringNotEmpty($sql) || !is_array($params)) {
            $this->debug('Invalid arguments');
            return NULL;
        }

        $result = $this->_db->query($sql, $params);
        $obj = new WizDbPearSth($result);
        return $obj;
    }

    //! Helper method to prepare a statement
    /*!
     * @param string $sql the SQL statement that you want to run
     *
     * @return mixed the prepare statements object or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function prepare($sql) {
        if (!isset($sql) || !WizardValidator::isStringNotEmpty($sql)) {
            return NULL;
        }
        $sth = $this->_db->prepare($sql);
        $obj = new WizDbPearSth($sth);
        return $obj;
    }

    //! Method used to free the prepared statement
    /*!
     * @param object $sth the prepared statement that we want to free
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function freePrepared($sth) {
        if (isset($sth)) {
            unset($sth);
        }
    }

    //! Helper method to execute a prepare statement
    /*!
     * @param object $sth the prepare statement object
     *
     * @param array $params array with argument that you want to bind
     *
     * @param bool $bind_type if we want to bind by type, this is requried for bool PDO
     *
     * @return mixed the object with the results set object or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * Note: This should be in STH, but on PEAR this is here since the STH is passed as argument
     * The third argument $bind_type is not used here
     */
    public function execute($sth, $params = array(), $bind_type = FALSE) {
        if (!isset($sth) || !is_object($sth) || !is_a($sth, 'WizDbPearSth') || $sth->getSth() === NULL) {
            return NULL;
        }
        if (!is_array($params)) {
            $this->debug('Execute was called the params were not array ' . $params);
        }
        // This is really redundant using the PEAR syntax
        $resSth = $this->_db->execute($sth->getSth(), $params);
        if (is_int($resSth)) {
            // this is equal with DB_OK
            return $resSth; 
        }
        $ret = new WizDbPearSth($resSth);
        return $ret;
    }

    //! Method used to get the number of rows from a INSERT, UPDATE, DELETE statement
    /*!
     * @return mixed int with the number of affected rows
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function affectedRows() {
        return $this->_db->affectedRows();
    }


    //! Helper method to quote a variable and prepare for a SQL 
    /*!
     * @param string $var the variable to quote 
     *
     * @return mixed the quoted variable 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
     public function quote($var) {
        if (!isset($var)) {
            return NULL;
        }
        return $this->_db->quote($var);
        
     }

    //! Helper method to get the next Id from a table 
    /*!
     * @return mixed the next Id on success, NULL on failure
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * XXX This should be deprecated, but there are libraries which are using this
     */
     public function nextId($sequence, $onDemand = TRUE) {
        if (!isset($sequence) || !WizardValidator::isStringNotEmpty($sequence)) {
            return NULL;
        }
        trigger_error('This function is deprecated, use instead WizardObject::getNextSequenceValue($sequence)!');
        return $this->_db->nextId($sequence, $onDemand);
     }



    //! Helper method to turn on or off the autocommit
    /*!
     * @param bool $onOff, method used to start transactions
     *
     * @return bool what ever the autocommit was set
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function autoCommit($onOff = FALSE) {
        // sane the argument
        if (!is_bool($onOff)) {
            return FALSE;
        }
        return $this->_db->autoCommit($onOff) === DB_OK ? TRUE : FALSE;
    }

    //! Helper method to that will determine if we are in a transaction
    /*!
     * @return NULL 
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     * 
     * Note: This function is not implemented for PEAR
     */
    public function inTransaction() {
        trigger_error("InTransaction is not implemented for PEAR Interface, switch to PDO");
        return NULL;
    }
    //! Helper method to begin a transactions to DB
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function begin() {
        if (($res = $this->_db->query('BEGIN')) !== DB_OK) {
            $this->debug("failed to begin transaction: " . $res->getMessage());
            return FALSE;
        }
        return TRUE;
    }

    //! Helper method to commit a set of transactions to DB
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function commit() {
        if (($res = $this->_db->query('COMMIT')) !== DB_OK) {
            $this->debug("failed to commit transaction: " . $res->getMessage());
            return FALSE;
        }
        return TRUE;
    }

    //! Helper method to rollback a set of transactions to DB
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function rollback() {
        // call straight the db
        if (($res = $this->_db->query('ROLLBACK')) !== DB_OK) {
            $this->debug("failed to rollback transaction: " . $res->getMessage());
            return FALSE;
        }
        return TRUE;
    }

    //! Getter method to return the acctualy DB
    /*!
     * @return bool what ever the commit was successful
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getDb() {
        return $this->_db;
    }

    //! Helper method to fetch the first column of the first row from a query result
    /*!
     * @param object $sth the prepare statement object
     *
     * @param array $params array with arguments that you want to bind
     *
     * @param bool $bind_type if we want to bind by type, this is requried for bool PDO
     *
     * @return mixed the value from first column or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getOne($sql, $params = array(), $bind_type = FALSE) {
        if (!isset($sql) ||  !WizardValidator::isStringNotEmpty($sql) || !is_array($params)) {
            return NULL;
        }
        return $this->_db->getOne($sql, $params);
    }

    //! Helper method to fetch all of the rows from a query result
    /*!
     * @param object $sth the prepare statement object
     *
     * @param array $params array with argument that you want to bind
     * 
     * @param int $fetchmode the fetch method dthat you prefer, 0 default
     *
     * @param bool $bind_type if we weant to bind by type, this is required for bool PDO
     *
     * @return mixed array with results or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getAll($sql, $params = array(), $fetchmode = 0, $bind_type = FALSE) {
        if (!isset($sql) ||  !WizardValidator::isStringNotEmpty($sql) || !is_array($params)) {
            return NULL;
        }
        return $this->_db->getAll($sql, $params, $this->_translateFetchMethod($fetchmode));
    }
    
    //! Helper method to quick fetch one rows from a query result
    /*!
     * @param object $sth the prepare statement object
     *
     * @param array $params array with argumetns that you want to bind
     *
     * @param int $fetchmode the fetch method that you prefer, 0 default
     *
     * @param bool $bind_type if we want to bind by type, this is required for bool PDO
     *
     * @return mixed array with results or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function getRow($sql, $params = array(), $fetchmode = 0, $bind_type = FALSE) {
        if (!isset($sql) ||  !WizardValidator::isStringNotEmpty($sql) || !is_array($params)) {
            $this->debug('Invalid arguments');
            return NULL;
        }
        return $this->_db->getRow($sql, $params, $this->_translateFetchMethod($fetchmode));
    }
}
