<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicMail (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2016-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id: lib.wizard_debug.php 159 2017-03-28 23:15:31Z rares $
 *
 * Usage:
 * // Create a new object
 * $debugObj = new WizardDebug();
 * 
 * // set a file where you want to log the message
 * // make sure that the file exists and is writable by Apache2
 * $debugObj->setLogFile('logfile');
 * 
 * // set the debug flag
 * $debugObj->setDebug(TRUE);
 * 
 * // use this flag if you want to dump the messages each time when you
 * // call WizardDebug::debug();
 * $debugObj->setShowDebug(TRUE);
 * // alternative is to use $debugObj->dumpDebug(); to dump all the messages at once
 * 
 * // method to choose the format on dumpDebug() and setShowDebug();, by default is HTML
 * $debugObj->setShowDebugFormat(WizardDebug::DISPLAY_TEXT);
 * 
 * // method to filter the messages that are buffered or logged 
 * $debugObj->setDebugLevel(WizardDebug::WARNING);
 * 
 * // call to debug, the class, function and line number are added automatic
 * // the level in this case is WizardDebug::DEBUG
 * $debugObj->debug('This is a debug message');
 * 
 * // same as above but with a different level
 * $debugObj->debug('This is a debug message with the level of WARNING', WizardDebug::WARNING);
 * 
 * // helper method to retrieve all the debug messages
 * $myArray = $debugObj->getDebugMessages();
 */


require_once(__DIR__ . '/lib.wizard_validator.php');

class WizardDebug {

    // definitions for display formats: useful for setting flags
    const DISPLAY_HTML = 1; 
    const DISPLAY_TEXT = 0;
    

    // definitions for log level: useful when we want to store message 
    // with a specific level of importance 
    const ALERT   = 4;
    const ERROR   = 3;
    const WARNING = 2;
    const INFO    = 1;
    const DEBUG   = 0;
    
    protected static $level_translate = array(
        // translate array and checker if the level is allowed
        4 => 'ALERT',
        3 => 'ERROR',
        2 => 'WARNING',
        1 => 'INFO',
        0 => 'DEBUG',
    );

    //! @array  
    // variable where we store all the messages
    // each message is a array element
    protected $_debug_messages;

    //! @string
    // by default this is null, but if set then messages are logged to a file
    protected static $_log_file = NULL;

    //! @bool 
    // this is the debug flag, if FALSE the debug messages will not get buffered or dump to display
    protected static $_debug_flag = FALSE;

    //! bool
    // This flag is useful if the flag _debug_flag is set to TRUE, then each message will be dumped to 
    // display when you call the debug function
    protected $_debug_show;

    //! int
    // To avoid pollution, you can set a level for the messages that you want to be displayed and/or buffered
    protected static $_debug_level = self::DEBUG;

    //! int 
    // Store the debug format for output HTML or plain text
    protected $_debug_format;

    //! the constructor of this class that will init all the member variables to their default values
    /*!
     */
    protected function __construct() {

        // init the array where we store the messages
        $this->_debug_messages = array();
        // by default we do not dump the debug 
        $this->_debug_show = FALSE;

        // by default the debug format should be HTML
        $this->_debug_format = self::DISPLAY_HTML;
    }

    //! function used to print debug message
    /*!
     * @param string $message  The message to output.
     * @param int $level The message leve, by default is DEBUG
     *
     * Note: If debug mode is on, then print out the debug message.
     */
    public function debug($message, $level = 0) {       
        // do we have a message
        if (!isset($message) || WizardValidator::isStringNotEmpty($message) === FALSE) {
            return;
        }
        // check the level and debug level to see if we need to do something with this message 
        if (WizardValidator::isIntPositiveOrZero($level) === FALSE || $level < static::$_debug_level 
            || !isset(self::$level_translate[$level])) {
            return;
        }

        // we clearly do not want to log or store the message
        if (static::$_debug_flag === FALSE && static::$_log_file === NULL) {
            return;
        }

        $mess = "";
        // get the caller function
        $backtrace = debug_backtrace();

        if (count($backtrace) < 2) {
            $fn = "unknown function";
            $line = $backtrace[0]['line'];
        } else {
            $fn = $backtrace[1]['function'];
            // seems that may not have line in some cases...
            // if we go higher up the tree we do, but is it reliable
            // to do so?
            // this at least occurs when calling via a callback to this debug
            // function in a non-class
            if (array_key_exists('line', $backtrace[0])) {
                $line = $backtrace[0]['line'];
            } else {
                $line = 'unknown';
            }
        }

        $thisClass = isset($backtrace[1]['class']) ? $backtrace[1]['class'] : '';
        $mess = '[' . self::$level_translate[$level] . '] ';
        $mess .= "$thisClass $fn: (line $line): $message";
        // DO we want just to log to file or we want to buffer the message 
        if (static::$_debug_flag === TRUE) {
            if ($this->_debug_show === TRUE) {
                print $this->wrapMessage($mess);
            } else {
                // let's buffer the message
                $this->_debug_messages[] = $mess;
            }
        }
        // check if there is a log file and if the directory is writable 
        if (static::$_log_file !== NULL && is_writable(dirname(static::$_log_file))) {
            $date = date('M d H:i:s');
            $line = "[$date] : $mess\n"; 
            // this could fail but there is nothing we can do at this point
            // the file should be writable since we check for that when we set the file
            file_put_contents(static::$_log_file, $line, FILE_APPEND | LOCK_EX);
        }
    }

    //! Helper to log message without an instance
    /*!
     * @param string $message the message that you want to log
     * @param string $level the level of the message
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public static function log($message, $level = 0) {
        // do we have a message
        if (!isset($message) || WizardValidator::isStringNotEmpty($message) === FALSE) {
            return;
        }
        // check the level and debug level to see if we need to do something with this message 
        if (WizardValidator::isIntPositiveOrZero($level) === FALSE || $level < static::$_debug_level 
            || !isset(self::$level_translate[$level])) {
            return;
        }

        // we clearly do not want to log or store the message
        if (static::$_debug_flag === FALSE && static::$_log_file === NULL) {
            return;
        }

        $mess = "";

        // get the caller function
        $backtrace = debug_backtrace();


        if (count($backtrace) < 2) {
            $fn = "unknown function";
            $line = $backtrace[0]['line'];
        } else {
            $fn = $backtrace[1]['function'];
            // seems that may not have line in some cases...
            // if we go higher up the tree we do, but is it reliable
            // to do so?
            // this at least occurs when calling via a callback to this debug
            // function in a non-class
            if (array_key_exists('line', $backtrace[0])) {
                $line = $backtrace[0]['line'];
            } else {
                $line = 'unknown';
            }
        }

        $thisClass = isset($backtrace[1]['class']) ? $backtrace[1]['class'] : '';
        $mess = '[' . self::$level_translate[$level] . '] ';
        $mess .= "$thisClass $fn: (line $line): $message";
        // DO we want just to log to file or we want to buffer the message 
        if (static::$_debug_flag === TRUE) {
            print $mess . "\n";
        }

        if (static::$_log_file !== NULL && is_writable(dirname(static::$_log_file))) {
            $date = date('M d H:i:s');
            $line = "[$date] : $mess\n"; 
            // this could fail but there is nothing we can do at this point
            // the file should be writable since we check for that when we set the file
            file_put_contents(static::$_log_file, $line, FILE_APPEND | LOCK_EX);
        }
    }

    //! helper function to wrap a message in plain text or HTML
    /*!
     * @param string $message  The message to output.
     *
     * Note: If debug mode is on, then print out the debug message.
     */
    protected function wrapMessage($message) {
        if (!isset($message) || WizardValidator::isStringNotEmpty($message) === FALSE) {
            return '';
        }    
        // wrap the message in HTML
        if ($this->_debug_format === self::DISPLAY_HTML) {
            $output = '<div class="debug-message">' . "\n";
            $output .= '    <h3>Debug Message</h3>' . "\n";
            $output .= "    <p>An error has occurred: {$message}</p>\n";
            $output .= '</div><!-- div.debug-message -->' . "\n";
        } else {
            $output = $message. "\n";
        }
        return $output;
    }

    //! Set whether or not to show debug messages
    /*!
     * @param bool $data True or false.
     *
     * @return bool whatever the value was set or not
     *
     * @note: Yes this is a static variable, we keep the function as it is for legacy
     *        but the value should belong to the class it self and not instance
     */
    public function setDEBUG($data) {
        if (isset($data) && is_bool($data)) {
            static::$_debug_flag = $data;
            return TRUE;
        } 
        return FALSE;
    }

    //! Returns status of the debug flag
    /*!
     * @return bool
     * 
     * @note: Yes this is a static variable, we keep the function as it is for legacy
     *        but the value should belong to the class it self and not instance
     */
    public function getDEBUG() {
        return static::$_debug_flag;
    }
    
    //! Utility function to dump out any/all debug messages
    /*!
     * @note: Utility function to dump out any/all debug messages that
     *        may have been buffered.
     */
    public function dumpDebug() {
        foreach($this->_debug_messages as $message) {
            print $this->wrapMessage($message);
        }
    }

    //! Set whether or not to actually display debug messages immediately.
    /*!
     * @param bool $data True or false.
     *
     * @return bool whatever the value was set or not
     */
    public function setShowDebug($data) {
        if (isset($data) && is_bool($data)) {
            $this->_debug_show = $data;
            return TRUE;
        }
        return FALSE;
    }

    //! Find out if we are displaying debug messages immediately.
    /*!
     * @return bool
     */
    public function getShowDebug() {
        return $this->_debug_show;
    }

    //! Set the display to a given type.
    /*!
     * @param integer $data type to set to.
     *
     * @return bool whatever the value was set or not
     */
    public function setShowDebugFormat($data) {
        if (!isset($data) || !is_int($data) || ($data !== self::DISPLAY_HTML && $data !== self::DISPLAY_TEXT)) {
            return FALSE;
        }
        $this->_debug_format = $data;
        return TRUE;
    }

    //! Get the value of display format
    /*!
     * @return int what ever format is set 0 or 1 .
     */
    public function getShowDebugFormat() {
        return $this->_debug_format;
    }

    //! Helper function get all the debug messages 
    /*!
     * @return array, could be an empty array
     */
    public function getDebugMessages() {
        return $this->_debug_messages;
    }

    //! Set a log file to write the debug messages 
    /*!
     * @param string $file the file where you want to write the log data 
     *
     * @return bool whatever the value was set or not
     *
     * Note: File must exists 
     */
    public static function setLogFile($file) {
        if (!isset($file) || WizardValidator::isStringNotEmpty($file) === FALSE || !is_writable(dirname($file))) {
            return FALSE;
        }
        static::$_log_file = $file;
    }

    //! get the Logfile path
    /*!
     * @return mix string if the file was set, NULL otherwise 
     */
    public static function getLogFile() {
        return static::$_log_file;
    }

    //! Set the debug level to restrict what messages are displayed and/or buffered 
    /*!
     * @param integer $level debug level you want to set
     *
     * @return bool whatever the value was set or not
     */
    public static function setDebugLevel($level) {
        if (!isset($level) || WizardValidator::isIntPositiveOrZero($level) === FALSE || $level > self::ALERT) {
            return FALSE;
        }
        static::$_debug_level = $level;
    }

    //! Get the value of debug level 
    /*!
     * @return int the debug level, by default is 0 DEBUG
     */
    public static function getDebugLevel() {
        static::$_debug_level;
    }
}
