<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Rares Harnagea <rares@linuxmagic.com>
 *
 * $Id: lib.wizard_ext.php 155 2017-03-07 22:36:08Z rares $
 */

// TODO consider removing all these require from libraries 
require_once(__DIR__ . '/lib.wizard.php');
require_once(__DIR__ . '/lib.wizard_debug.php');
require_once(__DIR__ . '/lib.wizard_validator.php');
require_once(__DIR__ . '/interface.wiz_db_abstract.php');

//!
/*! WizardObjectExt
 *
 * This is an extended base class which all table classes should extend.
 *
 * @author Rares Harnagea <rares@linuxmagic.com>
 */
abstract class WizardObjectExt extends WizardObject {

    //! @string the table name of the object
    protected $_table;

    //! @array of associative arrays. metadata about the columns.
    // Sample how that array would look like
    //    'activate' => array(
    //        'type' => 'bool',
    //        'max_length' => NULL,
    //    ),
    //    'category' => array(
    //        'type' => 'string',
    //        'max_length' => 256,
    //    ),
    protected static $columns = array();

    //! @char the prefix for member variable some of the members variable start with _ other 
    // classes implement starting with alpha directly 
    protected $_prefixMembers = '_';
    
    //! @bool flag if you want to bind variables by type
    // set this to true when you have bool columns
    protected $_bindType = FALSE;

    //! array with allowed setter and the callbacks required for validation
    protected $_setters = array();
        // method name     // callbacks that you want to use for validation, they must return bool
    //    'setLimit'      => array('WizardValidator::isInt'),
    //    'setPeekLength' => array('WizardValidator::isInt'),

    //! array with allowed getter methods
    // For this to work you need to follow the standards naming convention getImapFlags() => _imap_flags
    protected $_getters;
        // getter name
    //  'getImap',
    //  'getLimit',

    //! @string the primary key of the table
    protected $_primaryKey;
    
    protected $_driverName;

    //! Constructor
    /*! 
     * @param object $dbinstance  A value containing a valid PEAR::DB object.
     *
     * NOTE: This constructor should be called with the PEAR::DB or PDO object.
     */
    public function __construct($dbinstance='') {
        parent::__construct($dbinstance);

        $this->_prefixMembers = '_'; 
        $this->_primaryKey = 'id';
        if (static::$db_type === 'PDO' && $this->db->getDb()->getAttribute(PDO::ATTR_DRIVER_NAME) === 'mysql') {
            $this->_driverName = 'mysql';
        } else {
            $this->_driverName = 'postgres';
        }
    }

    //! Helper method that is doing initialization of the member vars 
    /*!
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    abstract protected function _initMemberVars();

    //! Helper method that is doing the full initialization of an instnace
    /*!
     * @return void
     *
     * @author Steven Hall <stevenh@linuxmagic.com>
     */
    abstract protected function _init();

    //! Helper method to make a query and check if it was valid or not 
    /*!
     * @param string $sql the SQL statement that you want to run
     * @param array $params array with argument if you want to bind arguments
     * @param bool $bind_type if we want to bind by type, this is required for bool PDO
     *
     * @return mixed the result set or NULL
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function &query($sql, $params = array(), $bindType = FALSE) {
        $null = NULL;
        if (!isset($sql)) {
            $this->debug('Invalid arguments');
            return $null;
        }
        // XXX DO NOT need to do sane check for $params and $bindType 
        // they exists for sure and the type will be checked by db->query 
        $query = $this->db->query($sql, $params, $bindType);
        if ($query === NULL) {
            return $null;
        }
        // when there is an error
        if (static::isDBError($query)) {
            return $null;
        } 
        
        return $query;
    }

    //! Query by any arbitrary column
    /*!
     * @param   string $queryBy The column to query by
     * @param   mixed $val Acceptable as long as it is of the type that the
     *                     column is. This is the value to query by
     *
     * @returns FALSE on fail, primary_key value on success
     * Side Effect: Updates the object's properties with what was found in the
     * row.
     *
     * @author  Steven Hall <stevenh@linuxmagic.com>
     */
    public function queryBy($queryBy=NULL, $val=NULL) {
        if (!WizardValidator::isStringNotEmpty($queryBy)) {
            $this->debug("queryBy is not a string.");
            return FALSE;
        }
        if (!isset(static::$columns)) {
            $this->debug("columns variable does not exist.");
            return FALSE;
        }
        if (!array_key_exists($queryBy, static::$columns)) {
            $this->debug("queryBy is not in the columns list.");
            return FALSE;
        }

        // Get our column from the array
        $column = static::$columns[$queryBy];
        if (!array_key_exists('type', $column)) {
            $this->debug("'type' is not a key of the column.");
            return FALSE;
        }
        $type = $column['type'];

        // Now make sure that the value we got as a paramenter is of the same
        // type as the $columns table says
        $type2validator = array(
            'string' => 'isString',
            'int' => 'isInt',
        );

        if (!array_key_exists($type, $type2validator)) {
            $this->debug("'$type' is not in type2validator");
            return FALSE;
        }

        if (!WizardValidator::$type2validator[$type]($val)) {
            $this->debug("val [$val] is not the expected type [$type]");
            return FALSE;
        }

        // You must have the _table set in your child class
        if (!isset($this->_table) || !WizardValidator::isStringNotEmpty($this->_table)) {
            $this->debug("You must set _table in your contructor to the table name on your class [" . get_class($this) . ']');
            return FALSE;
        }

        // At this point, all data is validated. Now, query the table
        $sql = ' SELECT * FROM ' . $this->_table . ' WHERE ' . $queryBy .'  = ?';
        $params = array($val);

        if (($res = $this->query($sql, $params)) === NULL) {
            // the query method will throw another debug message about this
            $this->debug("Unable to run the query [$sql] with id $id");
            return FALSE;
        }

        $rows = array();
        while ($row = $res->fetchRow(WizardObject::FETCH_ASSOC)) {
            $rows[] = $row;
        }
        if (count($rows) > 1) {
            $this->debug("multiple rows by id found: "
                . count($rows) . " row(s) found.");
            return FALSE;
        }
        if (count($rows) === 0) {
            $this->debug(".queryBy: single row by id not found: "
                . count($rows) . " row(s) found.");
            return FALSE;
        }
        if (!$this->setFromDb($rows[0])) {
            $this->debug("failed to populate");
            return FALSE;
        }
        return $rows[0][$this->_primaryKey];
    }

    //! populate the object by id.
    /*!
     * @param int $id The column id.
     *
     * @return mixed int The id, or bool FALSE if not found.
     *
     */
    public function queryById($id) {
        if (!isset($id) || !is_int($id)) {
            $this->debug("invalid parameter");
            return FALSE;
        }
        return $this->queryBy('id', $id);
    }

    //! Helper method to determine if $this object has a method, by looking at some dynamic methods 
    /*!
     * @param string $method the method name 
     *
     * @return bool success
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * NOTE: This is to overcome the weakness of method_exists() and __call() implementation 
     */
    public function hasMethod($method) {
        if (!isset($method) || !WizardValidator::isStringNotEmpty($method)) {
            $this->debug("Missing or invalid method argument!");
            return FALSE;
        } 
        // first check if the method is defined
        if (method_exists($this, $method)) {
            return TRUE;
        }
        // Why do we have these checks?
        // This is because we use the magic method __call() and will be impossible to
        // determine what method are implemented
        // so let's look if we setup a _setters or _getters array
        if (WizardValidator::isArrayNotEmpty($this->_setters) && isset($this->_setters[$method])) {
            return TRUE;
        }
        if (WizardValidator::isArrayNotEmpty($this->_getters) && isset($this->_gettters[$method])) {
            return TRUE;
        }
        return FALSE;
    }

    //! Helper method to convert member to method  
    /*!
     * @param string $member the member name that we want to convert  
     * @param string $prefix the prefix that we want to add
     *
     * @return mixed string with the method name or bool on failure 
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * Note: this will convert the member 'is_active', 'set' to setIsActive();
     *       PHP does ignore the case on methods for now but let's do a good job now
     */
    protected function memberToMethod($member, $prefix) {
        if (!isset($member) || !WizardValidator::isStringNotEmpty($member)) {
            $this->debug("Missing or invalid parameter member");
            return FALSE;
        }
        if (!isset($prefix) || !WizardValidator::isStringNotEmpty($prefix)) {
            $this->debug("Missing or invalid parameter prefix");
            return FALSE;
        }
        // if the member starts with a _ let's remove it
        if ($member[0] === '_') {
            $member = substr($member, 1);
        }

        $newMethod = $prefix;
        $upper = FALSE;
        // loop over the chars from this string and remove any '_'
        // and then upper case the next char
        // ex: is_active => IsActive
        for ($i = 0; $i < strlen($member); $i++) {
            // upper case the first char or if the upper flag is TRUE
            if ($i === 0 || $upper) {
                $newMethod .= strtoupper($member[$i]);
                $upper = FALSE;
            } elseif ($member[$i] === '_') {
                $upper = TRUE;
            } else {
                $newMethod .= $member[$i];
            }
        }
        return $newMethod;
    }

    //! helper method that transform a part of the method name into the member variable
    /*!
     * @param string $method the part of the $method for example ImapFlags from setImapFlags()
     *
     * @return string with the member variable in the above example case will return _imap_flags
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    protected function methodToMember($method, $prefix) {
        // sane the param
        if (!isset($method) || !WizardValidator::isStringNotEmpty($method)) {
            return FALSE;
        }
        if (!isset($prefix) || !WizardValidator::isStringNotEmpty($prefix)) {
            $this->debug("Missing or invalid parameter prefix");
            return FALSE;
        }
        $method = substr($method, strlen($prefix));
        if (empty($method)) {
            $this->debug("Empty method name after trimming the prefix");
            return FALSE;
        }
        // we want to convert Username to _username or ImapFlags to _imap_flags
        $var = '';
        for ($i = 0; $i < strlen($method); $i++) {
            $ord = ord($method[$i]);
            // is the char upper case letter
            if ($ord >= 65 && $ord <= 90) {
                // for first char we may or may not have a prefix
                // depending of the convention on that class
                if ($i === 0) {
                    $var .= $this->_prefixMembers . chr($ord + 32);
                } else {
                    // add the underline and convert the letter to lowercase
                    $var .= '_' . chr($ord + 32);
                }
            } else {
                $var .= $method[$i];
            }
        }
        return $var;
    }
    //! populate the object from a database associative array row.
    /*!
     * @param array $row
     *
     * @return bool success
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     */
    public function setFromDb($row) {
        if (!isset($row) || !is_array($row)) {
            $this->debug("invalid parameter");
            return FALSE;
        }
        // Note this need to be static::$columns to work
        // since the static::$columns is going to be declared on the
        // child class
        if (!WizardValidator::isArrayNotEmpty(static::$columns)) {
            $this->debug("You must declare/generate the static::\$columns array on your class [" . get_class($this) . ']');
            return FALSE;
        }
        // We want to make sure that we don't have any values from previous object
        $this->_initMemberVars();
        
        // Now let's see what values can we set
        foreach ($row as $key => $value) {
            if (!isset(static::$columns[$key])) {
                // $this->debug("Try to set the key [$key], but the key is not present in static::\$columns array on your class [" . get_class($this) . '], skipping ...');
                continue;
            }
            // I consider that the best approach is to use the setter method and 
            // not direct attribution
            $setter = $this->memberToMethod($key, 'set');
            
            // For boolean type we need to make sure that we accomodate bool value as 't', 'f' and TRUE and FALSE
            if (static::$columns[$key]['type'] === 'bool' && !is_bool($value)) {
                if ($value === 't') {
                    $value = TRUE;
                } elseif ($value === 'f') {
                    $value = FALSE;
                } else {
                    $this->debug("Unable to convert value to bool [$value] on class [" . get_class($this) . ']');
                }
            }

            if ($this->{$setter}($value) !== TRUE) {
                // Enable this when need it 
                // $this->debug("Unable to set the value for member [$key], value passed  " . print_r($value, TRUE));
            }
        }
        return TRUE;
    }
    
    //! Brief: Helper method to return the current object as JSON
    /*!
     * @return mixed JSON format of the object, NULL on unsuccess
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function toJson() {
        $ret = json_encode($this->toArray(), JSON_PRETTY_PRINT);
        if (empty($ret)) {
            $this->debug('There was problem encoding to JSON ' . json_last_error_msg());
            return NULL;
        }
        return $ret;
    }

    //! Brief: Helper method to return the current object as array / Hash 
    /*!
     * @returns array an array of current object
     * 
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    public function toArray() {
        $hash = array();
        foreach (static::$columns as $key => $prop) {
            $getter = $this->memberToMethod($key, 'get');        
            $hash[$key] = $this->$getter();
        }
        return $hash;
    }

    //! retrieve the next value in a named sequence.
    /*!
     * @param string $sequence
     *
     * @return mixed int the next value, or bool FALSE if failure.
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * NOTE: we use this instead of PEAR::DB::nextId() due to the limitations
     *        on that method. see the documentation about that. specifically
     *        I am referring to the fact that this method changes the name of
     *        the sequence and documents to not use sequences created manually.
     */
    public function getNextSequenceValue($sequence) {
        if (!isset($sequence) || !WizardValidator::isStringNotEmpty($sequence) === 0) {
            $this->debug("invalid parameter");
            return FALSE;
        }

        $sql = 'SELECT NEXTVAL(?) AS value';
        $params = array($sequence);
        $nextVal = $this->db->getOne($sql, $params);
        if (!isset($nextVal) ||  !WizardValidator::isIntPositiveOrZero($nextVal)) {
            return FALSE;            
        }
        return intval($nextVal);
    }

    //! update a row
    /*!
     * @return mixed Integer the row's id, or bool FALSE if failure.
     *
     * NOTE: you must ensure on your child class that you have the 
     *       we have the necessary data.
     *
     *       perform an insert if there is an object id, otherwise perform
     *       an update.
     */
    protected function _update() {
        if ($this->getId() === NULL) {
            $this->debug('There is no value for Id');
            return FALSE;
        }
        // Note this need to be static::$columns to work
        // since the static::$columns is going to be declared on the
        // child class
        if (!WizardValidator::isArrayNotEmpty(static::$columns)) {
            $this->debug("You must declare/generate the static::\$columns array on your class [" . get_class($this) . ']');
            return FALSE;
        }
        // You must have the _table set in your child class
        if (!isset($this->_table) || !WizardValidator::isStringNotEmpty($this->_table)) {
            $this->debug("You must set _table in your contructor to the table name on your class [" . get_class($this) . ']');
            return FALSE;
        }

        $sql = " UPDATE $this->_table SET ";

        $params = array();

        // some columns we do not want to include
        $excludeColumns = array(
            'create_datetime',
            'create_user',
            'update_datetime',
            'update_user',
            $this->_primaryKey,
        ); 
        foreach (static::$columns as $column => $meta) {
            // we do not want to add these column to out INSERT SQL
            if (in_array($column, $excludeColumns)) {
                continue;
            }
            // add the column to SQL
            $sql .= " $column = ?,";

            $member = $this->_prefixMembers . $column;

            // now add the value
            $params[] = $this->{$member};
        } 
        // chomp the last ',' because SQL will complain about it
        // and replace with ' '
        $lastChar = strlen($sql) -1;
        $sql[$lastChar] = ' ';
        
        $sql .= ' WHERE ' . $this->_primaryKey . ' = ?;';
        
        $params[] = $this->getId();
        $res = $this->db->query($sql, $params, $this->_bindType);
        if (WizardObject::isDBError($res)) {
            $this->debug("failed to store: " . $res->getMessage());
            return FALSE;
        }

        $numRows = $this->db->affectedRows();

        if ($numRows !== 1) {
            $this->debug("failed to update row: unexpected number"
                . " of affected rows: $numRows");
            return FALSE;
        }

        return $this->getId();
    }

    //! store an answer row. update or insert.
    /*!
     * @return mixed Integer the row's id, or bool FALSE if failure.
     *
     * NOTE: you must ensure on your child class that you have the 
     *       we have the necessary data.
     *
     *       perform an insert if there is an object id, otherwise perform
     *       an update.
     */
    protected function store() {
        // Note this need to be static::$columns to work
        // since the static::$columns is going to be declared on the
        // child class
        if (!WizardValidator::isArrayNotEmpty(static::$columns)) {
            $this->debug("You must declare/generate the static::\$columns array on your class [" . get_class($this) . ']');
            return FALSE;
        }
        // You must have the _table set in your child class
        if (!isset($this->_table) || !WizardValidator::isStringNotEmpty($this->_table)) {
            $this->debug("You must set _table in your contructor to the table name on your class [" . get_class($this) . ']');
            return FALSE;
        }

        // if we have an id, this is an update.
        if ($this->getId() !==  NULL) {
            return $this->_update();
        }
        // ensure we have what we need to insert.
        $sequence = $this->_table . '_id_seq';
        // get the id to use.
        $id = $this->getNextSequenceValue($sequence);
        if ($id === FALSE) {
            $this->debug("failed to find next id");
            return FALSE;
        }
        $sql = " INSERT INTO  $this->_table ( ";

        $params = array();

        // some columns we do not want to include
        $excludeColumns = array(
            'create_datetime',
            'create_user',
            'update_datetime',
            'update_user',
        ); 
        $placeholderSQL = ' VALUES( ';
        foreach (static::$columns as $column => $meta) {
            // we do not want to add these column to out INSERT SQL
            if (in_array($column, $excludeColumns)) {
                continue;
            }
            // add the column to SQL
            $sql .= " $column,";
            // create the SQL for the placeholders
            $placeholderSQL .= ' ?,';
                 
            // format the memeber variable name
            $member = $this->_prefixMembers . $column;

            // Now all the params should come from the object member variables
            // except for the id, this is the next sequence value 
            if ($column === $this->_primaryKey) {
                // now add the value to the params array
                $params[] = $id;
            } else { 
                // now add the value to the params array
                $params[] = $this->{$member};
            }
        } 
        // chomp the last , because SQL will complain about it
        // and replace with )
        $lastChar = strlen($sql) - 1;
        $sql[$lastChar] = ')';
        
        $lastChar = strlen($placeholderSQL) - 1;
        $placeholderSQL[$lastChar] = ')';

        // add the placeholder SQL 
        $sql .= $placeholderSQL;

        $res = $this->db->query($sql, $params, $this->_bindType);
        if (WizardObject::isDBError($res)) {
            $this->debug("SQL [$sql] with params " . print_r($params, 1) . " failed to store: " . $res->getMessage());
            return FALSE;
        }

        $numRows = $this->db->affectedRows();

        if ($numRows !== 1) {
            $this->debug(".store: failed to store row: unexpected number"
                . " of affected rows: $numRows");

            return FALSE;
        }

        $this->_primaryKey = $id;
        return $this->_primaryKey;
    }

    //! retrieve a count of rows taking into account the WHERE clause.
    /*!
     * @param string $where Where parameter. Optional.
     * @param array $whereParams Parameters to the here query
     *
     * @return mixed integer, the count, or bool FALSE if failure
     *
     */
    public function countRows($where = '', $whereParams = array()) {
        // You must have the _table set in your child class
        if (!isset($this->_table) || !WizardValidator::isStringNotEmpty($this->_table)) {
            $this->debug("You must set _table in your contructor to the table name on your class [" . get_class($this) . ']');
            return FALSE;
        }
        $sql = "SELECT COUNT(1) AS count FROM {$this->_table} ";
        $params = array();
        // do we have a where clause and where params
        if (WizardValidator::isStringNotEmpty($where) && WizardValidator::isArrayNotEmpty($whereParams)) {
            $sql .= "WHERE $where";
            $params = $whereParams;
        } 

        $res = $this->db->getOne($sql, $params);
        if (!is_numeric($res)) {
            $this->debug("unable to count the rows!");
            return FALSE;
        }
        return intval($res);
    }

    //! retrieve an array of our own objects from the table.
    /*!
     * @param mixed $limit Int maximum number to return, or NULL to
     *                     have no limit.
     * @param int $offset Offset into the results. Only applies if we have
     *                    a limit.
     * @param string $where Where parameter. Optional.
     * @param string $orderBy Order by parameter. Required.
     * @param array $whereParams Parameters specific to the WHERE clause you
     *                           provide. Optional.
     *
     * @return mixed Array of objects of the same type as the current object,
     *               or bool FALSE if failure.
     *
     * NOTE: autogenerated from wiz_generate_class version 2016-08-18
     */
    public function getRows($limit, $offset, $where, $orderBy, $whereParams) {
        // You must have the _table set in your child class
        if (!isset($this->_table) || !WizardValidator::isStringNotEmpty($this->_table)) {
            $this->debug("You must set _table in your contructor to the table name on your class [" . get_class($this) . ']');
            return FALSE;
        }

        if (!is_int($limit) && $limit !== NULL) {
            $this->debug("Invalid limit");
            return FALSE;
        }
        if (!isset($offset) || !is_int($offset)) {
            $this->debug("Invalid offset");
            return FALSE;
        }
        if (!isset($orderBy) || !WizardValidator::isStringNotEmpty($orderBy)) {
            $this->debug("Invalid orderby parameter");
            return FALSE;
        }

        $params = array();
        $sql = "SELECT * FROM " . $this->_table . " ";
        // We could have a where clause without any where arguments 
        if (WizardValidator::isStringNotEmpty($where) && WizardValidator::isArray($whereParams)) {
            $sql .= " WHERE $where ";
            $params = $whereParams;
        } 
        // TODO 
        $sql .= '
ORDER BY ' . $orderBy . '
';

        if ($limit !== NULL) {
            if ($this->_driverName === 'mysql') {
                $sql .= 'LIMIT ? OFFSET ?';
                $params[] = $limit;
                $params[] = $offset;
            } else {
                $sql .= 'OFFSET ? LIMIT ?';
                $params[] = $offset;
                $params[] = $limit;
            }
        }
        $res = $this->db->query($sql, $params, $this->_driverName === 'mysql' ? TRUE : FALSE);
        if (WizardObject::isDBError($res)) {
            $this->debug(".getRows: failed to query: " . $res->getMessage());
            return FALSE;
        }
        $objects = array();
        while ($row = $res->fetchRow(WizardObject::FETCH_ASSOC)) {
            // This is not making a static object
            // but it create an instance of the object you are currently in
            // More info http://php.net/manual/en/language.oop5.late-static-bindings.php
            $object = new static($this->db);
            $object->setDebug(static::$_debug_flag);
            $object->setShowDebug($this->getShowDebug());
            $object->setShowDebugFormat($this->getShowDebugFormat());
            if (!$object->setFromDb($row)) {
                $this->debug(".getRows: failed to populate object");
                $res->free();
                return FALSE;
            }
            $objects[] = $object;
        }
        return $objects;
    }

    //! PHP magic method that we use to have controlled setters and getters for the member variables
    /*!
     * @param string $method the method that you called e.g. setUsername()
     *
     * @param array $args the arguments that you passed to the method
     *
     * @return mixed bool for setters if the variable is sane and was set, mixed for getters
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     *
     * @Throw Exception
     */
    public function __call($method, $args) {
        // if is starting with set and the method is a key in the _setters array
        // then is a legit setter call, now we need to validate the args
        if (substr($method, 0, 3) === 'set' && isset($this->_setters[$method])) {
            // check if they passed the argument for the setter
            if (!isset($args) || !WizardValidator::isArrayNotEmpty($args) || sizeof($args) !== 1) {
                $this->debug("The method [$method] is a setter and is expecting the argument to be set");
                return FALSE;
            }
            $param = $args[0];
            foreach ($this->_setters[$method] as $callback) {
                if (is_null($param)) {
                    return FALSE;
                }
                if ($this->callTarget($callback, $param) !== TRUE) {
                    // $this->debug("The param [$param] is not valid for the method [$method], failed to validate against [$callback]!");
                    return FALSE;
                }
            }
            // If we are here the variable is sane
            // now we need to convert setUsername to _username to set the member variable
            $var = $this->methodToMember($method, 'set');
            // the colum index will start at 1, because we want to get rid of the _ so _id becomes id
            $columnIndex = substr($var, 1);
            // final stage, if the params is in the static $columns and is string we need 
            // to trim the variable to the max_length 
            if (isset(static::$columns[$columnIndex]) && static::$columns[$columnIndex]['type'] === 'string' && static::$columns[$columnIndex]['max_length'] !== NULL) {
                $maxLength = static::$columns[$columnIndex]['max_length'];
                $param = trim($param);
                if (strlen($param) > $maxLength) {
                    $this->debug("The data for column [$columnIndex] is longer than the allocated column length [$param] max length $maxLength");
                    return FALSE;
                }
            }       
            // For int type let's convert the value to an integer
            if (isset(static::$columns[$columnIndex]) && static::$columns[$columnIndex]['type'] === 'int') {
                $param = intval($param);
            }
            // because the value passed the validation,the value was set and we return TRUE
            $this->$var = $param;
            return TRUE;
        } elseif (substr($method, 0, '3') === 'get' && in_array($method, $this->_getters)) {
            $var = $this->methodToMember($method, 'get');
            return $this->$var;
        } else {
            // consider throwing an exception here
            $this->debug("Invalid method called [$method] on class " . get_class($this));
            return FALSE;
        }
    }

    //! helper method that can help us to validate a variable against a callable
    /*!
     * @param mixed $callable the callable method could be Closure, class static method, class method or method from this class
     *
     * @param mixed $dataToValidate the data that need to be validated
     *
     * @return mixed bool what ever the data is valid or not
     *
     * @author Rares Harnagea <rares@linuxmagic.com>
     */
    protected function callTarget(&$callable, &$dataToValidate) {
        if (!isset($callable) || !isset($dataToValidate)) {
            return;
        }

        // This is an Anonymous function, let's call it
        if ($callable instanceof Closure) {
            return $callable($dataToValidate);
        // We need to call a static method
        } elseif (is_string($callable) && strpos($callable, '::') !== FALSE) {
            list($class, $method) = explode('::', $callable, 2);
            return $class::$method($dataToValidate);

        // We need to call a method that requires an instance
        } elseif (is_string($callable) && strpos($callable, '@') !== FALSE) {
            list($class, $method) = explode('@', $callable, 2);
            $obj = new $class;
            return $obj->$method($dataToValidate);

        // We need to call a method from this class
        } elseif (is_string($callable) && strpos($callable, '$this->') !== FALSE) {
            list($class, $method) = explode('->', $callable, 2);
            // make sure that this method is callable
            if ($this->hasMethod($method)) {
                return $this->$method($dataToValidate);
            } 
        // We need to call a regular function
        } elseif (is_string($callable) && function_exists($callable)) {
            return $callable($dataToValidate);
        // Array of allowed values
        } elseif (is_array($callable)) {
            return in_array($dataToValidate, $callable);
        // This function is not callable 
        } else {
            $this->debug("The function [$callable] is not callable");
            return FALSE;
            //throw new Exception('Invalid callable, it not callable');
        }
    }

    //! Remove the given row from this table per the ID
    /*!
     * @param   int $id optional row ID (or uses object's ID)
     *
     * @returns TRUE on Success, FALSE on fail
     *
     * @author  Steven Hall <stevenh@linuxmagic.com>
     */
    public function remove($id=NULL) {
        if (!WizardValidator::isIntPositiveOrZero($id)) {
            if (!WizardValidator::isIntPositiveOrZero($this->_id)) {
                return FALSE;
            }
            $id = $this->_id;
        }

        $sql = "DELETE FROM " . $this->_table ." WHERE id=?";
        $params = array($id);
        if (($res = $this->query($sql, $params)) === NULL) {
            // the query method will throw another debug message about this
            $this->debug("Unable to run the query [$sql]");
            return FALSE;
        }
        return TRUE;
    }

    //! Get the max length of a column
    /*!
     * @param   $data           The name of the column type to access
     *
     * @return  BOOL on fail, or The max_length property of the specified column
     *
     * @author  Alex You <alexy@linuxmagic.com>
     */
    public static function getMaxLength($data=NULL) {
        if (!WizardValidator::isStringNotEmpty($data)) {
            return FALSE;
        }
        if (!array_key_exists($data, static::$columns)) {
            return FALSE;
        }
        return static::$columns[$data]['max_length'];
    }

    //! Get the table columns
    /*!
     * @param
     *
     * @return  Array hash table columns
     *
     * @author  Steven Hall <stevenh@linuxmagic.com>
     */
    public static function getColumns() {
        return static::$columns;
    }
}
