<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2010-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Anonymous
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: class.LMNotifications.php 2050 2018-04-11 16:20:09Z shaun $
 */

require_once('class.LMNotificationsLocalRepository.php');
require_once('class.LMNotificationsRemoteRepository.php');

class LMNotifications {
    // TODO: document these
    protected $product;
    protected $uri;
    protected $data_dir;
    protected $local_cache;
    protected $remote;

    //! LMNotificationsLocalRepository that looks at the 'read' items.
    protected $_local_cache_read;

    //! string. suffix for 'read' local repository file. normally this
    // is '.cache' for unread local repositories.
    protected $_read_file_suffix = '.read';

    //! string. suffix for 'unread' local repository file. normally this
    // is '.cache' for unread local repositories.
    protected $_unread_file_suffix = '.cache';

    //! integer. the number of days back to get entries for (maximum).
    // this is a sanity that prevents us from pulling entries from the
    // beginning of time. it's possible we'll want this to be per-feed
    // based (and to allow it to be configurable), but for the time
    // being it's not.
    private $_max_remote_entry_days_back = 30;

    //! constructor
    /*!
     * @author Anonymous
     *
     * @constructor
     *
     * @throws Exception
     */
    public function __construct($product, $uri, $data_dir) {
        if (!$product) {
            throw new Exception('No product defined.');
        } else if (!$uri) {
            throw new Exception('No uri defined.');
        } else if (!$data_dir) {
            throw new Exception('No data directory defined.');
        } else if (!is_dir($data_dir) || !is_readable($data_dir)) {
            throw new Exception('Data directory is not accessible.');
        }

        $this->product = $product;
        $this->uri = $uri;
        $this->data_dir = $data_dir;
    }

    //! retrieve unread entries.
    /*!
     * TODO: document
     */
    public function get($number, $offset = 0) {
        // download any new messages and save them to local cache
        $this->getRecent();

        // return the requested entries
        $localrepo = $this->getLocal();
        return $localrepo->get($number, $offset);
    }

    //! retrieve read entries.
    /*!
     * TODO: document
     */
    public function getRead($number, $offset = 0) {
        // download any new messages and save them to local cache
        $this->getRecent();

        // return the requested entries
        $localrepo = $this->getLocalRead();
        return $localrepo->get($number, $offset);
    }

    //! determine the most recent remote entry we have in our local
    // repositories.
    /*!
     * @return int unixtime
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * for the purposes of updates we always want to get the time of
     * the most recent remote-sourced feed entry.
     *
     * this allows us to fetch entries since that time.
     *
     * NOTE: we can't simply look at all entries we have and find the
     *   one with the highest timestamp because we have entries from
     *   both remote and locally generated sources (see post_system_alert
     *   which generates entries locally) in the same feed.
     *   so we find the highest timestamp of an entry that is from a
     *   feed's remote source.
     *   this assumes that only one remote source per feed.
     *
     * we must examine both the read and unread local repositories as
     * remote entries may be in one or the other.
     */
    private function _getMostRecentRemoteTimestamp() {
        // start with a timestamp at the 'max remote entry' days back.
        // this is to prevent us from requesting remote entries from the
        // beginning of time.
        $timestamp = time() - (60 * 60 * 24
            * $this->_max_remote_entry_days_back);

        // first find the maximum remote timestamp of the unread entries.
        $localUnreadRepo = $this->getLocal();
        $entries = $localUnreadRepo->getEntries($timestamp);
        foreach ($entries as $entry) {
            // we only care about remote sourced entries.
            if (!$entry->from_remote) {
                continue;
            }
            if ($entry->timestamp > $timestamp) {
                $timestamp = intval($entry->timestamp);
            }
        }
        // and then from the read set.
        // NOTE: we could do both at once, but that would be more memory
        //   heavy - this way we can load up just those with a timestamp
        //   higher than the max we have so far.
        $localReadRepo = $this->getLocalRead();
        $entries = $localReadRepo->getEntries($timestamp);
        foreach ($entries as $entry) {
            // we only care about remote sourced entries.
            if (!$entry->from_remote) {
                continue;
            }
            if ($entry->timestamp > $timestamp) {
                $timestamp = intval($entry->timestamp);
            }
        }
        return $timestamp;
    }

    /*
     * Returns all entries by combining the local repo
     * and the 'new' entries in the remote repo.
     * If there is nothing in the local repo, it will retrieve everything
     * from the remote.
     *
     * @returns array - all entries
     *
     * @author Logan Borean <loganb@linuxmagic.com>
     */
    public function getAllEntries() {
        $timestamp = null;
        $localrepo = $this->getLocal();
        $remoterepo = $this->getRemote();

        // get all existing tips from the local file
        $allExistingEntries = $localrepo->getEntries(1);

        // if we have nothing in the local repo, retrieve everything
        if (empty($allExistingEntries)) {
            $allEntries = $remoterepo->getEntries(1);
            $localrepo->addEntries($allEntries);
            $localrepo->save();
            return $allEntries;
        }
        
        // get all the latest entries from the remote
        $newRemoteEntries = $this->getRecent();

        if (!is_array($allExistingEntries) || !is_array($newRemoteEntries)) {
            return array();
        }

        // merge old and new entries and return (so we don't have to read from the file again)
        return array_merge($newRemoteEntries, $allExistingEntries);
    } 

    //! retrieve new entries from the remote repository.
    /*!
     * @returns Array of LMNotificationsEntry objects.
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     *
     * we retrieve the entries from the remote repository that we have
     * not yet seen.
     *
     * TODO: since entries are keyed uniquely by timestamp (two with
     *   the same timestamp cannot exist in the same repository right
     *   now) we have issues if we have multiple entries at the same
     *   time.
     *   this is normally not a big problem if we JUST had remote
     *   entries coming in (because support staff is unlikely to post
     *   more than one entry in a second), but if we have locally generated
     *   messages then we could have more than one entry per second, and
     *   we could even have conflicts where two entries have the same
     *   timestamp...
     */
    public function getRecent() {
        // find the most recent remote timestamp we have and fetch remote
        // entries since that time.
        $mostRecentRemoteTimestamp = $this->_getMostRecentRemoteTimestamp();
        $remoteRepo = $this->getRemote();
        $remoteEntries = $remoteRepo->getEntries($mostRecentRemoteTimestamp);

        // filter out those we haven't already read.
        // why? because we need to ensure that we don't take an entry that
        // we've read and add it back into the unread repository as a duplicate.
        // note we don't have to worry about filtering out those that are
        // already in the unread repository because we already do not permit
        // duplicates in the same repository.
        // NOTE: timestamp 0 to get *all* of the read entries for comparison...
        //   if we don't, then we could miss some.
        //   this assumes cleanup of the repository occurs elsewhere and that
        //   we can load all into memory...
        $localReadRepo = $this->getLocalRead();
        $readEntries = $localReadRepo->getEntries(0);
        $newEntries = array();
        foreach ($remoteEntries as $entry) {
            $isRead = FALSE;
            foreach ($readEntries as $readEntry) {
                // TODO: see the note about using timestamp as unique above.
                if (intval($readEntry->timestamp) === intval($entry->timestamp))
                {
                    $isRead = TRUE;
                    break;
                }
            }
            // if it's not in the read repository then it's a candidate
            // for the unread repository.
            if (!$isRead) {
                $newEntries[] = $entry;
            }
        }

        // now save. note we run getEntries() (and ignore the result) so that
        // we load all entries already in the unread repository into memory
        // so that they will be saved again when we save. again, timestamp 0
        // so we get all entries we have stored already.
        $localUnreadRepo = $this->getLocal();
        $localUnreadRepo->getEntries(0);
        // NOTE: addEntries() won't add an entry twice (because internally
        //   they are stored as a hash on timestamp).
        $localUnreadRepo->addEntries($newEntries);
        if (!$localUnreadRepo->save()) {
            // TODO: we should distinguish between error
            return $newEntries;
        }
        return $newEntries;
    }

    //! get the last 'unread' entry.
    /*!
     * @returns numeric Timestamp of the last local entry.
     *
     * TODO: review behaviour here if there are no entries!
     */
    public function getLastLocalEntry() {
        $local = $this->getLocalCache();
        $lastentry = $local->getMostRecent();
        return $lastentry->timestamp;
    }

    //! get the 'read' local repository.
    /*!
     * @returns LMNotificationsLocalRepository
     *
     * @author Anonymous.
     * @author William Storey <william@linuxmagic.com>
     *
     * we have two local repositories for each feed: one for unread (.cache),
     * and one for read (.read).
     *
     * this gets the 'unread' set.
     */
    public function getLocal() {
        if ($this->local_cache === NULL) {
            $this->local_cache = new LMNotificationsLocalRepository(
                $this->data_dir, $this->product, '.cache');
        }

        return $this->local_cache;
    }

    //! get the 'read' local repository.
    /*!
     * @returns LMNotificationsLocalRepository
     *
     * @author William Storey <william@linuxmagic.com>
     *
     * we have two local repositories for each feed: one for unread (.cache),
     * and one for read (.read).
     *
     * this gets the 'read' set.
     */
    public function getLocalRead() {
        if ($this->_local_cache_read === NULL) {
            $this->_local_cache_read = new LMNotificationsLocalRepository(
                $this->data_dir, $this->product, $this->_read_file_suffix);
        }
        return $this->_local_cache_read;
    }

    //! get the 'remote' local repository object.
    /*!
     * @returns LMNotificationsRemoteRepository
     *
     * @author Anonymous
     */
    public function getRemote() {
        if ($this->remote === NULL) {
            $this->remote = new LMNotificationsRemoteRepository($this->uri,
                $this->product);
        }
        return $this->remote;
    }
}
