<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: logSearchModule.php 8862 2018-11-27 19:11:35Z deiony $
 */
require_once('lib.module.php');
require_once('lib.country.php');
require_once('lib.authlogsearch.php');
require_once('wizard/lib.wizard_validator.php');

class authLogSearchModule extends Module
{
    //! string. request action.
    protected $action;

    //! string. auth user address search filter string.
    protected $auth_user;

    //! string. IP address search filter string.
    protected $ip_address;

    //! integer. the filter type
    protected $filter;

    //! integer. the number of log results to limit to.
    protected $result_limit;

    //! string. the American date search filter string (yyyy-mm-dd - yyyy-mm-dd)
    protected $date;

    //! string. the American date search filter string (yyyy-mm-dd)
    protected $startDate;

    //! string. the American date search filter string (yyyy-mm-dd)
    protected $endDate;

    //! string. alternative controller that might affect render links
    protected $controller;

    //! internal flag for IE rendering compatibility.
    protected $isIE;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->action = NULL;
        $this->auth_user = NULL;
        $this->ip_address = NULL;
        $this->filter = NULL;
        $this->result_limit = NULL;
        if (defined('LS_CONTROLLER')) {
            $this->controller = LS_CONTROLLER;
        }
        $this->isIE = FALSE;
    }

    public function actionProcessor($action, &$errors, &$notices) {
        extract($_POST);
        if (isset($action) && WV::isStringNotEmpty($action)) {
            $this->action = trim($action);
        }
        if (isset($auth_addr) && WV::isStringNotEmpty($auth_addr)) {
            $this->auth_user = trim($auth_addr);
        }
        if (isset($ip_address) && WV::isStringNotEmpty($ip_address)) {
            $ip_address = trim($ip_address);
            if (filter_var($ip_address, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)) {
                $ip_address = inet_ntop(inet_pton($ip_address));
            }
            $this->ip_address = $ip_address;
        }
        if (isset($filter) && WV::isIntPositiveOrZero($filter)) {
            $this->filter = $filter;
        }
        if (isset($result_limit) && WV::isIntPositive($result_limit)) {
            $this->result_limit = $result_limit;
        }
        if (isset($start_date, $end_date)
            && $this->isDate($start_date)
            && $this->isDate($end_date)) {
            $this->startDate = $start_date;
            $this->endDate = $end_date;
            $this->date = "$start_date - $end_date";
        }
        if (isset($ie) && WV::isIntPositive($ie)) {
            $this->isIE = TRUE;
        }
    }

    //! renders the display for screen, this is an overwritten method
    public function renderModule()  {
        global $module;

        // Adds the help content to the help pop-up
        $this->_i18n_js[] = 'logSearchHelp';
        $this->_i18n_js[] = 'logSearch.error.invalidSenderCriteria';
        $this->_i18n_js[] = 'logSearch.error.invalidRecipientCriteria';
        $this->_i18n_js[] = 'logSearch.error.invalidAuthUserCriteria';
        $this->_i18n_js[] = 'logSearch.error.invalidIPCriteria';
        $this->_i18n_js[] = 'logSearch.error.invalidLimitCriteria';
        // XXX: need to review the frontend error for date filter
        $this->_i18n_js[] = 'logSearch.error.invalidDateCriteria';
        $this->_i18n_js[] = 'logSearch.error.invalidDateRangeCriteria';

        renderTemplate('authLogSearch', array(
            "module"     => $module,
            "data_table" => $this->makeSearchForm(),
            "controller" => isset($this->controller) ? $this->controller : "logsearch.php",
            "subtitle"   => intl('auth_logsearch_subtitle'),
            "help"       => intl('help'),
            "desc"       => intl("logSearchDesc"),
        ));
    }

    //! method to retrieve any 'additional' JS files to include in the page rendering
    /*!
     * @returns array   an array of the files to include
     */
    public function getJS() {
        $js = array(
            'help-popup.js',
            'magicspam.js',
            'magicspam-menu.js',
            'limit-integer.js',
            'check-enabled.js',
            'ajax.js',
            'form-submit.js',
            'log-search.js',
            'nodep-date-input-polyfill.dist.js',
        );
        return $js;
    }

    public function getCSS() {
        return array();
    }

    //! method to generate a render string for the auth log search table
    protected function makeSearchForm() {
        // initialize a new data table
        $table = new magicspamDataTable;

        // add row for auth user input
        $table->addRow(array(
            intl("auth_user"),
            '<input class="input" type="text" placeholder="user@example.com" id="auth_addr" name="auth_addr" maxlength="' . ADDRESS_SEARCH_MAX . '"/>',
            '<div class="inputLabel">' . intl("authLogAuthUserLabel") . '</div>'
        ));

        // add row for ip address input
        $table->addRow(array(
            intl('ip'),
            '<input class="input" type="text" placeholder="192.168.0.210" id="ip_address" name="ip_address" maxlength="' . LSM_IP_ADDRESS_SEARCH_MAX . '">',
            '<div class="inputLabel">' . intl('logIPLabel') . '</div>'
        ));

        // add row for date range input
        $table->addRow(array(
            intl('date_range'),
            '<div>' .
                '<input type="date" id="auth_start_date" name="start_date" class="date" onkeydown="return false;">' .
                '<div class="ls_divider">' . intl('date_to') . '</div>' .
                '<input type="date" id="auth_end_date" name="end_date" class="date" onkeydown="return false;">' .
            '</div>',
            '<div class="inputLabel">' . intl('logDateLabel') . '</div>'
        ));

        // build the filter input
        $filterInput = '<div>'
                     .     '<select name="filter" id="filter">'
                     .         '<option value="' . LS_AUTH_FILTER_ALL   . '">' . intl('ls_auth_all')   . '</option>'
                     .         '<option value="' . LS_AUTH_FILTER_BLOCK . '"> '. intl('ls_auth_block') . '</option>'
                     .         '<option value="' . LS_AUTH_FILTER_PASS  . '">' . intl('ls_auth_pass')  . '</option>'
                     .     '</select>'
                     .     '<input class="limit" type="checkbox" name="limit_results" onclick="checkEnabled(this, this.form.result_limit, \'50\');" checked>'
                     .     intl("limit") . '&nbsp;'
                     .     '<input type="text" class="numeric" id="authresult_limit" name="result_limit" onkeyup="limitInteger(this);" size="2" value="50" maxlength="' .  RESULTS_SEARCH_MAX .  '">'
                     . '</div>';

        // add row for filter input
        $table->addRow(array(
            '<span class="filterAlign">' . intl('ls_filter_by') . '</span>',
            $filterInput,
            '<div class="inputLabel">' . intl('logFilterLabel') . '</div>'
        ));

        // build the html for the search and clear button
        $searchButton = '<button class="msbutton" id="log_search_btn" type="submit"><span>' . intl("search") . '</span></button>';
        $clearButton  = '<button class="msbutton" id="log_clear_btn" type="reset"><span>&nbsp;' . intl("clear") . '&nbsp;</span></button>';

        // add row for the search and clear button
        $table->addRow(array(
            "", "", $searchButton . $clearButton
        ));

        // return the HTML string for rendering the auth log search form
        return $table->getRenderString();
    }

    public function renderOutput($modules) {
        // initialize the auth logsearch handle
        $auth_logsearch_handle = new AuthLogSearch();

        // prepare the variables for the search
        $base_file = LOGDIR . "msauthlog";
        $log_max_rows = isset($this->result_limit) ? $this->result_limit : SEARCH_RESULTS_SANITY;
        $max_days = (int)getControl("logdays", 7, CONTROLDIR);
        $data = array();
        $more_than_requested = FALSE;

        // set up the filters
        $auth_logsearch_handle->setFilter($this->filter);
        $auth_logsearch_handle->setAuthUser($this->auth_user);
        $auth_logsearch_handle->setIpAddress($this->ip_address);
        $auth_logsearch_handle->setDate($this->startDate, $this->endDate);

        // perform the auth log search
        if ($auth_logsearch_handle->parseLogFiles($base_file, $log_max_rows, $max_days + 1, $data, $more_than_requested) === FALSE) {
            return;
        }

        // prepare the headers
        $headers = array(
            "timestamp",
            "status",
            "ip",
            "host",
            "country",
            "auth_user",
        );

        // render the output in the appropriate format depending on the action
        if ($this->action === "search") {
            $this->renderOutputAsHTML($headers, $data, $more_than_requested);
        } else if ($this->action === "csv") {
            $output_file = "search_auth_result.csv";
            $this->renderOutputAsCSV($output_file, $headers, $data, $more_than_requested);
        }
    }

    protected function renderOutputAsHTML($headers, $log_data, $more_than_requested) {
        // required for CSV download handoff (ajax call)
        global $module;

        // otherwise we state we didn't find anything
        if (empty($log_data)) {
            echo "<div id=\"logresults_NA\">" . intl("no_result") . "</div>\n";
            return;
        }

        // build the list of i18n labels and CSS classes for each header
        foreach ($headers as $header) {
            $headers_label[] = intl($header);
            $headers_class[] = "header_$header log_cell";
            $columns_class[] = "log_$header log_cell";

            // the adjacent resize cells
            $headers_label[] = "";
            $headers_class[] = "resize_$header log_cell";
            $columns_class[] = "resize_$header log_cell";
        }

        // set up the log search header
        $search_header = new magicspamDataTable;
        $search_header->setHeader($headers_label);
        $search_header->setClass("headerRow", $headers_class);

        // set up the log search table
        $search_table = new magicspamDataTable;
        $search_table->setClass("row", $columns_class);

        // process each row of log data
        foreach ($log_data as $rows) {
            // reset the data row
            $drow = array();

            // process each data item in row
            foreach ($rows as $index => $ditem) {
                // skip the auth rule index
                if ($index === LS_AUTH_INDEX_RULE) {
                    continue;
                }

                // reset to default values
                $class = "";
                $data = $ditem;
                $title = $ditem;

                // format the data appropriately based on the index type
                switch ($index) {
                case LS_AUTH_INDEX_DATETIME:
                    // format datestring into friendlier form
                    $data = date("l, M d Y @ g:i:s A", strtotime($ditem));
                    break;
                case LS_AUTH_INDEX_TYPE:
                    $class = "isHam";
                    if ($ditem === "BLOCK") {
                        $class = "isSpam";
                        $data = $ditem . "[" . $rows[LS_AUTH_INDEX_RULE] . "]";
                    } else if ($ditem === "EXEMPT") {
                        $class = "isExempt";
                        $data = $ditem . "[" . $rows[LS_AUTH_INDEX_RULE] . "]";
                    }
                    break;
                case LS_AUTH_INDEX_COUNTRY:
                    // set the country name as the title
                    $title = Country::getCountryNameFromCode($ditem);
                    break;
                default:
                    // no formatting required
                    break;
                }

                // encode the data and title for html
                $encodedData = htmlspecialchars($data);
                $encodedTitle = htmlspecialchars($title);

                // append the html to the data row
                $drow[] = "<a class=\"$class\" title=\"$encodedTitle\">$encodedData</a>";
                $drow[] = "";
            }

            // append the data row to the search table
            $search_table->addRow($drow);
        }

        // build the link for downloading the log search results in CSV format
        $csv_download = sprintf("<a href=\"#\" onclick=\"downloadAuthCSV('%s', 'authLogSearch', 'csv', '%s', '%s', '%s', '%s', %d, %d, '%s'); return false;\">%s</a>",
            htmlspecialchars($module),
            htmlspecialchars($this->auth_user),
            htmlspecialchars($this->ip_address),
            htmlspecialchars($this->startDate),
            htmlspecialchars($this->endDate),
            htmlspecialchars($this->filter),
            isset($this->result_limit) ? htmlspecialchars($this->result_limit) : -1,
            isset($this->controller) ? htmlspecialchars($this->controller) : "",
            intl('click_to_download')
        );

        // build the log search result div which includes the csv download link
        $search_result = sprintf("<div class=\"search_result\">%s: %s %d %s $csv_download</div>",
            intl("rows_found"),
            $more_than_requested ? intl('more_than') : "",
            count($log_data),
            $more_than_requested ? intl('too_much_rows') : ""
        );

        // build the the entire log search results page
        $html = $search_result
              . "<div id=\"logresults\" class=\"authlogresults\">"
              . $search_header->getRenderString()
              . $search_table->getRenderString()
              . "</div>"
              . $search_result;

        // render it out!
        echo $html;
    }

    protected function renderOutputAsCSV($output_file, $headers, $data, $more_than_requested) {
        // build the list of i18n labels for each header
        foreach ($headers as $header) {
            $headers_label[] = intl($header);
        }

        // turn off output compression
        ini_set('zlib.output_compression', 'Off');

        // determine the octet-stream MIME type based on IE browser
        $octetstream = ($this->isIE) ? "octetstream" : "octet-stream";

        // full list of potential HTTP headers for CSV download
        $csv_headers = array(
            "HTTP/1.1 200 OK",
            "Pragma: public",
            "Last-Modified: " . gmdate('D, d M Y H:i:s') . " GMT",
            "Cache-Control: no-store, no-cache, must-revalidate",
            "Cache-Control: pre-check=0, post-check=0, max-age=0",
            "Content-Transfer-Encoding: none",
            "Content-Type: application/$octetstream; name=\"$output_file\"",
            "Content-Type: application/force-download",
            "Content-Disposition: attachment; filename=\"$output_file\"",
        );

        // remove the Cache-Control headers when not IE browser
        if (!$this->isIE) {
            array_splice($csv_headers, 3, 2);
        }

        // remove the status code header when not building the CSV headers string
        if (!defined("LS_CSV_HEADERS_STRING")) {
            array_splice($csv_headers, 0, 1);
        }

        // send the HTTP headers or build the CSV headers string
        $csv_headers_string = "";
        foreach ($csv_headers as $csv_header) {
            if (defined("LS_CSV_HEADERS_STRING")) {
                $csv_headers_string .= "$csv_header\n";
            } else {
                header($csv_header);
            }
        }

        // open the output buffer mechanism for writing
        $handle = fopen('php://output', 'w');

        // write the CSV headers string to the output buffer if necessary
        if (defined("LS_CSV_HEADERS_STRING")) {
            fwrite($handle, "$csv_headers_string\n");
        }

        // write the log search column headers to the output buffer
        fputcsv($handle , $headers_label);

        // write the log search results to the output buffer
        foreach ($data as $line) {
            // combine the log search type and rule if necessary
            $rule = array_splice($line, LS_AUTH_INDEX_RULE, 1)[0];
            if ($line[LS_AUTH_INDEX_TYPE] === "BLOCK") {
                $line[LS_AUTH_INDEX_TYPE] .= "[$rule]";
            }
            fputcsv($handle, $line);
        }
        fclose($handle);
    }

    //! validate date string with yyyy-mm-dd format
    protected function isDate($date) {
        // sanity check
        if (!isset($date) || !WV::isStringNotEmpty($date)) {
            return FALSE;
        }
        // ensure month, day and year are separated by slash character
        $date_arr = explode('-', $date);
        if (count($date_arr) != 3) {
            return FALSE;
        }
        // ensure month, day and year are positive integers
        if (!WV::isIntPositive($date_arr[0])
            || !WV::isIntPositive($date_arr[1])
            || !WV::isIntPositive($date_arr[2])) {
            return FALSE;
        }
        // ensure month, day and year are valid
        return checkdate($date_arr[1], $date_arr[2], $date_arr[0]);
    }
}
