<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Logan Borean <loganb@linuxmagic.com>
 *            Deion Yu <deiony@linuxmagic.com>
 *
 * $Id: countryAuthModule.php 11866 2020-07-08 00:08:11Z deiony $
 */
require_once("lib.module.php");
require_once("lib.country.php");

class countryAuthModule extends Module {

    //! name of the Country Auth ACL file
    const COUNTRY_AUTH_ACL_CONTROL = 'country_auth.acl';

    //! name of the Country Auth default server country file
    const COUNTRY_AUTH_DEFAULT_CONTROL = 'default_server_country';

    //! whether to build the default country list
    private $build_default;

    //! class constructor
    public function __construct($l) {
        parent::__construct($l, FALSE);
        // set module to be migratable
        $this->migratable = TRUE;
        $this->build_default = TRUE;
    }

    //! process any actions
    public function actionProcessor($action, &$errors, &$notices) {
        global $isFrontend, $home;

        // process the country auth save action
        if ($action === "save") {
            // determine the type of the ACL list to save
            if (isset($_POST['all_block'])) {
                $list = array();
                $wildcard = 'deny';
                $word = 'allow';
            } else if (isset($_POST['allowed_countries'])) {
                $list = $_POST['allowed_countries'];
                $wildcard = 'deny';
                $word = 'allow';
            } else {
                $list = isset($_POST['blocked_countries']) ? $_POST['blocked_countries'] : array();
                $wildcard = 'allow';
                $word = 'deny';
            }

            // build the content for the Country ACL file
            $content = '';
            foreach($list as $country_code) {
                $content .= "$country_code:$word\n";
            }
            $content .= "*:$wildcard";

            // attempt to save the content to the Country Auth ACL file
            // NOTE: setControl() handles saving to global and user level
            if (setControl(self::COUNTRY_AUTH_ACL_CONTROL, $content) !== TRUE) {
                $errors[] = intl('country_auth_save_failure');
            } else {
                $notices[] = intl('country_auth_save_success');
            }
        }

        // process the country auth save default server country action
        if ($action === "save_default") {
            // sanity check on the POST variable
            if (!isset($_POST['default_server_country'])
                || !WV::isStringNotEmpty($_POST['default_server_country'])) {
                return;
            }

            // sanitize and validate the default server country
            $default_server_country = trim($_POST['default_server_country']);
            if (!Country::isCountryCode($default_server_country)) {
                $errors[] = intl('country_auth_save_default_unknown');
                return;
            }

            // attempt to save the default server country
            if (setControl(self::COUNTRY_AUTH_DEFAULT_CONTROL, $default_server_country) === TRUE) {
                $notices[] = intl('country_auth_save_default_success');
            } else {
                $errors[] = intl('country_auth_save_default_failure');
            }
        }
    }

    //! retrieve any optional CSS files for rendering
    public function getCSS() {
        return array('country-auth.css');
    }

    //! retrieve any optional JS files for rendering
    public function getJS() {
        return array('country-auth.js', 'form-submit.js', 'ajax.js');
    }

    //! render the module
    public function renderModule() {
        global $isFrontend, $home, $module;

        // initialize the error variables
        $errors = array();
        $error_msg = NULL;

        // lookup server country code using GeoIP location
        $country_code = Country::lookupGeoIP($_SERVER['SERVER_ADDR']);
        if ($country_code === FALSE) {
            $errors[] = intl('country_auth_geoip_lookup_failure');
            $country_code = "**";
        }

        // determine whether the country code was detected
        $detected_country_code = ($country_code == "**");

        // load the default server country code
        $default_country_code = getControl(self::COUNTRY_AUTH_DEFAULT_CONTROL, NULL, MASTERCONTROLDIR);
        if (!Country::isCountryCode($default_country_code)) {
            if (file_exists(MASTERCONTROLDIR . self::COUNTRY_AUTH_DEFAULT_CONTROL)) {
                $errors[] = intl('country_auth_server_country_code_default_failure');
            }
            $default_country_code = "**";
        }

        // fallback to default country code when detected country code is unknown
        if ($country_code === "**" && $default_country_code !== "**") {
            $country_code = $default_country_code;
        }

        // build the options for the default country select dropdown on the admin interface
        $default_country_options = '';
        if ($isFrontend === FALSE) {
            $default_country_options = $this->buildDefaultCountryOptions($default_country_code);
        }

        // build list of countries associated with the server
        $server_country = array($country_code, $default_country_code);

        // retrieve the admin country lists
        $admin_acls = $this->getCountryList(MASTERCONTROLDIR, $error_msg);
        if ($admin_acls === FALSE) {
            $errors[] = intl('country_auth_admin_level') . $error_msg;
        }

        // retrieve the user country lists when running on user interface
        $user_acls = array();
        if ($isFrontend === TRUE) {
            $user_acls = $this->getCountryList("$home/magicspam/control/", $error_msg);
            if ($user_acls === FALSE) {
                $errors[] = intl('country_auth_user_level') . $error_msg;
            }
        }

        // initialize the country lists
        $allowed = array();
        $blocked = array();
        $allowed_enforced = array();
        $blocked_enforced = array();

        // consolidate the admin and user country lists so long as there were no previous errors
        if (!isset($error_msg)) {
            $consolidated_acl = $this->consolidateCountryLists($admin_acls, $user_acls, $server_country);
            list($allowed, $blocked, $allowed_enforced, $blocked_enforced) = $consolidated_acl;
            $this->build_default = empty($blocked) && empty($blocked_enforced);
        }

        // determine whether to expand or collapse the country auth module
        $expand_state = FALSE;
        if ($isFrontend || (isset($_POST['ms_action'], $_POST['submodule']) && $_POST['submodule'] == "countryAuth")) {
            $expand_state = TRUE;
        }

        // prepare the template bind variables
        $bind = array(
            "module" => $module,
            "exemptions_module" => $isFrontend ? "feExemption" : "settings",
            "exemptions_page" => $isFrontend ? "" : "exemption",
            "errors" => $errors,
            "detected" => $detected_country_code,
            "is_frontend" => $isFrontend,
            "server_country_name" => Country::getCountryNameFromCode($country_code),
            "default_country_options" => $default_country_options,
            "allowedCountriesHtml" => $this->buildMultiSelect($allowed, $allowed_enforced, 'allowed_countries'),
            "blockedCountriesHtml" => $this->buildMultiSelect($blocked, $blocked_enforced, 'blocked_countries'),
            "collapsible_icon" => $expand_state ? "collapse-icon.png" : "expand-icon.png",
            "collapsible_class" => $expand_state ? "collapsibleModuleActive" : "",
        );

        // render the country auth template
        renderTemplate('countryAuth', $bind);
    }

    protected function buildDefaultCountryOptions($default_country_code) {
        // sanity check
        if (!isset($default_country_code) || !Country::isCountryCode($default_country_code)) {
            return FALSE;
        }

        // initialize the default country options
        $default_country_options = '';

        // add the unknown option if we failed to retrieve the default country
        // code from the control file for any reason
        if ($default_country_code == "**") {
            $default_country_options .= "<option selected>Unknown</option>";
        }

        // add the rest of the countries as an option
        foreach (Country::$codes as $country_code => $country_name) {
            // select the country option when the country matches the default
            $selected = ($country_code == $default_country_code) ? 'selected' : '';

            // build and append the country option
            $default_country_options .= "<option value=\"$country_code\" $selected>"
                                     .  "    $country_name"
                                     .  "</option>";
        }
        return $default_country_options;
    }

    //! retrieves and parses the content of a country list file
    /*!
     * @param   string          the path to the directory containing the list file
     *
     * @returns mixed           the parsed content of the country list file, FALSE on error
     */
    protected function getCountryList($directory, &$message) {
        // sanity check
        if (!isset($directory) || !WV::isStringNotEmpty($directory)) {
            return FALSE;
        }

        // ensure that the files in the directory can be listed
        if (!is_readable($directory)) {
            $message = intl('country_auth_cannot_read_dir');
            return FALSE;
        }

        // initialize the default list content
        $listContent = array(
            'allow'    => array(),
            'deny'     => array(),
            'wildcard' => 'allow',
        );

        // build path to country list file
        $list_file = $directory . self::COUNTRY_AUTH_ACL_CONTROL;

        // return the default list content when the list file does not exist
        if (!file_exists($list_file)) {
            return $listContent;
        }

        // it's an error when list file exists but is not readable
        if (!is_readable($list_file)) {
            $message = intl('country_auth_cannot_read_file');
            return FALSE;
        }

        // read content of list file ignoring new lines characters and empty lines
        $content = file($list_file, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);

        // parse each line of content
        foreach ($content as $line) {
            // split the line into tokens
            $tokens = explode(':', $line);
            if ($tokens === FALSE || count($tokens) !== 2) {
                $message = intl('country_auth_file_bad_line');
                return FALSE;
            }

            // assign friendly names to tokens
            list($country_code, $state) = $tokens;

            // ensure state is either deny or allow
            if ($state !== 'deny' && $state !== 'allow') {
                $message = intl('country_auth_file_bad_state');
                return FALSE;
            }

            // set the wildcard when the country code is the wildcard character
            // otherwise append the country code to the allow or deny list
            if ($country_code === '*') {
                $listContent['wildcard'] = $state;
            } else {
                $listContent[$state][] = $country_code;
            }
        }

        return $listContent;
    }

    //! consolidates the admin and user level country lists
    /*!
     * @param    array          the admin level country lists
     * @param    array          the user level country lists
     * @param    array          the list of country codes associated with the server
     *
     * @returns  mixed          the consolidated country lists, FALSE on error
     */
    protected function consolidateCountryLists($admin_acls, $user_acls, $server_country_code) {
        // sanity check
        if (!isset($admin_acls, $user_acls, $server_country_code)
            || !WV::isArrayNotEmpty($admin_acls)
            || !WV::isArray($user_acls)
            || !WV::isArrayNotEmpty($server_country_code)) {
            return FALSE;
        }

        // initialize the country status lists
        $deny = array();
        $allow = array();
        $deny_enforced = array();
        $allow_enforced = array();

        // when the wilcard for the admin level ACL is allow, move all specified
        // countries from the allowed to denied list and vice-versa
        if ($admin_acls['wildcard'] === 'allow') {
            $allow = Country::getCountryCodes();
            foreach ($admin_acls['deny'] as $code) {
                if (!array_key_exists($code, $allow)) {
                    continue;
                }
                $deny[$code] = $allow[$code];
                unset($allow[$code]);
            }
        } else {
            $deny = Country::getCountryCodes();
            foreach ($admin_acls['allow'] as $code) {
                if (!array_key_exists($code, $deny)) {
                    continue;
                }
                $allow[$code] = $deny[$code];
                unset($deny[$code]);
            }
        }

        // move the server country code to the allowed enforced list
        foreach ($server_country_code as $code) {
            if ($code !== "**" && !array_key_exists($code, $allow_enforced)) {
                if (array_key_exists($code, $deny)) {
                    $allow_enforced[$code] = $deny[$code];
                    unset($deny[$code]);
                } else {
                    $allow_enforced[$code] = $allow[$code];
                    unset($allow[$code]);
                }
            }
        }

        // consolidate the user level ACL with the admin level ACL if necessary
        if (!empty($user_acls)) {
            // admin level denied countries are denied enforced on user level
            $deny_enforced = $deny;

            // empty out the denied list
            $deny = array();

            // move all user level specified countries from allowed to denied list
            if ($user_acls['wildcard'] === 'allow') {
                foreach ($user_acls['deny'] as $code) {
                    if (array_key_exists($code, $deny_enforced) || array_key_exists($code, $allow_enforced)) {
                        continue;
                    }
                    $deny[$code] = $allow[$code];
                    unset($allow[$code]);
                }
            }
        }

        // return the country status lists
        return array($allow, $deny, $allow_enforced, $deny_enforced);
    }

    //! builds the HTML for the multi country select
    /*!
     * @param   array           the list of countries
     * @param   array           the list of enforced countries
     * @param   string          the type of country select (allowed, blocked)
     *
     * @returns mixed           the HTML for rendering the multi country select, FALSE on error
     */
    protected function buildMultiSelect($list, $enforced_list, $input_name) {
        // initialize array of allowed input names
        $allowed_input_name = array(
            'allowed_countries',
            'blocked_countries',
        );

        // sanity check
        if (!isset($list, $enforced_list, $input_name)
            || !WV::isArray($list)
            || !WV::isArray($enforced_list)
            || !in_array($input_name, $allowed_input_name)) {
            return FALSE;
        }

        // build the list of countries that the user can manipulate
        $user_policies = '';
        foreach ($list as $country_code => $country_name) {
            $user_policies .= "<li data-value=\"$country_code\">"
                           .  "    $country_name"
                           .  "    <input type=\"hidden\" name=\"{$input_name}[]\" value=\"$country_code\" />"
                           .  "</li>\n";
        }

        // build the list of countries that are enforced by the system
        $system_enforced = '';
        if (!empty($enforced_list)) {
            $system_enforced  = "<li class='mm-optgroup'><span>System Enforced</span><ul class='mm-multiselect-child'>";
            foreach ($enforced_list as $country_code => $country_name) {
                if ($input_name == "allowed_countries") {
                    $desc = sprintf(intl('country_auth_tooltip_allowed_enforced'), $country_name);
                    $input = "<input type=\"hidden\" name=\"{$input_name}[]\" value=\"$country_code\" />";
                } else {
                    $desc = sprintf(intl('country_auth_tooltip_blocked_enforced'), $country_name);
                    $input = "";
                }

                $system_enforced .= "<li class=\"non-selectable tooltip-description\" data-desc=\"$desc\" data-value=\"$country_code\">"
                                 .  "    $country_name"
                                 .  "    $input"
                                 .  "</li>\n";
            }
            $system_enforced .= "</ul></li>";
        }

        // build default list of countries should both lists be empty
        if (empty($list) && empty($enforced_list)) {
            if ($input_name == "allowed_countries" && $this->build_default) {
                foreach (Country::$codes as $country_code => $country_name) {
                    $system_enforced .= "<li data-value=\"$country_code\">"
                                     .  "    $country_name"
                                     .  "</li>\n";
                }
            }
        }

        // build and return the HTML for the multi country select block
        return "<ul id='$input_name' class='mm-multiselect' data-bind='${input_name}[]'>"
             . "    $user_policies"
             . "    $system_enforced"
             . "</ul>";
    }

    //! retrieve the migratable configurations of the module
    /*!
     * @returns array  the migratable configurations
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function getMigrationData() {
        // the list of migratable configurations
        $configurations = array();

        // retrieve and store the default server country
        $configurations[self::COUNTRY_AUTH_DEFAULT_CONTROL] = getControl(self::COUNTRY_AUTH_DEFAULT_CONTROL, "", MASTERCONTROLDIR);

        // retrieve and store the country authentication ACL
        $error_msg = "";
        $configurations[self::COUNTRY_AUTH_ACL_CONTROL] = $this->getCountryList(MASTERCONTROLDIR, $error_msg);

        // return the migration configurations
        return $configurations;
    }

    //! loads the migratable configurations of the module
    /*!
     * @param   array  the configurations to load
     *
     * @returns bool   whether the configurations were loaded successfully
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function loadMigrationData($configurations) {
        // sanity check
        if (!isset($configurations) || !WV::isArrayNotEmpty($configurations)) {
            return FALSE;
        }

        // prepare the country authentication ACL
        $wildcard = $configurations[self::COUNTRY_AUTH_ACL_CONTROL]['wildcard'];
        $word = ($wildcard == 'allow') ? "deny" : "allow";
        $list = $configurations[self::COUNTRY_AUTH_ACL_CONTROL][$word];

        // build and store country authentication ACL
        $content = '';
        foreach($list as $country_code) {
            $content .= "$country_code:$word\n";
        }
        $content .= "*:$wildcard";
        setControl(self::COUNTRY_AUTH_ACL_CONTROL, $content);

        // store the default server country
        setControl(self::COUNTRY_AUTH_DEFAULT_CONTROL, $configurations[self::COUNTRY_AUTH_DEFAULT_CONTROL]);

        // success, migratable configurations successfully loaded
        return TRUE;
    }
}
