<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * Copyright (c) LinuxMagic Inc. 2019 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Lucas Nincao <lucasn@linuxmagic.com>
 *
 * $Id: countryEmailStatsModule.php 10940 2020-01-14 00:44:10Z deiony $
 */

require_once('wizard/lib.wizard_validator.php');
require_once('lib.basestatsmodule.php');
require_once('lib.logsearch.php');
require_once('lib.country.php');

class countryEmailStatsModule extends BaseStatsModule {
    // values for time period filter
    const PERIOD_ALL_TIME = 0;
    const PERIOD_CURR_DAY = 1;

    // default values for filters
    const DEFAULT_PERIOD = countryEmailStatsModule::PERIOD_CURR_DAY;
    const DEFAULT_TOP_NUM = 5;
    const DEFAULT_EMAIL_TYPE = 'ALL';

    // arrays to store the allowed values posted from the form dropdown menus
    protected $allowedTopNum = array(5, 10, 15, 20, 25, 30);
    protected $allowedTimePeriod = array(
        countryEmailStatsModule::PERIOD_ALL_TIME,
        countryEmailStatsModule::PERIOD_CURR_DAY
    );
    protected $allowedEmailType = array(
        'ALL',
        'HAM',
        'SPAM',
        'EXEMPT',
        'RATELIMIT'
    );

    // values posted on any of the dropdrowns change
    protected $topNum;
    protected $timePeriod;
    protected $emailType;

    // array to store all the country email stats to avoid parsing twice
    protected $emailStatsData;

    //! class constructor
    public function __construct($l) {
        parent::__construct($l, FALSE);
        $this->topNum = countryEmailStatsModule::DEFAULT_TOP_NUM;
        $this->timePeriod = countryEmailStatsModule::DEFAULT_PERIOD;
        $this->emailType = countryEmailStatsModule::DEFAULT_EMAIL_TYPE;
        $this->emailStatsData = NULL;
    }

    //! method to process any actions and react accordingly
    public function actionProcessor($action, &$errors, &$notices) {
        // set the posted filter values
        if (isset($_POST['top-email'], $_POST['period-email'], $_POST['email-type'])) {
            // set an error message if the post validation fails
            if (!$this->setTopNum($_POST['top-email'])
                || !$this->setTimePeriod($_POST['period-email'])
                || !$this->setEmailType($_POST['email-type'])) {
                $errors[] = intl('invalid_entry');
                return;
            }

            // save the POST in the SESSION for the future
            $_SESSION['email_form'] = $_POST;
         }
    }

    //! method to render the module
    public function renderModule() {
        global $module;

        // Email: Top X countries: Inbound and Outbound
        $dataEmail = array();

        // get the email type header label
        $emailTypeHeaderLabel = $this->getEmailTypeHeaderLabel($this->emailType);

        // email tables (two columns: inbound and outbound)
        $dataEmail[0]['table'] = $this->countryEmailStatsTable('INBOUND');
        $dataEmail[0]['headerLabel'] = intl('emailCountryStatsHeaderInbound') . ": " .
             $emailTypeHeaderLabel;
        $dataEmail[0]['JSlabel'] = 'emailStatsHelpInbound';

        $dataEmail[1]['table'] = $this->countryEmailStatsTable('OUTBOUND');
        $dataEmail[1]['headerLabel'] = intl('emailCountryStatsHeaderOutbound') . ": " .
             $emailTypeHeaderLabel;
        $dataEmail[1]['JSlabel'] = 'emailStatsHelpOutbound';

        // help bubble dictionary for the JS
        $this->_i18n_js[] = 'emailStatsHelpInbound';
        $this->_i18n_js[] = 'emailStatsHelpOutbound';

        renderTemplate('countryEmailStats', array(
            'module'          => $module,
            'submodule'       => 'countryStats',
            'showSelection'   => TRUE,
            'topNum'          => $this->topNum,
            'timePeriod'      => $this->timePeriod,
            'emailType'       => $this->emailType,
            'data'            => $dataEmail,
            'help'            => intl('help'),
        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    public function getCSS() {
        return array();
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    public function getJS() {
        return array();
    }

    //! Creates a graph with the email stats, divided by country
    /*!
     * @param   string          INBOUND or OUTBOUND
     *
     * @returns string          stats graph html
     */
    protected function countryEmailStatsTable($direction) {
        // if any of these variables isn't set, return default empty table
        if ($this->topNum === NULL
            || $this->timePeriod === NULL
            || $this->emailType === NULL) {
            return $this->getDefaultTableHTML();
        }

        // check if we already have this data before going through the logs again
        if ($this->emailStatsData === NULL) {
            $this->emailStatsData = $this->getEmailByCountryLogs();
        }

        // assign friendly names
        $type = $this->emailType;
        $data = $this->emailStatsData;

        // returns an empty table if there are
        // no countries for this type of email
        if (empty($data[$direction][$type])) {
            return $this->getDefaultTableHTML();
        }

        // generate email stats table
        $table = $this->generateTable($data[$direction][$type]);

        // build the email stats table html string
        $html = $table->getRenderString();

        // return default empty table if an error occurs
        return $html ? $html : $this->getDefaultTableHTML();
    }

     //! Creates an array with all the email stats
     /*!
     * @returns array with all the email stats from the logs
     *
     * NOTE: separated by INBOUND/OUTBOUND -> EMAIL TYPE -> countries
     */
     protected function getEmailByCountryLogs() {
        $logSearch = new LogSearch();

        // determine how far back in time to parse the log files
        // NOTE: by default, only parse the log for the current day
        //       otherwise parse up to the log days control set by user
        $stop = 0;
        if ($this->timePeriod == countryEmailStatsModule::PERIOD_ALL_TIME) {
            $stop = (int)getControl("logdays", 7, CONTROLDIR);
        }

        return $logSearch->getTopEmailStatsByCountryRange(0, $stop, $this->topNum);
    }

    //! generates the label for the email country stats tables
    /*!
     * @param   string            country code index
     *
     * @returns html             string
     *
     * NOTE: overloaded to handle countries with names too long
     */
    protected function generateLabelForStatsTable($countryCode) {
        $countryName = Country::getCountryNameFromCode($countryCode);

        return "<div class='country-name' title='". $countryName ."'>"
              . $countryName
              ."</div>\n"
              ."<div class='country-code' title='". $countryName ."'>"
              . "($countryCode)"
              ."</div>\n";
    }

    //! Returns the appropriate label for each type of email
    /*!
     * @param   string            email type constant
     *
     * @returns string            email type label
     *
     */
    protected function getEmailTypeHeaderLabel($emailTypeValue) {
        switch($emailTypeValue) {
        case "SPAM":
            return intl('ls_spam');
        case "HAM":
            return intl('ls_not_spam');
        case "EXEMPT":
            return intl('ls_exempt');
        case "RATELIMIT":
            return intl('ls_ratelimit');
        default:
            return intl('ls_all');
        }
    }

    //! validates the top number
    /*!
     * @returns boolean         whether the number of selections was valid
     */
    protected function validateTopNum($top_num) {
        return isset($top_num)
            && WV::isIntPositiveOrZero($top_num)
            && in_array($top_num, $this->allowedTopNum);
    }

    //! validates the time period
    /*!
     * @returns boolean         whether the time period was valid
     */
    protected function validateTimePeriod($time_period) {
        return isset($time_period)
            && WV::isIntPositiveOrZero($time_period)
            && in_array($time_period, $this->allowedTimePeriod);
    }

    //! validates the email type
    /*!
     * @returns boolean            whether the email type was valid
     */
    protected function validateEmailType($email_type) {
        return isset($email_type)
            && WV::isStringNotEmpty($email_type)
            && in_array($email_type, $this->allowedEmailType);
    }

    //! set the number of selections
    /*!
     * @param int               the number of selections
     *
     * @returns boolean         whether the number of selections was set
     */
    public function setTopNum($top_num) {
        if (!isset($top_num) || !$this->validateTopNum($top_num)) {
            return FALSE;
        }
        $this->topNum = $top_num;
        return TRUE;
    }

    //! set the time period filter value
    /*!
     * @param int               the time period filter value
     *
     * @returns boolean         whether the time period was set
     */
    public function setTimePeriod($time_period) {
        if (!isset($time_period) || !$this->validateTimePeriod($time_period)) {
            return FALSE;
        }
        $this->timePeriod = $time_period;
        return TRUE;
    }

    //! set the type of email type filter value
    /*!
    * @param int               the email type filter value
     *
     * @returns boolean         whether the email type was set
    */
    public function setEmailType($email_type) {
        if (!isset($email_type) || !$this->validateEmailType($email_type)) {
            return FALSE;
        }
        $this->emailType = $email_type;
        return TRUE;
    }

   //! Returns the number of selections
   /*!
    * @returns int              the number of selections
    */
    public function getTopNum() {
        return $this->topNum;
    }

    //! Returns the period of time
    /*!
     * @returns int             the time period
     */
    public function getTimePeriod() {
        return $this->timePeriod;
    }

    //! Returns the email type
    /*!
     * @returns string          the email type
     */
    public function getEmailType() {
        return $this->emailType;
    }

}
