<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: feSpamSettingsModule.php 5131 2015-09-24 22:55:06Z shaun $
 */
require_once('functions.php');
require_once('lib.module.php');
require_once('lib.datatable.php');

class feSpamSettingsModule extends Module {
    protected $rules;

    //! string. this is to track the request action that was last run.
    // why do we have it? because for 'set recommended defaults' we want to also
    // be able to show, from javascript, a popup bubble warning. this is a way
    // to communicate with javascript to tell it which action was run.
    private $_action_flag = '';

    private $_flag_rules = array();

    // constants for the status of settings
    const SETTING_STATUS_OFF = 0;
    const SETTING_STATUS_FLAG = 1;
    const SETTING_STATUS_ON = 2;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     *
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, TRUE);

        $this->rules = array(
            'block_ip_in_addr' => array(
                'description' => 'Block addresses without domain (e.g. johnny@192.168.1.1)',
                'help' => 'sp_noDomain',
                'default' => self::SETTING_STATUS_ON,
            ), 'check_dynamic_reverse_dns_default' => array(
                'description' => 'Block Mail Servers using COMMON Dynamic/DUL IP space (DYNA REGEX COMMON)',
                'help' => 'sp_dynaRegex',
                'default' => self::SETTING_STATUS_FLAG,
            ), 'check_dynamic_reverse_dns' => array(
                'description' => 'Block Mail Servers using Probable Dynamic/DUL IP space (DYNA REGEX)',
                'help' => 'sp_dynaRegex',
                'default' => self::SETTING_STATUS_FLAG,
            ), 'check_confirmed_dyna' => array(
                'description' => 'Block Mail Servers using Confirmed Dynamic/DUL IP space (DYNA REGEX)',
                'help' => 'sp_dynaRegex',
                'default' => self::SETTING_STATUS_ON,
            ), "check_ip_reverse_dns" => array(
                "description" => "Require Mail Servers to have rDNS configured",
                "help" => 'sp_revDns',
                'default' => self::SETTING_STATUS_ON,
            ), 'check_reverse_dns_list' => array(
                'description' => 'Block Mail Servers reported as Spam Source (NONDUL REGEX)',
                'help' => 'sp_dnsList',
                'default' => self::SETTING_STATUS_FLAG,
            ), 'local_require_auth' => array(
                'description' => 'Local SMTP sender addresses must authenticate',
                'help' => 'sp_localRequireAuth',
                'default' => self::SETTING_STATUS_FLAG,
            ), "require_helo" => array(
                "description" => "Sending Server must identify itself (HELO)",
                "help" => 'sp_reqHelo',
                'default' => self::SETTING_STATUS_ON,
            ), "valid_helo_domain" => array(
                "description" => "Server Identification should be sane (FQDN HELO)",
                "help" => 'sp_validHelo',
                'default' => self::SETTING_STATUS_ON,
            ), "resolve_helo_domain" => array(
                "description" => "Confirm Server Identification Resolves (HELO)",
                "help" => 'sp_resolveHelo',
                'default' => self::SETTING_STATUS_OFF,
            ), "rfc_mail_from" => array(
                "description" => "MAIL FROM: must meet RFC2822 specifications",
                "help" => 'sp_rfcFrom',
                'default' => self::SETTING_STATUS_FLAG,
            ), "require_full_addr" => array(
                "description" => "Strict Email Address Parsing (RFC Compliance)",
                "help" => 'sp_fullAdd',
                'default' => self::SETTING_STATUS_ON,
            ), "valid_from_domain" => array(
                "description" => "Valid FROM domain (A or MX Record)",
                "help" => 'sp_validFrom',
                'default' => self::SETTING_STATUS_ON,
            ), "valid_ptr" => array(
                "description" => "PTR record should be FQDN (Best Practices)",
                "help" => 'sp_validPtr',
                'default' => self::SETTING_STATUS_ON,
            ),
        );
    }

    public function getPageTitle() {
        return intl($this->label);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        // store the action - we want to provide it to the javascript
        // environment in some cases.
        $this->_action_flag = $action;

        // sets the state of the collaspible modules
        if ($action === 'remember_expand_state') {

                $module = isset($_POST['changed_module']) ? $_POST['changed_module'] : NULL;
                $value  = isset($_POST['expanded']) ? $_POST['expanded'] : NULL;
                $var = $module . "__collapse";
                $_SESSION[$var] = $value;
        }

        // action to update what rules are enabled.
        if ($action == 'save_server_policies') {
            $rulesarr = array();
            $flagarr = array();
            $rulesstr = "";
            foreach ($_POST["rules"] as $key=>$value) {
                if ($value == "true" || $value == "flag") {
                    $rulesarr[] = $key;
                }
                if ($value == "flag") {
                    $flagarr[] = $key;
                }
            }
            $rulesstr = implode("\n", $rulesarr);

            if (!strlen($rulesstr)) {
                $ret = clearControl(RULES_ENABLED);
            } else {
                $ret = setControl(RULES_ENABLED, $rulesstr);
            }

            // let write the flags
            $flagRet = TRUE;
            // if PRO write the flag rules
            if (defined('RULES_FLAG')) {
                $flagstr = implode("\n", $flagarr);
                if (!strlen($flagstr)) {
                    $flagRet = clearControl(RULES_FLAG);
                } else {
                    $flagRet = setControl(RULES_FLAG, $flagstr);
                }
            }

            if ($ret === TRUE && $flagRet === TRUE) {
                $notices[] = intl("serverPolicies"). " " . intl("saved");
            } else { // TODO: localize the error from functions
                if ($ret !== TRUE) {
                    $errors[] = $ret;
                }
                if ($flagRet !== TRUE) {
                    $errors[] = $flagRet;
                }
            }

        // action to set rules to the defaults.
        } else if ($action == 'set_server_policies_to_defaults') {
            $rules_on = array();
            $rules_flag = array();

            foreach ($this->rules as $rule => $props) {
                switch ($props['default']) {
                case self::SETTING_STATUS_FLAG:
                    $rules_flag[] = $rule;
                case self::SETTING_STATUS_ON:
                    $rules_on[] = $rule;
                    break;
                case self::SETTING_STATUS_OFF:
                default:
                    break;
                }
            }

            $ret = setControl(RULES_ENABLED, implode("\n", $rules_on));
            if ($ret !== TRUE) {
                $errors[] = intl('errorRestoreRecommended');
                return;
            }

            $ret = setControl(RULES_FLAG, implode("\n", $rules_flag));
            if ($ret !== TRUE) {
                $errors[] = intl('errorRestoreRecommended');
                return;
            }

            $notices[] = intl("serverPolicies"). " " . intl("saved");
        }

        // action to save the quarantine settings
        if ($action === 'save_quarantine_settings') {
            $flag_local = isset($_POST['flag_local']) ? 1 : 0;
            $quarantine_local = isset($_POST['quarantine_local']) ? 1 : 0;
            $qsubject = isset($_POST['qsubject']) ? trim($_POST['qsubject']) : "";
            if (setControl('flag_local', $flag_local) !== TRUE
                || setControl('quarantine_local', $quarantine_local) !== TRUE
                || setControl('qsubject', $qsubject) !== TRUE) {
                $errors[] = intl('fe_quarantine_save_error');
                return;
            }

            // print save success message
            $notices[] = intl('fe_quarantine_save_success');
        }
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $module;

        $enabled_rules = array();

        // adding warning labels so they are available in js for the KSF warning dialog
        $this->_i18n_js[] = 'ksfCloseLabel';
        $this->_i18n_js[] = 'ksfWarningMessage';

        // and help popups
        foreach ($this->rules as $rule => &$data) {
            $this->_i18n_js[] = $data['help'];
        }

        // load the enabled items from rules.enabled
        $tmp = getControl(RULES_ENABLED, "");
        if (strlen($tmp) > 0) { // something was returned, set the rules and do the count
            $enabled_rules = explode("\n", $tmp);
        }

        // set the enabled values
        $this->_setEnabled($enabled_rules);

        // set up the base column classes for rendering
        $rowClasses = array('ruleDesc', 'recommendation', 'ruleStatus');

        $header = '<span class="ruleDesc">' . intl('serverPoliciesRule') . "</span>\n"
                . '<span class="recommendation">&nbsp;</span>' . "\n"
                . '<span class="ruleStatus">' . intl('status') . "</span>\n";

        // if this is not a 'client' frontend
        if (!$this->_setLockedRules()) {
            $rowClasses[] = 'ruleActions';
        }

        $this->_getFlagRules();

        // prepare our table
        $table = $this->_prepareDataTable($rowClasses);

        // Prints out the menu
        renderMenu();

        // Renders the template
        renderTemplate('feServerPolicies', array(
            'mainHeader'            => intl("serverPolicies"),
            'blurb'                 => intl("serverPoliciesBlurb"),
            'table'                 => $table->getRenderString(),
            'module'                => $module,
            'save'                  => intl("save"),
            'defaults'              => intl("defaults"),
            'header'                => $header,
            'showRejectionsLink'    => '',
            'rejectionEditor'       => intl('rejectionEditor'),
            'rejectionInstructions' => intl('rejectionInstructions'),
            'charsLeft'             => intl('charsLeft'),
            'image_path'            => IMAGEPATH,
            'actionFlag'            => htmlspecialchars($this->_action_flag),
            'collapseState'         => 1,
            // grab the user-level quarantine settings
            'flag_local'            => getControl('flag_local', getControl('flag_local', 0, MASTERCONTROLDIR)),
            'qsubject'              => getControl('qsubject', getControl('qsubject', '', MASTERCONTROLDIR)),
            'quarantine_local'      => getControl('quarantine_local', getControl('quarantine_local', 0, MASTERCONTROLDIR)),
        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     */
    public function getCSS() {
        return array('server-policies.css', 'quarantine-settings.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array(
                     'ajax.js'
                     ,'server-policies.js'
                     ,'help-popup.js'
                    );
    }

    //! internal method to prep the 'default' rule status
    /*!
     * @param   array   the array of rules that are currently enabled.
     */
    private function _setEnabled($enabled_rules) {
        // safety check to ensure we are working with an array.
        if (!is_array($enabled_rules)) {
            $enabled_rules = array();
        }
        // for each rule that is in the list
        foreach ($this->rules as $rule => &$data) {
            // then, set the 'enabled' status based on the passed in array.
            $data['enabled'] = in_array($rule, $enabled_rules);
        }
        // done - all other information pass through is held in internal variable '$rules'
    }

    //! internal method for determining any 'locked out' rules based on where this is being run.
    /*!
     * @returns boolean TRUE if this is a 'per user' frontend that has locked rules
     *                  FALSE if this is the administrator interface and thus unlocked.
     */
    private function _setLockedRules() {
        // we need access to a global variable for cases of MS PRO
        global $controller, $isFrontend;
        // we zero in on the current PRO designation for client frontend
        if (  (isset($controller) && $controller == 'magicspam.live.php')
           || (isset($isFrontend) && $isFrontend) ) {
            $masterEnabled = array();

            // grab the 'master' server level enabled rules
            $tmp = getControl(RULES_ENABLED, "", MASTERCONTROLDIR);
            if (strlen($tmp) > 0) {
                $masterEnabled = explode("\n", $tmp);
            }

            $masterFlag = array();
            // grab the 'master' server level Flag rules
            $tmp = getControl(RULES_FLAG, "", MASTERCONTROLDIR);
            if (strlen($tmp) > 0) {
                $masterFlag = explode("\n", $tmp);
            }

            // and now we lock these out from being edited
            // to go only higher than the master control
            foreach ($this->rules as $rule => &$data) {
                // if the rule is flag on master the we set bymaster 1, frontend user can change to ON only
                // if the rule is enabled on master the we set bymaster 2, frontend user can NOT change at all
                // if the rule is NOT enabled on master the we set bymaster 0, frontend user can change to flag or ON
                if (in_array($rule, $masterEnabled) && in_array($rule, $masterFlag)) {
                    $data['bymaster'] = 1;
                } elseif (in_array($rule, $masterEnabled)) {
                    $data['bymaster'] = 2;
                } else {
                    $data['bymaster'] = 0;
                }
            }
            return TRUE;
        }
        return FALSE;
    }

    //! internal method to prepare a data table for rendering
    /*!
     * @param array $columnClasses TODO document
     *
     * @return magicspamDataTable
     *
     * @author Anonymous
     * @author William Storey <william@linuxmagic.com>
     */
    private function _prepareDataTable($columnClasses) {
        global $isFrontend;
        $defstat = "";

        // safety check
        if (!is_array($columnClasses)) {
            $columnClasses = array();
        }
        // create our table object
        $table = new magicspamDataTable();

        // set the classes
        // XXX: what happens if its empty?
        $table->setClass("row", $columnClasses);

        // and generate our data rows
        foreach ($this->rules as $rule => &$data) {
            $help = $data['help'];
            $description = $data['description'];

            $internal = htmlspecialchars(intl('rule_name')) . ": " . htmlspecialchars($rule);
            $desc = renderListDescription($description, $internal, NULL,
                "MS.showHelpPopup(this,'X', '".$help."'); return false;",
                "_blank");

            // determine the rule status (OFF, FLAG or ON)
            $ruleEnabled = FALSE;
            if (isset($data['enabled']) && $data['enabled'] === TRUE) {
                $ruleEnabled = TRUE;
            }
            if ($ruleEnabled === TRUE && in_array($rule, $this->_flag_rules)) {
                $ruleEnabled = 'flag';
            }

            // alter rule status and generate label depending on the rule status on master
            switch ($data['bymaster']) {
                // if the rule is enabled on master the we set bymaster 2, frontend user can NOT change at all
                case 2:
                    // force rule to be ON
                    $ruleEnabled = TRUE;
                    $defstat = generateLabel("enforced");
                    break;

                // if the rule is flag on master the we set bymaster 1, frontend user can change between FLAG and ON only
                case 1:
                    // default to flag when previously OFF
                    if ($ruleEnabled == FALSE) {
                        $ruleEnabled = 'flag';
                    }
                    $defstat = generateLabel("restricted");
                    break;

                // if the rule is NOT enabled on master the we set bymaster 0, frontend user can change to OFF, FLAG or ON
                default:
                    $defstat = generateLabel("recommended", $data['default'] !== self::SETTING_STATUS_OFF ? TRUE : FALSE);
                    if ($data['default'] === self::SETTING_STATUS_FLAG) {
                        $defstat = str_replace("isdefault", "isdefaultflag", $defstat);
                    }
                    break;
            }

            $isLocked = FALSE;
            if (isset($data['bymaster']) ) {
                $isLocked = $data['bymaster'];
            }

            // build the slider
            $statusWidget = renderSliderThree($ruleEnabled, $isLocked, "rules[$rule]", 'serverPoliciesForm');

            // add row to the table
            $rowArr = array($desc, $defstat, $statusWidget);
            $table->addRow($rowArr, NULL);
        }
        return $table;
    }

    //! internal method to get flag rules
    private function _getFlagRules() {
        // load the flagonly items from rules.flagonly
        $tmp = getControl(RULES_FLAG, "");
        // something was returned, explode the rules into an array
        if (strlen($tmp) > 0) {
            $flag_rules = explode("\n", $tmp);
        }
        // make sure that we have acutally something in flag_rules array
        if (isset($flag_rules) && is_array($flag_rules) && count($flag_rules)) {
            $this->_flag_rules = $flag_rules;
        } else {
            $this->_flag_rules = array();
        }
    }

}
