<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: logSearchModule.php 11866 2020-07-08 00:08:11Z deiony $
 */
require_once('lib.module.php');
require_once('lib.country.php');
require_once('lib.logsearch.php');
require_once('wizard/lib.wizard_validator.php');

class logSearchModule extends Module
{
    //! string. request action.
    protected $action;

    //! string. sender address search filter string.
    protected $sender;

    //! string. recipient address search filter string.
    protected $recipient;

    //! string. auth user address search filter string.
    protected $auth_user;

    //! string. IP address search filter string.
    protected $ip_address;

    //! integer. the filter type
    protected $filter;

    //! integer. the mua type
    protected $mua;

    //! integer. the number of log results to limit to.
    protected $result_limit;

    //! string. the American date search filter string (yyyy-mm-dd - yyyy-mm-dd)
    protected $date;

    //! string. the American date search filter string (yyyy-mm-dd)
    protected $startDate;

    //! string. the American date search filter string (yyyy-mm-dd)
    protected $endDate;

    //! integer the hour to search for the hourly log search
    protected $hour;

    //! integer whether to use yesterday or today log in hourly log search
    protected $yesterday;

    //! string. alternative controller that might affect render links
    protected $controller;

    //! internal flag for IE rendering compatibility.
    protected $isIE;

    //! internal flag for Microsoft Management Console (MailEnable Admin Console)
    protected $isMMC;

    //! the output filename when downloading the logsearch results in CSV format
    const CSV_OUTPUT_FILE = "search_result.csv";

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        if (defined('FE_LS_CONTROLLER')) {
            $this->controller = FE_LS_CONTROLLER;
        } else if (defined('LS_CONTROLLER')) {
            $this->controller = LS_CONTROLLER;
        }
        $this->isIE = FALSE;
        $this->isMMC = ($_SERVER['SERVER_PORT'] == 6242);
    }

    public function actionProcessor($action, &$errors, &$notices) {
        extract($_POST);
        if (isset($action) && WV::isStringNotEmpty($action)) {
            $this->action = trim($action);
        }
        if (isset($from_addr) && WV::isStringNotEmpty($from_addr)) {
            $this->sender = trim($from_addr);
        }
        if (isset($rcpt_addr) && WV::isStringNotEmpty($rcpt_addr)) {
            $this->recipient = trim($rcpt_addr);
        }
        if (isset($auth_addr) && WV::isStringNotEmpty($auth_addr)) {
            $this->auth_user = trim($auth_addr);
        }
        if (isset($ip_address) && WV::isStringNotEmpty($ip_address)) {
            $ip_address = trim($ip_address);
            if (filter_var($ip_address, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)) {
                $ip_address = inet_ntop(inet_pton($ip_address));
            }
            $this->ip_address = $ip_address;
        }
        if (isset($mua_status) && WV::isIntPositiveOrZero($mua_status)) {
            $this->mua = $mua_status;
        }
        if (isset($filter) && WV::isIntPositiveOrZero($filter)) {
            $this->filter = $filter;
        }
        if (isset($result_limit) && WV::isIntPositive($result_limit)) {
            $this->result_limit = $result_limit;
        }
        if (isset($start_date, $end_date)
            && $this->isDate($start_date)
            && $this->isDate($end_date)) {
            $this->startDate = $start_date;
            $this->endDate = $end_date;
            $this->date = "$start_date - $end_date";
        }
        if (isset($ie) && WV::isIntPositive($ie)) {
            $this->isIE = TRUE;
        }
        if (isset($ms_hour) && $ms_hour != -1) {
            $this->hour = intval($ms_hour);
        }
        if (isset($yesterday) && $yesterday != -1) {
            $this->yesterday = intval($yesterday);
        }
    }

    //! renders the display for screen, this is an overwritten method
    /*!
     * @returns void
     *
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author William Storey <william@linuxmagic.com>
     * @author Robert Bisewski <robertb@linuxmagic.com>
     *
     */
    public function renderModule()  {
        global $module, $isFrontend, $feUser;

        // Adds the help content to the help pop-up
        $this->_i18n_js[] = 'logSearchHelp';
        $this->_i18n_js[] = 'logSearch.error.invalidSenderCriteria';
        $this->_i18n_js[] = 'logSearch.error.invalidRecipientCriteria';
        $this->_i18n_js[] = 'logSearch.error.invalidAuthUserCriteria';
        $this->_i18n_js[] = 'logSearch.error.invalidIPCriteria';
        $this->_i18n_js[] = 'logSearch.error.invalidLimitCriteria';
        // XXX: need to review the frontend error for date filter
        $this->_i18n_js[] = 'logSearch.error.invalidDateCriteria';
        $this->_i18n_js[] = 'logSearch.error.invalidDateRangeCriteria';

        renderTemplate('logSearch', array(
            "module"     => $module,
            "data_table" => $this->makeSearchForm(),
            "isFrontend" => isset($isFrontend) ? $isFrontend : FALSE,
            "controller" => isset($this->controller) ? $this->controller : "logsearch.php",
            "limit"      => intl("limit"),
            "search"     => intl("search"),
            "title"      => intl("logSearchTitle"),
            "subtitle"   => intl("logSearchSubtitle"),
            "help"       => intl('help'),
            "desc"       => intl("logSearchDesc"),
            "overview"   => intl('logSearchOverview'),
            "target"     => $this->isMMC ? "" : "__NEW",
            // XXX: little hack for interworx
            "magicspam_siteworx_user" => isset($feUser) ? $feUser : "",
        ));
    }

    //! method to retrieve any 'additional' JS files to include in the page rendering
    /*!
     * @returns array   an array of the files to include
     */
    public function getJS() {
        return array(
            'help-popup.js',
            'magicspam.js',
            'magicspam-menu.js',
            'limit-integer.js',
            'check-enabled.js',
            'ajax.js',
            'form-submit.js',
            'log-search.js',
            'nodep-date-input-polyfill.dist.js',
        );
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    public function getCSS() {
        return array('log-search.css');
    }

    //! method to generate a render string for the log search table
    protected function makeSearchForm() {
        // initialize a new data table
        $table = new magicspamDataTable;

        // add row for sender input
        $table->addRow(array(
            intl("from"),
            '<input class="input" type="text" placeholder="user@example.com" id="from_addr" name="from_addr" maxlength="' . ADDRESS_SEARCH_MAX . '"/>',
            '<div class="inputLabel">' . intl("logSenderLabel") . '</div>'
        ));

        // add row for recipient input
        $table->addRow(array(
            intl("to"),
            '<input class="input" type="text" placeholder="user@example.com" id="rcpt_addr" name="rcpt_addr" maxlength="' . ADDRESS_SEARCH_MAX . '"/>',
            '<div class="inputLabel">' . intl("logRecipLabel") . '</div>'
        ));

        // add row for auth user input
        $table->addRow(array(
            intl("auth_user"),
            '<div>' .
                '<input class="input" type="text" placeholder="user@example.com" id="auth_addr" name="auth_addr" maxlength="' . ADDRESS_SEARCH_MAX . '"/>' .
                '<div class="ls_divider">' . intl('mua') . '</div>' .
                '<select name="mua_status" id="mua_status" onchange="logSearchToggleAuthUserFilter(this)">' .
                    '<option value="' . FILTER_MUA_BOTH . '" selected="selected">' . intl('both') . '</option>' .
                    '<option value="' . FILTER_MUA_YES . '">Yes</option>' .
                    '<option value="' . FILTER_MUA_NO . '">No</option>' .
                '</select>' .
            '</div>',
            '<div class="inputLabel">' . intl("logAuthUserLabel") . '</div>'
        ));

        // add row for ip address input
        $table->addRow(array(
            intl('ip'),
            '<input class="input" type="text" placeholder="192.168.0.210" id="ip_address" name="ip_address" maxlength="' . LSM_IP_ADDRESS_SEARCH_MAX . '">',
            '<div class="inputLabel">' . intl('logIPLabel') . '</div>'
        ));

        // add row for date range input
        $table->addRow(array(
            intl('date_range'),
            '<div>' .
                '<input type="date" id="start_date" name="start_date" class="date" onkeydown="return false;">' .
                '<div class="ls_divider">' . intl('date_to') . '</div>' .
                '<input type="date" id="end_date" name="end_date" class="date" onkeydown="return false;">' .
            '</div>',
            '<div class="inputLabel">' . intl('logDateLabel') . '</div>'
        ));

        // build the filter input
        $filterInput = '<div>'
                     .     '<select name="filter" id="filter">'
                     .         '<option value="' . FILTER_ALL . '">' . intl('ls_all') . '</option>'
                     .         '<option value="' . FILTER_SPAM . '" selected="selected"> ' . intl('ls_spam') . '</option>'
                     .         '<option value="' . FILTER_HAM . '">' . intl('ls_not_spam') . '</option>'
                     .         '<option value="' . FILTER_EXEMPT . '">' . intl('ls_exempt') . '</option>'
                     .         '<option value="' . FILTER_RATELIMIT . '">' . intl('ls_ratelimit') . '</option>'
                     .     '</select>'
                     .     '<input class="limit" type="checkbox" name="limit_results" onclick="checkEnabled(this, this.form.result_limit, \'50\');" checked>'
                     .     intl("limit") . '&nbsp;'
                     .     '<input type="text" class="numeric" id="result_limit" name="result_limit" onkeyup="limitInteger(this);" size="2" value="50" maxlength="' . RESULTS_SEARCH_MAX . '">'
                     . '</div>';

        // add row for filter input
        $table->addRow(array(
            '<span class="filterAlign">' . intl('ls_filter_by') . '</span>',
            $filterInput,
            '<div class="inputLabel">' . intl('logFilterLabel') . '</div>'
        ));

        // build the html for the search and clear button
        $searchButton = '<button class="msbutton" id="log_search_btn" type="submit"><span>' . intl("search") . '</span></button>';
        $clearButton  = '<button class="msbutton" id="log_clear_btn" type="reset"><span>&nbsp;' . intl("clear") . '&nbsp;</span></button>';

        // add row for the search and clear button
        $table->addRow(array(
            "", "", $searchButton . $clearButton
        ));

        // return the HTML string for rendering the log search form
        return $table->getRenderString();
    }

    //! trigger when an ajax call is made
    public function renderOutput($modules) {
        global $home;

        // initialize the logsearch handle
        $logsearch_handle = new LogSearch();

        // prepare the variables for the search
        $base_file = (isset($home) ? "$home/magicspam/logs/" : LOGDIR) . "mslog";
        $log_max_rows = isset($this->result_limit) ? $this->result_limit : SEARCH_RESULTS_SANITY;
        $max_days = (int)getControl("logdays", 7, CONTROLDIR);
        $data = array();
        $more_than_requested = FALSE;

        // set up the filters
        $logsearch_handle->setFilter($this->filter);
        $logsearch_handle->setMua($this->mua);
        $logsearch_handle->setSender($this->sender);
        $logsearch_handle->setRecipient($this->recipient);
        $logsearch_handle->setAuthUser($this->auth_user);
        $logsearch_handle->setIpAddress($this->ip_address);
        $logsearch_handle->setHour($this->hour);
        $logsearch_handle->setYesterday($this->yesterday);
        $logsearch_handle->setDate($this->startDate, $this->endDate);

        // perform the log search
        if ($logsearch_handle->parseLogFiles($base_file, $log_max_rows, $max_days + 1, $data, $more_than_requested) === FALSE) {
            return;
        }

        // prepare the headers
        $headers = array(
            "timestamp",
            "type",
            "mua",
            "auth_user",
            "ip",
            "host",
            "country",
            "helo",
            "from",
            "rcpt"
        );

        // remove the country header when not professional
        if (!isProfessional()) {
            array_splice($headers, 6, 1);
        }
        // render the output in the appropriate format depending on the action
        if ($this->action === "search") {
            $this->renderOutputAsHTML($headers, $data, $more_than_requested);
        } else if ($this->action === "csv") {
            $this->renderOutputAsCSV(self::CSV_OUTPUT_FILE, $headers, $data, $more_than_requested);
        }
    }

    //! render the output as a csv file
    protected function renderOutputAsCSV($output_file, $headers, $data, $more_than_requested) {
        // build the list of i18n labels for each header
        foreach ($headers as $header) {
            $headers_label[] = intl($header);
        }

        // turn off output compression
        ini_set('zlib.output_compression', 'Off');

        // determine the octet-stream MIME type based on IE browser
        $octetstream = ($this->isIE) ? "octetstream" : "octet-stream";

        // full list of potential HTTP headers for CSV download
        $csv_headers = array(
            "HTTP/1.1 200 OK",
            "Pragma: public",
            "Last-Modified: " . gmdate('D, d M Y H:i:s') . " GMT",
            "Cache-Control: no-store, no-cache, must-revalidate",
            "Cache-Control: pre-check=0, post-check=0, max-age=0",
            "Content-Transfer-Encoding: none",
            "Content-Type: application/$octetstream; name=\"$output_file\"",
            "Content-Type: application/force-download",
            "Content-Disposition: attachment; filename=\"$output_file\"",
        );

        // remove the Cache-Control headers when not IE browser
        if (!$this->isIE) {
            array_splice($csv_headers, 3, 2);
        }

        // remove the status code header when not building the CSV headers string
        if (!defined("LS_CSV_HEADERS_STRING")) {
            array_splice($csv_headers, 0, 1);
        }

        // send the HTTP headers or build the CSV headers string
        $csv_headers_string = "";
        foreach ($csv_headers as $csv_header) {
            if (defined("LS_CSV_HEADERS_STRING")) {
                $csv_headers_string .= "$csv_header\n";
            } else {
                header($csv_header);
            }
        }

        // open the output buffer mechanism for writing
        $handle = fopen('php://output', 'w');

        // write the CSV headers string to the output buffer if necessary
        if (defined("LS_CSV_HEADERS_STRING")) {
            fwrite($handle, "$csv_headers_string\n");
        }

        // write the log search column headers to the output buffer
        fputcsv($handle, $headers_label);

        // write the log search results to the output buffer
        foreach ($data as $line) {
            // drop the country column when not professional
            if (!isProfessional()) {
                array_splice($line, INDEX_COUNTRY, 1);
            }
            // combine the log search type and rule if necessary
            $rule = array_splice($line, INDEX_RULE, 1)[0];
            if ($line[INDEX_TYPE] !== "HAM") {
                $line[INDEX_TYPE] .= "[$rule]";
            }
            fputcsv($handle, $line);
        }
        fclose($handle);
    }

    //! renders the output as HTML
    /*!
     * @param array $headers the headers of the table
     *
     * @return void
     *
     * @author Peter Chung <peterc@linuxmagic.com>
     * @author Shaun A. Johnson <shaun@linuxmagic.com>
     * @author Sean West <sean@linuxmagic.com>
     */
    protected function renderOutputAsHTML(&$headers, &$log_data, $more_than_requested) {
        // required for CSV download handoff (ajax call)
        global $module, $isFrontend;

        // ensure there is log data to render
        if (empty($log_data)) {
            echo "<div id=\"logresults_NA\">" . intl("no_result") . "</div>\n";
            return;
        }

        // build the list of i18n labels and CSS classes for each header
        foreach ($headers as $header) {
            $headers_label[] = intl($header);
            $headers_class[] = "header_$header log_cell";
            $columns_class[] = "log_$header log_cell";

            // the adjacent resize cells
            $headers_label[] = "";
            $headers_class[] = "resize_$header log_cell";
            $columns_class[] = "resize_$header log_cell";
        }

        // set up the log search header
        $search_header = new magicspamDataTable;
        $search_header->setHeader($headers_label);
        $search_header->setClass("headerRow", $headers_class);

        // set up the log search table
        $search_table = new magicspamDataTable;
        $search_table->setClass("row", $columns_class);

        // process each row of log data
        foreach ($log_data as $row_index => $row) {
            // reset the data row and mail type
            $drow = array();
            $mailType = "";

            // process each data item in row
            foreach ($row as $index => $ditem) {
                // skip the rule index and country index when not professional
                if ($index === INDEX_RULE || (!isProfessional() && $index == INDEX_COUNTRY)) {
                    continue;
                }

                // reset to default values
                $class = "";
                $data = $ditem;
                $title = $ditem;
                $menu = NULL;

                // format the data appropriately based on the index type
                switch ($index) {
                case INDEX_DATETIME:
                    // format datestring into friendlier form
                    $data = date("l, M d Y @ g:i:s A", strtotime($ditem));
                    break;
                case INDEX_TYPE:
                    // XXX: don't actually need specialized mailTypes since we can pass rule name to GUI JS functions
                    switch ($ditem) {
                    case "SPAM":
                        $class = "isSpam";
                        $mailType = "spam";
                        if ($row[INDEX_RULE] == "ip_blacklist") {
                            $mailType = "spam_ip_blacklist";
                        }
                        break;
                    case "QUARANTINE":
                        $class = "isSpam";
                        $mailType = "quarantine";
                        break;
                    case "HAM":
                    case "NDR":
                        $class = "isHam";
                        $mailType = "ham";
                        break;
                    case "EXEMPT":
                        $class = "isExempt";
                        // XXX: exempt mail type is not actually being used at the moment
                        //      however it does seems to have been used in the past
                        $mailType = "exempt";
                        break;
                    case "RATELIMIT":
                        $class =  "isRatelimit";
                        $mailType = "ratelimit";
                        if ($row[INDEX_RULE] == "inbound") {
                            $mailType = "ratelimit_inbound";
                        }
                        break;
                    default:
                        break;
                    }
                    if ($mailType === 'ham') {
                        if (isProfessional()) {
                            $menu = $this->generateMenuGuiOnClick($mailType, "Type", $row_index);
                        }
                    } else {
                        $data = $ditem . "[" . $row[INDEX_RULE] . "]";
                    }
                    break;
                case INDEX_IP:
                    if ($mailType !== "exempt" && $mailType !== "ratelimit"
                        && !($mailType === "ratelimit_inbound" && isset($isFrontend) && $isFrontend)) {
                        $menu = $this->generateMenuGuiOnClick($mailType, "IP", NULL);
                    }
                    break;
                case INDEX_COUNTRY:
                    $title = Country::getCountryNameFromCode($ditem);
                    if (($mailType === "ham" || $mailType === "quarantine") && $ditem != "**") {
                        $menu = $this->generateMenuGuiOnClick($mailType, "Country", $title);
                    }
                    break;
                case INDEX_SENDER:
                    if ($mailType !== "spam_ip_blacklist" && $mailType !== "exempt"
                        && $mailType !== "ratelimit_inbound" && $mailType !== "ratelimit") {
                        $menu = $this->generateMenuGuiOnClick($mailType, "Sender", NULL);
                    }
                    break;
                case INDEX_AUTHUSER:
                    // outbound ratelimit functionality only exists on Lite and Pro products
                    if ($mailType === "ratelimit" && isProfessional()) {
                        $menu = $this->generateMenuGuiOnClick($mailType, "Authuser", NULL);
                    }
                    break;
                default:
                    // no formatting required
                    break;
                }

                // encode the data and title for html
                $encodedData = htmlspecialchars($data);
                $encodedTitle = htmlspecialchars($title);

                // append the html to the data row
                if (!isset($menu)) {
                    $drow[] = "<a class=\"$class\" title=\"$encodedTitle\">$encodedData</a>";
                } else {
                    $drow[] = "<a class=\"$class\""
                            . "   title=\"" . intl('ls_click_for_more_options') . "\""
                            . "   href=\"#\""
                            . "   onclick=\"$menu\""
                            . ">$encodedData</a>";
                }

                // append empty string to the data row to initialize the resize cell
                $drow[] = "";
            }

            // append the data row to the search table
            $search_table->addRow($drow);
        }

        // build the link for downloading the log search results in CSV format
        $csv_download = sprintf("<a href=\"#\" onclick=\"downloadCSV('%s', 'logSearch', 'csv', '%s', '%s', '%s', %d, '%s', '%s', '%s', %d, %d, %d, %d, '%s'); return false;\">%s</a>",
            htmlspecialchars($module),
            htmlspecialchars($this->sender),
            htmlspecialchars($this->recipient),
            htmlspecialchars($this->auth_user),
            htmlspecialchars($this->mua),
            htmlspecialchars($this->ip_address),
            htmlspecialchars($this->startDate),
            htmlspecialchars($this->endDate),
            htmlspecialchars($this->filter),
            isset($this->result_limit) ? htmlspecialchars($this->result_limit) : -1,
            isset($this->hour) ? htmlspecialchars($this->hour) : -1,
            isset($this->yesterday) ? htmlspecialchars($this->yesterday) : -1,
            isset($this->controller) ? htmlspecialchars($this->controller) : "",
            intl('click_to_download')
        );

        // build the log search result div which includes the csv download link
        $search_result = sprintf("<div class=\"search_result\">%s: %s %d %s $csv_download</div>",
            intl("rows_found"),
            $more_than_requested ? intl('more_than') : "",
            count($log_data),
            $more_than_requested ? intl('too_much_rows') : ""
        );

        // prepare to inject interface information to the frontend
        $inject["isFrontend"] = (isset($isFrontend) && $isFrontend) ? "true" : "false";

        // render some additional html for replayer when professional
        $replay = "";
        if (isProfessional()) {
            // perpare to inject log data information to the frontend
            $inject["searchResult"] = json_encode($log_data);
            // add empty div for replayer results
            $replay = "<div id=\"magicspam-wrapper\">"
                    . "    <div id=\"replay-popup\"></div>"
                    . "</div>";
        }

        // build the script for injecting information to frontend
        $inject_string = "";
        foreach ($inject as $key => $val) {
            $inject_string .= "    MS.$key = $val;\n";
        }
        $script = "<script>\n"
                . "    var MS = MS || {};\n"
                . "    $inject_string\n"
                . "</script>\n";

        // build the the entire log search results page
        $html = $search_result
              . "<div id=\"logresults\">"
              . $search_header->getRenderString()
              . $search_table->getRenderString()
              . "</div>"
              . $search_result
              . $replay
              . $script;

        // render it out!
        echo $html;
    }

    protected function generateMenuGuiOnClick($mail_type, $menu_type, $optional) {
        return sprintf("MS.menuGUI(this, '%s', '%s' %s); return false;",
            $mail_type,
            $menu_type,
            isset($optional) ? WV::isInt($optional) ? ", $optional" : ", '$optional'" : ""
        );
    }

    //! validate date string with yyyy-mm-dd format
    protected function isDate($date) {
        // sanity check
        if (!isset($date) || !WV::isStringNotEmpty($date)) {
            return FALSE;
        }
        // ensure month, day and year are separated by slash character
        $date_arr = explode('-', $date);
        if (count($date_arr) != 3) {
            return FALSE;
        }
        // ensure month, day and year are positive integers
        if (!WV::isIntPositive($date_arr[0])
            || !WV::isIntPositive($date_arr[1])
            || !WV::isIntPositive($date_arr[2])) {
            return FALSE;
        }
        // ensure month, day and year are valid
        return checkdate($date_arr[1], $date_arr[2], $date_arr[0]);
    }
}
