<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of the MagicSpam (TM) Product line and is
 * Copyright (c) LinuxMagic Inc. 2008-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Shaun A. Johnson <shaun@linuxmagic.com>
 *            William Storey <william@linuxmagic.com>
 *
 * $Id: protectionKeyModule.php 11866 2020-07-08 00:08:11Z deiony $
 */
require_once('lib.module.php');
require_once('lib.listmanager.php');
require_once('lib.register.php');

class protectionKeyModule extends Module {
    //! whether magicspam was freshly installed
    protected $first_installation;

    //! the license key edit mode
    protected $edit_mode;

    //! the license key edit mode type
    const EDIT_MODE_UNKNOWN = 0;
    const EDIT_MODE_LICENSE = 1;
    const EDIT_MODE_ADDON   = 2;

    //! class constructor
    public function __construct($l) {
        parent::__construct($l, FALSE);

        // initialize class variables
        $this->first_installation = FALSE;
        $this->edit_mode = self::EDIT_MODE_UNKNOWN;

        // load the submodules
        // NOTE: most magicspam products loads the serviceStatus submodule in
        //       the main controller, but we load it here anyway as an explicit
        //       reminder that the module uses the submodule functionality
        $this->submodules = array(
            'serviceStatus',
        );
        foreach ($this->submodules as $m) {
            loadModule($m);
        }
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        global $admin_email, $license, $pages, $modules;

        // pass on any requested action to submodule
        // XXX: the subsubmodule methodology is dumb, lets try to figure out a
        //      better way to work with this in the future
        $submodule = isset($_POST['subsubmodule']) ? $_POST['subsubmodule'] : NULL;
        if ($submodule && isset($modules[$submodule]) && in_array($submodule, $this->submodules)) {
            return $modules[$submodule]->actionProcessor($action, $errors, $notices);
        }

        // retrieve the magicspam status
        $status = getControl("enabled", 0);

        // toggle the magicspam status
        if ($action === 'toggleStatus') {
            // stop magicspam from enabling when license is not valid
            if(!isFreemium() && !$license->isValid() && $status == 0) {
                $errors[] = intl('upgrade_required');
                return;
            }

            // ensure admin email is set before enabling magicspam
            if ($status == 0 && $admin_email === '') {
                $errors[] = intl('ae_please_complete');
                return;
            }

            // toggle the magicspam status using the magicspam register utility
            $register = Register::getInstance();
            if (!$register->toggle()) {
                $errors[] = $register->error();
                return;
            }

            // run the magicspam and spamassassin custom rules updater
            $this->_runUpdater();
            $this->handleSaCustomRules();
        }

        // move from license view mode to license edit mode
        if ($action === 'editLicense') {
            $this->edit_mode = self::EDIT_MODE_LICENSE;
        } else if ($action === 'editAddonLicense') {
            $this->edit_mode = self::EDIT_MODE_ADDON;
        }

        // save the license key
        if ($action === 'saveLicense') {
            // retrieve the license key user input
            $license_key = isset($_POST['licenseKey']) ? $_POST['licenseKey'] : NULL;
            if (!$license_key) {
                return;
            }

            // XXX: ensure admin email is set before saving license key?

            // determine whether magicspam is running in freemium mode prior to
            // registering the license key as we want to take different steps
            // when transistioning from freemium to licensed mode
            $previously_freemium = isFreemium();

            // XXX: should an empty license be indicator to reset to freemium?

            // register the license key
            // NOTE: use the global license manager to propo
            //       knows the registration result and can take the appropriate
            //       actions when rendering the interface
            $rv = $license->update($license_key);
            if ($rv === -1) {
                $errors[] = $license->error();
                $this->edit_mode = self::EDIT_MODE_LICENSE;
                return;
            }

            // double check that the license was registered correctly
            // NOTE: expired license warnings are handled in the main controller
            if (!$license->isValid() && !$license->isExpired()) {
                $errors[] = intl('empty_license');
                return;
            }

            // at this point, the license has been registered successfully
            $notices[] = intl('license_saved');

            // notify the user when the admin email has not been set
            if ($admin_email === '') {
                $notices[] = intl('ae_please_complete');
            }

            // disable and remove PSBL RBL which is only meant for freemium mode,
            // as we distribute the PSBL BMS List
            if ($previously_freemium) {
                $this->removePSBL();
            }

            // auto-enable magicspam when all conditions are met
            // NOTE: we use a successful license key registration and magicspam
            //       being disabled as a proxy for determining whether magicspam
            //       was freshly installed, which does not always hold true but
            //       works well enough for our purposes
            if ($status == 0) {
                // turn magicspam status to ON
                $register = Register::getInstance();
                if (!$register->toggle()) {
                    $errors[] = $register->error();
                    return;
                }

                // display the first installation popup message
                $this->first_installation = TRUE;

                // run the magicspam and spamassassin custom rules updater
                $this->_runUpdater();
                $this->handleSaCustomRules();
            }
        }

        // save the addon license key
        if ($action == 'saveAddon') {
            // don't bother with addon licenses in freemium
            if (isFreemium()) {
                return;
            }

            // retrieve the license key user input
            $license_key = isset($_POST['licenseKey']) ? $_POST['licenseKey'] : NULL;
            if (!$license_key) {
                return;
            }

            // initialize addon license manager
            $addon_license = new LicenseManager();

            // register the addon license key
            $rv = $addon_license->updateAddon($license_key);
            if ($rv === -1) {
                $errors[] = $addon_license->error();
                $this->edit_mode = self::EDIT_MODE_ADDON;
                return;
            }

            // double check that the addon license was registered correctly
            if (!$addon_license->isValid()) {
                if (!$addon_license->isExpired()) {
                    $errors[] = intl('empty_license');
                } else {
                    $errors[] = intl('expired_license');
                }
                return;
            }

            // at this point, the addon license has been registered successfully
            $notices[] = intl('license_saved');
        }

        // remove the license key
        // NOTE: removing the license key should only exist in freemium version
        if ($action === 'removeLicense') {
            // ensure that the product can run in freemium mode
            if (!defined('ISFREEMIUM')) {
                return;
            }

            // remove the registered license key
            if ($license->removeLicense() === -1) {
                $errors[] = $license->error();
                return;
            }

            // at this point, the license key has been removed successfully
            $notices[] = intl('license_removed');
        }
    }

    //! method to render the full page module.
    /*!
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function renderModule() {
        global $license, $module, $modules;

        // build the magicspam status information section (left-side)
        // retrieve the magicspam status and magicspam services status
        $status = (getControl("enabled", 0) == 1);
        $service_status = $modules['serviceStatus']->isRunning();

        // determine how to render the section based on the statuses
        if ($status && $service_status) {
            $ms_status = "enabled";
            $ms_status_icon = IMAGEPATH . "icon-success.png";
        } else {
            $ms_status = "disabled";
            $ms_status_icon = IMAGEPATH . "icon-warning.png";
        }
        if ($service_status) {
            $ms_status_blurb = intl('statusBlurb');
            $form_action = 'toggleStatus';
            $ms_toggle_disabled = '';
        } else {
            $ms_status_blurb = intl('serviceBlurb');
            $form_action = 'start_service';
            $ms_toggle_disabled = 'disabled="disabled"';
        }
        if ($status) {
            $ms_toggle_status = intl('disable');
        } else {
            $ms_toggle_status = intl('enable');
        }

        // build the license information section (right-side)
        // determine whether the addon licenses has been enabled on the system
        $addon_enabled = getControl('addon_enabled', 0);

        // load the license key
        $license_key = $license->getCode();

        // load the addon license key if necessary
        $addon_license_key = '';
        $addon_license_valid = '';
        if ($addon_enabled) {
            $addon_license = new LicenseManager(FALSE);
            $addon_license->loadAddon();
            $addon_license_key = $addon_license->getCode();
            $addon_license_valid = $addon_license->isValid();
        }

        // load the magicspam version
        $ms_version = MAGICSPAM_VERSION;

        // load the license expiry date
        if ($license->isExpired()) {
            // license is expired, go straight to license edit mode
            $expiry_date = intl('expired');
            $this->edit_mode = self::EDIT_MODE_LICENSE;
            $expiry_date_help = FALSE;
        } else if ($license->isNew()) {
            // no license has been registered, go straight to license edit mode
            $expiry_date = '';
            if (strlen($license_key) == 0) {
                $this->edit_mode = self::EDIT_MODE_LICENSE;

            // unless when running in freemium mode
            } else {
                $expiry_date = intl('never');
                $ms_version .= ' Free Edition';
            }
            $expiry_date_help = FALSE;
        } else {
            // license is valid (not expired, not new)
            $expiry_date = $license->getExpiry();
            $expiry_date_help = TRUE;
        }

        // determine how to render the section based on the mode
        if ($this->edit_mode == self::EDIT_MODE_LICENSE) {
            $license_key_blurb = intl('please_enter_key');
            $license_action = 'saveLicense';
            $license_requester = '';
            $edit_license_key = $license_key;
            // NOTE: license manager will set the license key to 'MSPLSK-FREE',
            //       when running in freemium mode which we do not want to show
            //       when in license edit mode
            if ($edit_license_key == 'MSPLSK-FREE') {
                $edit_license_key = '';
            }
        } else if ($this->edit_mode == self::EDIT_MODE_ADDON) {
            $license_key_blurb = intl('please_enter_addon');
            $license_action = 'saveAddon';
            $license_requester = '';
            $edit_license_key = $addon_license_key;
        } else {
            $license_key_blurb = '';
            // NOTE: the editAddonLicense action is set via JS
            $license_action = 'editLicense';
            $license_requester = $license->getRequester();
            $edit_license_key = '';
        }

        // add the JS i18n locales
        $this->_i18n_js[] = 'confirm_license_remove';
        $this->_i18n_js[] = 'protection_help';
        $this->_i18n_js[] = 'expiry_blurb';
        $this->_i18n_js[] = 'addon_descr';

        // render the protection key template
        renderTemplate('protectionKey', array(
            'module' => $module,
            'first_installation' => $this->first_installation,
            'edit_mode' => $this->edit_mode,

            'status' => $status,
            'service_status' => $service_status,
            'addon_enabled' => $addon_enabled,

            'form_action' => $form_action,
            'ms_status' => $ms_status,
            'ms_status_icon' => $ms_status_icon,
            'ms_status_blurb' => $ms_status_blurb,
            'ms_toggle_disabled' => $ms_toggle_disabled,
            'ms_toggle_status' => $ms_toggle_status,

            'license_key' => $license_key,
            'addon_license_key' => $addon_license_key,
            'addon_license_valid' => $addon_license_valid,
            'ms_version' => $ms_version,
            'expiry_date' => $expiry_date,
            'expiry_date_help' => $expiry_date_help,
            'license_key_blurb' => $license_key_blurb,
            'edit_license_key' => $edit_license_key,
            'license_action' => $license_action,
            'license_requester' => $license_requester,
            'license_type' => $license->getSKUType(),
        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getCSS() {
        return array_merge(array('protectionKey.css', 'splash.css'), $this->subModuleCSS());
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author         Shaun A. Johnson <shaun@linuxmagic.com>
     */
    public function getJS() {
        return array_merge(array('license-key.js', 'form-submit.js'), $this->subModuleJS());
    }

    //! run the updater.
    /*!
     * @return bool Whether successful
     *
     * @author William Storey <william@linuxmagic.com>
     */
    private function _runUpdater() {
        // get the current status
        $status = getControl("enabled", 0);
        // if the status is 1 then that means enabled
        if ($status == 1) {
            // and then we can run updates..but...
            // we run things different if it is a WINDOWS installation.
            if (defined('ISWINDOWS') && ISWINDOWS == 1) {
                pclose(popen("start /b " . UPDATER, "r"));

            // vs. a Linux installation!
            } else {
                exec(UPDATER . " >/dev/null 2>&1 &");
            }
        }
    }

    //! remove the PSBL RBL
    protected function removePSBL() {
        // initialize the default psbl hostname and description
        $psbl_hostname = 'psbl.surriel.com';
        $psbl_description = 'PSBL Passive Spam Block List (http://psbl.org)';

        // initialize the list manager
        $list_manager = ListManager::getInstance();

        // disable the PSBL RBL
        $args = array('-E', '-d', $psbl_hostname, '-h', DATA_PATH);
        if ($list_manager->delete($args) !== TRUE) {
            return;
        }

        // remove the PSBL RBL
        $args = array('-R', '-d', "$psbl_hostname $psbl_description", '-h', DATA_PATH);
        $ret = $list_manager->delete($args);
    }

    //! toggle the magicspam custom spamassassin rules
    protected function handleSaCustomRules() {
        // only professional suites with the exception of DirectAdmin have
        // access to the custom spamassassin rules
        if (!isProfessional() || !defined("SA_MANAGER")) {
            return;
        }

        // load the spamassassin manager utility
        require_once("lib.spamassassinmanagerutility.php");

        // nothing to do when spamassassin is not installed
        if (!file_exists(SA_BIN)) {
            return;
        }

        // retrieve the magicspam and custom spamassassin rules status
        $ms_status = getControl("enabled", 0);
        $sa_status = getControl("sa_status", 0);

        // determine whether to enable or disable custom spamassassin rules
        $flag = ($ms_status && $sa_status) ? "e" : "d";

        // call the spamassassin manager utility
        $buffer = "";
        $sa_manager = SpamAssassinManagerUtility::getInstance();
        $sa_manager->callSpamAssassinManagerUtility($flag, $buffer);
    }
}
