<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2017-2018 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Deion Yu <deiony@linuxmagic.com>
 *
 * $Id:
 */
require_once('functions.php');
require_once("lib.module.php");
require_once("lib.spamassassinmanagerutility.php");

class spamAssassinSettingsModule extends Module {

    protected $status = null;
    protected $displayStatus = TRUE;

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     *
     * NOTE: This is not an AJAX request, the page will render again, so no die()
     */
    public function actionProcessor($action, &$errors, &$notices) {
        $accepted_action = array("save", "restore");

        // sanity check
        if (empty($_POST) || empty($action) || !in_array($action, $accepted_action, TRUE)) {
            $errors[] = intl('errorInvalidAction');
            return;
        }

        // set default values for restore
        $selected_status = TRUE;
        $cmd = "e";

        // determine whether to actually
        if ($action === "save" && !empty($_POST['sa-custom-rule'])) {
            if ($_POST['sa-custom-rule'] === "false") {
                setControl("sa_status", 0);
                $selected_status = FALSE;
                $cmd = "d";
            } elseif ($_POST['sa-custom-rule'] === "true") {
                setControl("sa_status", 1);
            } else {
                $errors[] = intl('errorUnknownAction');
                return;
            }
        } elseif ($action === "restore") {
            setControl("sa_status", 1);
        }

        // retrieve the status
        $this->status = $this->_getStatus();
        if (is_null($this->status)) {
            $errors[] = intl('errorRetrieveStatus');
            return;
        }

        // nothing to be done if already set to user's selection
        if ($this->status === $selected_status) {
            $notices[] = $selected_status ? intl('noticeAlreadyEnable') : intl('noticeAlreadyDisable');
            return;
        }

        // run the SpamAssassin Custom Rules Manager
        $buffer = '';
        $saManagerCmd = SpamAssassinManagerUtility::getInstance();
        if ($saManagerCmd->callSpamAssassinManagerUtility($cmd, $buffer) === FALSE) {
            $errors[] = $selected_status ? intl('errorFailedEnable') : intl('errorFailedDisable');
            return;
        }

        // keep track of the new status
        $notices[] = $selected_status ? intl('noticeSuccessEnable') : intl('noticeSuccessDisable');
        $this->status = $selected_status;
    }

    //! Method to render the full page for SpamAssassin Settings
    /*!
     * @brief   Assembles our SpamAssassin Settings page
     */
    public function renderModule() {
        global $module;

        // Pass i18n labels to JS for SA stats graph
        $this->_i18n_js[] = 'customRuleHelp';

        // grab the status of magicspam
        $msStatus = getControl("enabled", 0);

        // check whether spamassassin in installed
        if (!file_exists(SA_BIN)) {
            $error = intl('errorSANotInstalled');
        // check whether magicspam is enabled
        } else if ($msStatus != 1) {
            $error = intl("errorMagicSpamDisabled");
        // no point retrieving the status twice if we got here from the action processor
        } else if (is_null($this->status)) {
            $this->status = $this->_getStatus();
            if (is_null($this->status)) {
                $error = intl('errorRetrieveStatus');
            } elseif ($this->status === "unsupported") {
                $error = intl('errorSAVersionUnsupported');
            }
        }

        // renders the template
        renderTemplate('SAsettings', array(
                                         'module' => $module,
                                         'submodule' => 'spamAssassinSettings',
                                         'status' => $this->status,
                                         'error' => isset($error) ? $error : NULL,
                                        ));
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array     an array of the files to include.
     */
    public function getCSS() {
        return array("SA-settings.css");
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array     an array of the files to include.
     */
    public function getJS() {
        return array();
    }

    //! retrieves the status of the SpamAssassin Custom Rules
    /*!
     * @returns boolean   null on error, TRUE on enabled, FALSE on disabled
     */
    protected function _getStatus() {

        $valid_status = array( "enabled", "disabled", "unsupported" );

        // call the SA manager utility and retrieve the JSON encoded status buffer
        $buffer = '';
        $saManagerCmd = SpamAssassinManagerUtility::getInstance();
        if ($saManagerCmd->callSpamAssassinManagerUtility("s", $buffer) === FALSE) {
            return null;
        }

        // decode the JSON encoded status buffer
        $decode = json_decode($buffer, TRUE);
        if (empty($decode) || !isset($decode['status']) || !in_array($decode['status'], $valid_status, TRUE)) {
            return null;
        }

        if ($decode['status'] === "unsupported") {
            return $decode['status'];
        }

        // return the decoded status
        return $decode['status'] === "enabled" ? TRUE : FALSE;
    }

}
