<?php
/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 *
 * This file is part of MagicSpam (TM) Project and is
 * Copyright (c) LinuxMagic Inc. 2014-2020 All Rights Reserved
 *
 * This file contains Original Code as created by LinuxMagic Inc.
 *
 * The Original Code is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND LINUXMAGIC
 * HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET
 * ENJOYMENT OR NON-INFRINGEMENT.
 *
 * Do NOT download, distribute, use or alter this software or file in any
 * way without express written permission from LinuxMagic Inc. or its parent
 * company Wizard Tower TechnoServices signed by an authorized company officer.
 *
 * Author(s): Robert Bisewski <robertb@linuxmagic.com>
 *
 * $Id: systemOptionsModule.php 11866 2020-07-08 00:08:11Z deiony $
 *
 * Purpose of this module is to be a combined form for all of the system
 * options that MagicSpam end-users can control.
 */

require_once('functions.php');
require_once('lib.module.php');
require_once('lib.datatable.php');

class systemOptionsModule extends Module {

    const SERVER_IP_ON_BMS_REPORT_MODE_DEFAULT = "3"; 
    const PHP_TZ_DEFAULT = "Use php.ini default";
    const MS_LOCALE_DEFAULT = "en-US";

    //! class constructor
    /*!
     * @param   string  the 'label' that is used for links, and locale.
     * @author    Sean West <sean@linuxmagic.com>
     *            Peter Chung <peterc@linuxmagic.com>
     */
    public function __construct($l) {
        parent::__construct($l, FALSE);
        // set module to be migratable
        $this->migratable = TRUE;
    }

    //! method to process any actions and react accordingly.
    /*!
     * @param   string  the action being requested.
     * @param   array   the array where error messages can be stacked.
     * @param   array   the array where any 'friendly' notices can be stacked.
     * @author    Sean West <sean@linuxmagic.com>
     *            Peter Chung <peterc@linuxmagic.com>
     */
    public function actionProcessor($action, &$errors, &$notices) {
        // set up globals for what is expected to be loaded by the calling process.
        global $admin_email, $module;
        
        // we only react to one action type
        // XXX: should we consider changing action name to something less generic
        // (eg: can't be confused with other modules...)
        if (is_string($action) && $action == 'save') {
            // Firstly, attempts to save the email address
            if (  !is_string($_POST['email_address']) || strlen(trim($_POST['email_address'])) == 0
                || !filter_var(trim($_POST['email_address']),FILTER_VALIDATE_EMAIL)
                || !isValidEmail(trim($_POST['email_address']))) {
                $errors[] = intl('invalid_email');
                return;
            } else {
                // try to set the control.
                $rv = setControl('admin_email', $_POST['email_address']);
                if ($rv === FALSE) {    // unknown error condition
                    $errors[] = intl('invalid_email');
                    return;
                } else if (is_string($rv)) {    // known error condition
                    $errors[] = $rv;
                    return;
                } else {    // that just leaves TRUE: success!
                    $admin_email = $_POST['email_address'];
                }
            }

            // Secondly, attempts to save the number of days to log
            $logdays = "";
            if (!isset($_POST['logdays'])) {
                $errors[] = intl('ld_isBlank');
                return;
            }
            if (is_string($_POST['logdays'])) {
                $logdays = trim($_POST['logdays']);
            }

            if (strlen($logdays) === 0) {
                $errors[] = intl('ld_isBlank');
                return;
            }
            if (!is_numeric($logdays)) {
                $errors[] = intl('ld_isNaN');
                return;
            }
            // number of days must be between 1 and 60
            if (intval($logdays) < 1) {
                $errors[] = intl('ld_isLessThanOne');
                return;
            } else if (intval($logdays) > 60) {
                $errors[] = intl('ld_isMoreThan60');
                return;
            }
            $daysInt = (string) intval($logdays);
            if (setControl('logdays', $daysInt) !== TRUE) {
                $errors[] = intl('ld_saveFailed');
                return;
            }

            // save the server ip on bms report mode setting
            $server_ip_on_bms_report_mode_valid = array("0", "1", "2", "3");
            if (!isset($_POST['server_ip_on_bms_report_mode'])) {
                $errors[] = intl('server_ip_on_bms_report_mode_missing_fail');
                return;
            } else if (in_array($_POST['server_ip_on_bms_report_mode'], $server_ip_on_bms_report_mode_valid)) {
                if (setControl('server_ip_on_bms_report_mode', $_POST['server_ip_on_bms_report_mode']) !== TRUE) {
                    $errors[] = intl('server_ip_on_bms_report_mode_save_fail');
                    return;
                }
            } else {
                $errors[] = intl('server_ip_on_bms_report_mode_unknown_fail');
                return;
            }

            // save the php timezone setting
            if (!isset($_POST['php_tz'])) {
                $errors[] = intl('php_tz_missing_fail');
                return;
            } else if ($_POST['php_tz'] === self::PHP_TZ_DEFAULT) {
                if (clearControl('php_tz') !== TRUE) {
                    $errors[] = intl('php_tz_save_fail');
                    return;
                }
            } else if (in_array($_POST['php_tz'], timezone_identifiers_list())) {
                if (setControl('php_tz', $_POST['php_tz']) !== TRUE) {
                    $errors[] = intl('php_tz_save_fail');
                    return;
                }
            } else {
                $errors[] = intl('php_tz_unknown_fail');
                return;
            }

            // save the ms locale setting
            if (!isset($_POST['ms_locale'])) {
                $errors[] = intl('ms_locale_missing_fail');
                return;
            } else if (in_array($_POST['ms_locale'], array_values($this->getMSLocaleList()))) {
                if (getControl('ms_locale', NULL) !== $_POST['ms_locale']) {
                    if (setControl('ms_locale', $_POST['ms_locale']) !== TRUE) {
                        $errors[] = intl('ms_locale_save_fail');
                        return;
                    }

                    // hack to reload the locales for the current page
                    global $locale, $pages, $ms_words;
                    if ($locale != $_POST['ms_locale']) {
                        $locale = $_POST['ms_locale'];
                        require(loadMSLocale());
                        require(loadMSLocale("logSearch"));
                        foreach ($pages as $page) {
                            $path = loadMSLocale($page);
                            if (file_exists($path)) {
                                require($path);
                            }
                        }
                        new systemModule("");
                    }
                }
            } else {
                $errors[] = intl('ms_locale_unknown_fail');
                return;
            }

            // save the inbound/outbound rate limiter settings
            if (isset($_POST["rl_count_limit"], $_POST["rl_flag_timeout"], $_POST["rl_block_timeout"])
                && isset($_POST["ra_count_limit"], $_POST["ra_flag_timeout"], $_POST["ra_block_timeout"])) {

                // ensure the rate limiter numeric settings are valid
                if (!isValidNum($_POST["rl_count_limit"])
                    || !isValidNum($_POST["rl_flag_timeout"])
                    || !isValidNum($_POST["rl_block_timeout"])
                    || !isValidNum($_POST["ra_count_limit"])
                    || !isValidNum($_POST["ra_flag_timeout"])
                    || !isValidNum($_POST["ra_block_timeout"])) {
                    $errors[] = intl('rl_NaN');
                    return;
                }

                // convert the rate limiter settings into numeric strings
                // NOTE: the counting period and expiry time are stored as seconds internally
                $inbound_max  = (string) intval(trim($_POST["rl_count_limit"]));
                $inbound_ft  = (string) (intval(trim($_POST["rl_flag_timeout"])) * 60);
                $inbound_bt  = (string) (intval(trim($_POST["rl_block_timeout"])) * 60);
                $outbound_max = (string) intval(trim($_POST["ra_count_limit"]));
                $outbound_ft = (string) (intval(trim($_POST["ra_flag_timeout"])) * 60);
                $outbound_bt = (string) (intval(trim($_POST["ra_block_timeout"])) * 60);

                // convert the outbound rate limiter block notification setting
                $outbound_bn = isset($_POST["ra_block_notify"]) ? 1 : 0;

                // save the rate limiter settings
                if (setControl('rl_count_limit', $inbound_max) !== TRUE
                    || setControl('rl_flag_timeout', $inbound_ft) !== TRUE
                    || setControl('rl_block_timeout', $inbound_bt) !== TRUE
                    || setControl('ra_count_limit', $outbound_max) !== TRUE
                    || setControl('ra_flag_timeout', $outbound_ft) !== TRUE
                    || setControl('ra_block_timeout', $outbound_bt) !== TRUE
                    || setControl('ra_block_notify', $outbound_bn) !== TRUE) {
                    $errors[] = intl('rl_saveFailed');
                    return;
                }
            }

            // attempt to save the quarantine options
            $rewrite_local = isset($_POST['rewrite_local']) ? 1 : 0;
            $rewrite_remote = isset($_POST['rewrite_remote']) ? 1 : 0;
            $quarantine_local = isset($_POST['deliver_quarantine']) ? 1 : 0;
            $rewrite_subject = isset($_POST['rewrite_subject']) ? trim($_POST['rewrite_subject']) : "";
            if (setControl('flag_local', $rewrite_local) !== TRUE
                || setControl('flag_remote', $rewrite_remote) !== TRUE
                || setControl('quarantine_local', $quarantine_local) !== TRUE
                || setControl('qsubject', $rewrite_subject) !== TRUE) {
                $errors[] = intl('q_saveFailed');
                return;
            }

            // If all options have been saved successfully, prints out a success message
            $notices[] = $this->notice = intl('options') . " " . intl('saved');
        }
    }

    //! method to render the full page module.
    /*!
     * @return void
     *
     * @author Sean West <sean@linuxmagic.com>
     * @author Peter Chung <peterc@linuxmagic.com>
     *
     * SIDE EFFECT: outputs to stdout.
     */
    public function renderModule() {
        global $module;

        // Grabs the inbound max / outbound max / admin email
        // The inbound and outbound count & expiry times are divided by 60
        // since they are stored as seconds but displayed as minutes.
        $inbound_max   = getControl("rl_count_limit");
        $inbound_ft    = intval(getControl("rl_flag_timeout") / 60);
        $inbound_bt    = intval(getControl("rl_block_timeout") / 60);
        $outbound_max  = getControl("ra_count_limit");
        $outbound_ft   = intval(getControl("ra_flag_timeout") / 60);
        $outbound_bt   = intval(getControl("ra_block_timeout") / 60);
        $outbound_bn  = getControl("ra_block_notify");
        $email_address = getControl("admin_email");

        // grab the quarantine settings
        $rewrite_local = getControl('flag_local', '0');
        $rewrite_remote = getControl('flag_remote', '0');
        $rewrite_subject = getControl('qsubject', '*****SPAM*****');
        $deliver_quarantine = getControl('quarantine_local', '0');

        $rewrite_local = ($rewrite_local == "1") ? "checked=\"checked\"" : "";
        $rewrite_remote = ($rewrite_remote == "1") ? "checked=\"checked\"" : "";
        $deliver_quarantine = ($deliver_quarantine == "1") ? "checked=\"checked\"" : "";

        // Grabs if the advanced options
        $isChecked = "";
        if (isset($_POST["advanced_options"])) {
            $isChecked = "checked=\"checked\"";
        }

        // Retrieves the number of days to store logs, default is 7 days
        $log_days = getControl("logdays");
        if (!isset($log_days)) {
            $log_days = 7;
        }

        $server_ip_on_bms_report_mode_control = getControl("server_ip_on_bms_report_mode", self::SERVER_IP_ON_BMS_REPORT_MODE_DEFAULT);
        $php_tz_control = getControl("php_tz", self::PHP_TZ_DEFAULT);
        $ms_locale_control = getControl("ms_locale", self::MS_LOCALE_DEFAULT);

        $this->_i18n_js[] = 'so_help_desc';
        $this->_i18n_js[] = 'ld_help_max_inbound_msg';
        $this->_i18n_js[] = 'ld_help_max_outbound_msg';

        // Prints out the module
        renderTemplate( 'systemOptionsModule',
                        array (
                             'module'                  =>  $module,
                             'ae_label'                =>  intl('ae_label'),
                             'ld_label'                =>  intl('ld_label'),
                             'ae_txt'                  =>  intl('ae_txt'),
                             'ld_txt'                  =>  intl('ld_txt'),
                             'options'                 =>  intl('options'),
                             'help'                    =>  intl('help'),
                             'rl_flag_timeout'         =>  intl('rl_flag_timeout'),
                             'rl_inbound_max'          =>  intl('rl_inbound_max'),
                             'rl_outbound_max'         =>  intl('rl_outbound_max'),
                             'rl_IP_bt'                =>  intl('rl_IP_bt'),
                             'rl_user_bt'              =>  intl('rl_user_bt'),
                             'rl_inbound_ml'           =>  intl('rl_inbound_ml'),
                             'rl_inbound_cp'           =>  intl('rl_inbound_cp'),
                             'rl_inbound_et'           =>  intl('rl_inbound_et'),
                             'rl_outbound_ml'          =>  intl('rl_outbound_ml'),
                             'rl_outbound_cp'          =>  intl('rl_outbound_cp'),
                             'rl_outbound_et'          =>  intl('rl_outbound_et'),
                             'q_rewrite_local'         =>  intl('q_rewrite_local'),
                             'q_rewrite_local_blurb'   =>  intl('q_rewrite_local_blurb'),
                             'q_rewrite_remote'        =>  intl('q_rewrite_remote'),
                             'q_rewrite_remote_blurb'  =>  intl('q_rewrite_remote_blurb'),
                             'q_rewrite_subject'       =>  intl('q_rewrite_subject'),
                             'q_rewrite_subject_blurb' =>  intl('q_rewrite_subject_blurb'),
                             'q_deliver_quarantine'    =>  intl('q_deliver_quarantine'),
                             'q_deliver_quarantine_blurb' =>  intl('q_deliver_quarantine_blurb'),
                             'rewrite_subject'         =>  $rewrite_subject,
                             'rewrite_remote_checked'  =>  $rewrite_remote,
                             'rewrite_local_checked'   =>  $rewrite_local,
                             'deliver_quarantine_checked' => $deliver_quarantine,
                             'advanced_options'        =>  intl('advanced_options'),
                             'so_help_max_inbound_msg' =>  'ld_help_max_inbound_msg',
                             'so_help_max_outbound_msg'=>  'ld_help_max_outbound_msg',
                             'inbound_max'             =>  $inbound_max,
                             'inbound_ft'              =>  $inbound_ft,
                             'inbound_bt'              =>  $inbound_bt,
                             'outbound_max'            =>  $outbound_max,
                             'outbound_ft'             =>  $outbound_ft,
                             'outbound_bt'             =>  $outbound_bt,
                             'outbound_bn'             =>  $outbound_bn,
                             'admin_email'             =>  $email_address,
                             'logs_days'               =>  $log_days,
                             'save_key'                =>  intl('save'),
                             'isChecked'               =>  $isChecked,
                             'notice'                  =>  $this->notice,
                             'server_ip_on_bms_report_mode_list'    =>  $this->getServerIPonBMSReportModeList(),
                             'server_ip_on_bms_report_mode_control' =>  $server_ip_on_bms_report_mode_control,
                             'php_tz_list'             =>  $this->getPHPTimezoneList(),
                             'php_tz_control'          =>  $php_tz_control,
                             'ms_locale_list'          =>  $this->getMSLocaleList(),
                             'ms_locale_control'       =>  $ms_locale_control,
                         )
                      );
    }

    //! method to retrieve any 'optional' CSS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author    Sean West <sean@linuxmagic.com>
     *            Peter Chung <peterc@linuxmagic.com>
     */
    public function getCSS() {
        return array('system-options.css');
    }

    //! method to retrieve any 'optional' JS files to include in the page rendering.
    /*!
     * @returns array   an array of the files to include.
     * @author    Sean West <sean@linuxmagic.com>
     *            Peter Chung <peterc@linuxmagic.com>
     */
    public function getJS() {
        return array('system.js', 'limit-integer.js');
    }

    //! retrieve the list of server IP on BMS report modes
    /*!
     * @returns array   the list of server IP on BMS report modes
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    protected function getServerIPonBMSReportModeList() {
        return array(
            intl('server_ip_on_bms_report_mode_never'),
            intl('server_ip_on_bms_report_mode_hit_only'),
            intl('server_ip_on_bms_report_mode_daily'),
            intl('server_ip_on_bms_report_mode_daily_hit_only'),
        );
    }

    //! retrieve the list of PHP timezones
    /*!
     * @returns array   the list of PHP timezones
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    protected function getPHPTimezoneList() {
        # retrieve the list of all PHP timezones
        $php_tz_list = timezone_identifiers_list();
        if ($php_tz_list === FALSE) {
            return array(self::PHP_TZ_DEFAULT);
        }

        # prepend the PHP timezone default
        array_unshift($php_tz_list, self::PHP_TZ_DEFAULT);

        # return the list of PHP timezones
        return $php_tz_list;
    }

    //! retrieve the list of MS locales
    /*!
     * @returns array   the list of MS locales
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    protected function getMSLocaleList() {
        return array(
            "Dutch" => "nl-NL",
            "English" => "en-US",
            "German" => "de-DE",
            "Portuguese" => "pt-BR",
            "Spanish" => "es-ES",
        );
    }

    //! retrieve the migratable configurations of the module
    /*!
     * @returns array  the migratable configurations
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function getMigrationData() {
        // the list of migratable configurations
        $configurations = array();

        // the list of options
        $options = array(
            "rl_count_limit",
            "rl_flag_timeout",
            "rl_block_timeout",
            "ra_count_limit",
            "ra_flag_timeout",
            "ra_block_timeout",
            "ra_block_notify",
            "admin_email",
            "logdays",
            'flag_local',
            'flag_remote',
            'qsubject',
            'quarantine_local',
        );

        // retrieve and store the value of each option
        foreach ($options as $option) {
            $configurations[$option] = getControl($option, 0);
        }

        // return the migration configurations
        return $configurations;
    }

    //! loads the migratable configurations of the module
    /*!
     * @param   array  the configurations to load
     *
     * @returns bool   whether the configurations were loaded successfully
     *
     * @author  Deion Yu <deiony@linuxmagic.com>
     */
    public function loadMigrationData($configurations) {
        // sanity check
        if (!isset($configurations) || !WV::isArrayNotEmpty($configurations)) {
            return FALSE;
        }

        // load each option with the value
        foreach ($configurations as $option => $value) {
            setControl($option, $value);
        }

        // success, migratable configurations successfully loaded
        return TRUE;
    }
}
